/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file: callbacks.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "browse.h"
#include "form_card.h"
#include "main.h"
#include "callbacks.h"
#include "prefer.h"
#include "prefer_int.h"
#include "search_int.h"
#include "search.h"
#include "sort.h"
#include "interface.h"
#include "dialog.h"
#include "file.h"
#include "import_int.h"
#include "import.h"
#include "export.h"
#include "preview.h"
#include "autosave.h"
#include "form_cb.h"
#include "view.h"
#include "data_view.h"
#include "card.h"
#include "rubrica_type.h"
#include "rubrica.h"
#include "card_item.h"


void 
on_populate_popup(GtkLabel *label, GtkMenu *arg1, gpointer user_data)
{
  g_return_if_fail(arg1 != NULL);

  gtk_menu_popup (GTK_MENU(arg1), NULL, NULL, NULL, NULL, 0, 0);
}


void
on_new_file_activate(GtkMenuItem *menuitem, gpointer user_data)
{ 
  RubricaViewType view;

  view = rubrica_data_view_get_view_type();

  switch(view)
    {
    case CARD_VIEW: 
      rubrica_card_add_page_view(NULL);
      break;
      
    default:
      break;
    }
}


void	
on_open_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
  rubrica_file_load_file();
}


void
on_save_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();

  if (!doc)
    {
//    TODO: rubrica_error();

      return;
    }

  rubrica_file_save(doc);
}


void
on_save_as_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();
  
  if (!doc)
    {
//    TODO: rubrica_error();

      return;
    }

  rubrica_file_save_as(doc);
}

void	
on_close_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  if (user_data)
    doc = (RubricaDoc *) user_data;
  else
    doc = rubrica_folder_get_current_document();

  rubrica_doc_close(doc);
}


void 
on_import_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
  GtkWidget *app;
  GtkWidget *import = NULL;
  gboolean imported = FALSE; 
  gint result;
  
  app = rubrica_get_app();
  import = create_import_dialog(app);
  gtk_widget_show(import);
    
  while (!imported)
    {
      result = gtk_dialog_run(GTK_DIALOG(import));
      
      switch (result)
	{
	case GTK_RESPONSE_ACCEPT:
	  imported = rubrica_import_data(app);
	  break;

	case GTK_RESPONSE_CLOSE:
	default:
	  imported = TRUE;
	  break;
	}  
    }
  
  gtk_widget_destroy(import);
}

void 
on_export_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  GtkWidget *app;
  GtkWidget *export = NULL;
  gboolean exported = FALSE; 
  gint result;
  
  app = rubrica_get_app();
  export = create_export_dialog(app);
  gtk_widget_show(export);
    
  while (!exported)
    {
      result = gtk_dialog_run(GTK_DIALOG(export));
      
      switch (result)
	{
	case GTK_RESPONSE_ACCEPT:
	  exported = rubrica_export_export_data();
	  break;

	case GTK_RESPONSE_CLOSE:
	default:
	  exported = TRUE;
	  break;
	}  
    }
  
  gtk_widget_destroy(export);
}

void 
on_preview_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  GtkWidget *RubricaPreviewFs;
 
  RubricaPreviewFs = rubrica_preview_create_preview();
  gtk_widget_show (RubricaPreviewFs);  
}

void
on_exit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();  
  exit_rubrica(doc);
}


void 
on_undo_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  /* not yet implemented */
  rubrica_dialog_will_do();
}

void
on_redo_activate (GtkMenuItem *menuitem,  gpointer user_data)
{ 
  /* not yet implemented */
  rubrica_dialog_will_do();
}


void
on_cut_activate (GtkMenuItem *menuitem, gpointer user_data)
{   
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();  
  rubrica_doc_cut_item(doc);  
}


void
on_copy_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();  
  rubrica_doc_copy_item(doc);
}


void
on_paste_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();
  rubrica_doc_paste_item(doc);
}


/*   POPUP CALLBACKS
*/
void
on_pop_cut_activate (GtkMenuItem *menuitem, gpointer user_data)
{ 
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();
  rubrica_doc_cut_item(doc);  
}

void
on_pop_copy_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();
  rubrica_doc_copy_item(doc);
}

void
on_pop_paste_activate (GtkMenuItem *menuitem, gpointer user_data)
{ 
  RubricaDoc *doc;
  
  doc = rubrica_folder_get_current_document();
  rubrica_doc_paste_item(doc);
}

void 
on_pop_add_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  on_bt_add_cb (NULL, NULL);
}

void 
on_pop_remove_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  on_bt_remove_cb (NULL, NULL);
}

void
on_pop_modify_activate (GtkMenuItem *menuitem,  gpointer user_data)

{
  on_bt_modify_cb (NULL, NULL);
}


void 
on_pop_check_item_clicked(GtkMenuItem *menuitem, gpointer user_data)
{
  GConfClient *client;
  gchar *key = user_data;
  gboolean bool;

  client = gconf_client_get_default();
  bool = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem));
  gconf_client_set_bool(client, key, bool, NULL);
}

void 
on_pop_radio_item_clicked(GtkMenuItem *menuitem, gpointer user_data)
{
  GConfClient *client;
  gchar *value = user_data;

  client = gconf_client_get_default();
  gconf_client_set_string(client, RUBRICA_KEY_CRYPTOGRAPHY, value, NULL);
}


/*****************************************************************************
 */


/*  SORT TYPE CALLBACKS
*/
void
on_alphabetic_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_sort_set_sort_type(SORT_ALPHA);
}

void 
on_creation_date_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_sort_set_sort_type(SORT_CREATION_DATE);
}

void 
on_modification_date_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_sort_set_sort_type(SORT_MODIFICATION_DATE);
}


void
on_increasing_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_sort_set_sort_direction(SORT_ASCENDING);
}


void
on_decreasing_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_sort_set_sort_direction(SORT_DESCENDING);
}
/*****************************************************************************
 */


void
on_preferences_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
  gint result;
  GtkWidget *app      = NULL;
  GtkWidget *prefer   = NULL;
  GConfClient *client = NULL;
  
  app    = rubrica_get_app();
  client = gconf_client_get_default(); 
  prefer = rubrica_preferences_get_preferencesbox();
  
  if (!prefer) 
    {
      prefer = create_preferences_dialog(app, client);
    }
  else
    {
      GtkTreePath *path;
      GtkTreeView *treeview;
      GtkTreeSelection *selection;
      GtkWidget *notebook;

      notebook  = g_object_get_data(G_OBJECT(prefer), "notebook");   
      treeview  = g_object_get_data(G_OBJECT(prefer), "treeview");
      selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
      path      = gtk_tree_path_new_from_string("0");

      gtk_tree_selection_select_path(selection, path);
      gtk_tree_path_free(path);

      if (GTK_IS_NOTEBOOK(notebook))	 
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0); 
    } 
  gtk_widget_show(prefer);    

  do
    {
      result = gtk_dialog_run(GTK_DIALOG(prefer));
    }
  while(result != GTK_RESPONSE_CLOSE); 
  
  gtk_widget_hide(prefer);
}


void
on_bt_new_cb (GtkButton *button, gpointer user_data)
{ 
  RubricaViewType view;

  view = rubrica_data_view_get_view_type();

  switch(view)
    {
    case CARD_VIEW: 
      rubrica_card_add_page_view(NULL);
      break;
/*      
    case ACCOUNT_VIEW:
      rubrica_account_add_page_view(NULL);
      break;
*/
    default:
      break;
    }
}


void
on_bt_open_cb (GtkButton *button, gpointer user_data)
{
  rubrica_file_load_file();
}


void
on_bt_save_cb (GtkButton *button, gpointer user_data)
{
  RubricaDoc *doc;
  gchar *filename;

  doc = rubrica_folder_get_current_document();
  filename = rubrica_doc_get_name(doc);
  if (filename && (g_ascii_strcasecmp(filename, _("no name")) == 0))
    rubrica_file_save_as(doc);
  
  rubrica_file_save(doc);
}

void
on_bt_save_as_cb (GtkButton *button, gpointer user_data)
{
  RubricaDoc *doc;
  
  doc = rubrica_folder_get_current_document();
  rubrica_file_save_as(doc);
}

void
on_bt_close_cb  (GtkButton *button, gpointer user_data)
{
  RubricaDoc *doc;

  if (user_data)
    doc = (RubricaDoc *) user_data;
  else
    doc = rubrica_folder_get_current_document();
  
  rubrica_doc_close(doc);
}


void
on_bt_add_cb (GtkButton *button, gpointer data)
{
  RubricaFormCard* form = NULL;
  RubricaViewType view_type;
  gint response;

  view_type = rubrica_data_view_get_view_type();

  switch(view_type)
    {
    case CARD_VIEW: 
      if ((form = rubrica_form_card_get_form()) == NULL)
	{
	  if ((form = rubrica_form_card_create_form()) == NULL)
	    return;	 
	}
      else
	{
	  rubrica_form_card_clean_form();
//	  rubrica_form_card_set_form_type(NEW_CARD);
//	  gtk_widget_show(form->dialog);
	}

      response = gtk_dialog_run(GTK_DIALOG(form->dialog));  
      switch(response)
	{
	case GTK_RESPONSE_OK:
	  rubrica_form_card_add_item();
	  break;
	  
	case GTK_RESPONSE_CLOSE:
	  rubrica_form_card_close_form();
	  break;
	  
	default:
	  break;
	}
      break;
      
    case ACCOUNT_VIEW:
//      form = rubrica_form_account_new();
      break;

    default:
      break;
    }
}


void
on_bt_remove_cb (GtkButton *button, gpointer data)
{
  RubricaViewType view;
  RubricaDoc *doc;

  doc  = rubrica_folder_get_current_document();
  view = rubrica_data_view_get_view_type();

  switch(view)
    {
    case CARD_VIEW:   
      rubrica_doc_delete_selected_item(doc);
      break;

    case ACCOUNT_VIEW:
      rubrica_dialog_will_do();
      break;

    default:
      break;
    }      
}

/*
void rubrica_modify_card_item(RubricaCardItem *item, gint index)
{
  RubricaFormCard* form = NULL;
  gint response;

  if ((form = rubrica_form_card_get_form()) == NULL)
    form = rubrica_form_card_create_form();  

  rubrica_form_card_clean_form();
  rubrica_form_card_set_form_type(OLD_CARD);
  rubrica_form_card_insert_item(item, index);

  response = gtk_dialog_run(GTK_DIALOG(form->dialog));  
  switch(response)
    {
    case GTK_RESPONSE_OK:
      rubrica_form_card_add_item();
     break;

    case GTK_RESPONSE_CLOSE:
      rubrica_form_card_close_form();
      break;

    default:
      break;
    }
}


void rubrica_modify_account_item(RubricaAccountItem *item, gint index)
{
  GtkWidget *form = NULL;
  gint response;

  g_return_if_fail(item != NULL);

  form = rubrica_form_account_new();  
  rubrica_form_account_insert_item(RUBRICA_ACCOUNT(form), item);

  response = gtk_dialog_run(GTK_DIALOG(form));  
  switch(response)
    {
    case GTK_RESPONSE_OK:
      rubrica_form_account_extract_change_close(RUBRICA_ACCOUNT(form), index);
      break;

    case GTK_RESPONSE_CLOSE:
      rubrica_form_account_close (RUBRICA_ACCOUNT(form));
      break;

    default:
      break; 
    }
}
*/
/*  
    Modifica il contenuto di una scheda.
    Funzione richiamata da:
    pressione del bottone modifica sulla toolbar
    doppio click su un elemento della lista
    menu popup - voce modifica
*/
void
on_bt_modify_cb (GtkButton *button, gpointer user_data)
{
  RubricaDoc* doc;
  gpointer item;
  gulong id;
  RubricaViewType type;

  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreePath *path;
  GtkTreeSelection *select;
  gint index;

  type = rubrica_data_view_get_view_type();
  doc = rubrica_folder_get_current_document();
  
  switch(type)
    {
    case CARD_VIEW:   
      if (rubrica_doc_is_empty(doc)) 
	{ 
	  rubrica_dialog_message(_("Address book is empty. "
				   "I can't continue\n"),
				 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 
	  return; 
	} 

      view   = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
      model  = gtk_tree_view_get_model (view);
      select = gtk_tree_view_get_selection(view);
      
      if (gtk_tree_selection_get_selected(select, &model, &iter))
	{
	  path = gtk_tree_model_get_path(model, &iter);
	  index = gtk_tree_path_get_indices (path)[0];
	  
	  gtk_tree_model_get(model, &iter, ID_COLUMN, &id, -1);

	  rubrica_doc_set_index(doc, index);
	  item = rubrica_doc_get_item(doc, id);
	  rubrica_doc_modify_item(doc, item);
	}
      else
	rubrica_dialog_message(_("I can't continue if you don't "
				 "select a card"), 
			       GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 
      break;
      
    case ACCOUNT_VIEW:
      rubrica_dialog_will_do();
      break;
      
    default:
      break;
    }
}
  


void
on_bt_browse_cb (GtkButton *button, gpointer data)
{
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *select;
  RubricaDoc *doc;
  RubricaBrowseType browse;
  
  browse = (RubricaBrowseType) data;
  doc    = rubrica_folder_get_current_document();
  view   = GTK_TREE_VIEW(rubrica_doc_get_view(doc));
  model  = gtk_tree_view_get_model(view);
  select = gtk_tree_view_get_selection(view);

  switch (browse)
    {
    case RB_FIRST_CARD:
      rubrica_browse_first_card(select, GTK_TREE_MODEL(model));
      break;

    case RB_LAST_CARD:
      rubrica_browse_last_card(select, GTK_TREE_MODEL(model));
      break;

    case RB_NEXT_CARD:
      rubrica_browse_next_card(select, GTK_TREE_MODEL(model));
     break;
      
    case RB_PREV_CARD:
      rubrica_browse_prev_card(select, GTK_TREE_MODEL(model));
      break;

    case RB_NTH_CARD:
      break;

    default:
      break;
    }
}


void
on_bt_find_cb (GtkButton *button, gpointer user_data)
{
  GtkWidget *app;
  GtkWidget *search;
  GConfClient *client;
  gint response = GTK_RESPONSE_NONE;

  app    = rubrica_get_app();
  client = gconf_client_get_default();

  search = create_search_dialog(client);
  gtk_widget_show(search);

  while (response != GTK_RESPONSE_CLOSE)
    {
      response = gtk_dialog_run(GTK_DIALOG(search));
      
      if (response == GTK_RESPONSE_ACCEPT) 
	rubrica_search_perform_search(search);
    }

  gtk_widget_destroy(search);  
  g_object_unref(G_OBJECT(client));
}


void
on_bt_exit_cb  (GtkButton *button, gpointer user_data)
{
  RubricaDoc *doc;

  doc = rubrica_folder_get_current_document();
  exit_rubrica(doc);
}



void
on_bt_cards_cb(GtkButton *button, gpointer data)
{
  rubrica_data_view_set_view_type(CARD_VIEW);
  rubrica_folder_set_active_folder(ADDRESSBOOK_FOLDER);
}


void
on_bt_account_cb (GtkButton *button, gpointer data)
{
  rubrica_data_view_set_view_type(ACCOUNT_VIEW);
  rubrica_folder_set_active_folder(ACCOUNT_FOLDER);
}


void document_notebook_switch_pages_cb(GtkNotebook *notebook, 
				       GtkNotebookPage *page,
				       gint page_num, 
				       gpointer user_data)
{
  RubricaDoc *doc;
  GtkWidget *app;
  
  rubrica_data_view_clean_view();
  doc = rubrica_folder_get_nth_document(NULL, page_num);
  if (!doc) return;

  app = rubrica_get_app();
  rubrica_folder_set_doc_as_current(NULL, doc);

  rubrica_data_view_set_path(rubrica_doc_get_path(doc));
  rubrica_data_view_set_items(rubrica_doc_get_items(doc));

  if (rubrica_doc_is_modified(doc)) 
    {
      gchar *tmp; 

      tmp = g_strdup_printf(_("Rubrica -- %s changed --"), 
			    rubrica_doc_get_name(doc)); 

      gtk_window_set_title(GTK_WINDOW(app), tmp); 
      
      g_free(tmp); 
    } 
  else 
    gtk_window_set_title(GTK_WINDOW(app), "Rubrica"); 

  if (rubrica_doc_get_index(doc) != -1)
    {
       GtkWidget *view;
       GtkTreePath *path;
       GtkTreeSelection *select;
       gchar *path_str;
       
       view   = rubrica_doc_get_view(doc);
       select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));     
       
       path_str = rubrica_doc_get_list_path(doc);
       if (path_str != NULL)
	 {   
	   gtk_tree_selection_unselect_all(GTK_TREE_SELECTION(select));
	   path = gtk_tree_path_new_from_string(path_str);
	   
	   gtk_tree_selection_select_path(select, path);
	   gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(view), path, NULL, 
					TRUE, 0.5, 0.0);  
	   
	   gtk_tree_path_free(path);
	 } 
    }
}


/*  supporto alle callback
*/
void rubrica_destroy(GtkWidget *app, gpointer data)
{
  RubricaDoc *doc = NULL;

  doc = rubrica_folder_get_current_document();

  if(doc)
    exit_rubrica(doc);
  else
    gtk_main_quit();
}


void exit_rubrica(RubricaDoc *doc)
{
  GtkWidget *dialog;
  GtkResponseType scelta;
  gchar *mes;
  gboolean exit;
  
  /*
    modificare la routine di uscita. Bisogna controllare che 
    tutte le le rubrica siano non modificate prima di uscire
  */
  exit = rubrica_preferences_get_exit();
  
  if (exit && !rubrica_doc_is_modified(doc) && 
      !rubrica_doc_others_docs_are_modified())
    {
      if (rubrica_preferences_get_autosave())
	rubrica_autosave_remove_tmp_files();
      
      gtk_main_quit();
    }
  else
    {
      if (!rubrica_doc_is_modified(doc) && 
	  rubrica_doc_others_docs_are_modified())
	mes = g_strdup_printf(_("Warning. %s\n"				  
				"Changes not saved will be lost.\n\n"
				"Do you want continue?"), 
			      _("Some Addressbooks have changed."));
      else
	if (rubrica_doc_is_modified(doc))
	  mes = g_strdup_printf(_("Warning. %s\n"
				  "Changes not saved will be lost.\n\n"
				  "Do you want continue?"), 
				_("Addressbook has changed."));
	else
	  mes = g_strdup(_("\nDo you want exit rubrica?"));
      
      dialog = rubrica_dialog_query(mes, GTK_BUTTONS_YES_NO);
      gtk_widget_show(dialog);
      g_free(mes);

      scelta = gtk_dialog_run(GTK_DIALOG(dialog));
      
      switch(scelta)
	{
	case GTK_RESPONSE_YES:	  
	  gtk_main_quit();
	  break;
	  
	case GTK_RESPONSE_NO:
	default:
	  gtk_widget_destroy(dialog);
	  break;
	}  
    }
}
