/*
*  Rubrica
*  file: rubrica.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <gnome.h>
#include <gconf/gconf-client.h>

#include "rubrica.h"
#include "autosave.h"
#include "log.h"
#include "data_view.h"
#include "utils.h"
#include "prefer.h"
#include "main.h"
#include "file.h"
#include "dictionary.h"
#include "dialog.h"
#include "card.h"
#include "card_item.h"
#include "sort.h"


RubricaCardItem* rubrica_card_item_new(void)
{
  RubricaCardItem *item;
  RubricaDoc *doc;

  item = (RubricaCardItem*) g_malloc (sizeof(RubricaCardItem));

  if (!item)
    return NULL;
  
  doc = rubrica_folder_get_current_document();
  rubrica_card_item_init(doc, item);

  return (RubricaCardItem*) item;
}


void rubrica_card_item_init(RubricaDoc *doc, RubricaCardItem *item)
{
  item->id        = rubrica_doc_get_id(doc);
  item->father    = doc;

  item->deletable = TRUE;
  item->deleted   = FALSE;
  item->marked    = FALSE;
  item->rank      = RUBRICA_RANK_NORMAL;
  item->groups    = NULL;
  item->card      = NULL;

  item->personal.first    = item->personal.middle = 
    item->personal.last   = item->personal.nick   = 
    item->personal.prof   = item->personal.prefix = 
    item->personal.title  = item->personal.photo  = NULL;

  item->personal.birth    = FALSE;
  item->personal.bdate    = time(NULL);
      
  item->addresses         = NULL;
  item->web               = NULL;
  item->email             = NULL;
  item->phone             = NULL;

  item->company.name      = item->company.street   =
    item->company.number  = item->company.zip      =
    item->company.city    = item->company.province = 
    item->company.country = item->company.web      = 
    item->company.email   = item->company.operator =
    item->company.fax     = item->company.green    = 
    item->company.ccare   = item->company.notes    = NULL;
  
  item->work.assigment    = item->work.org       =
    item->work.dep        = item->work.subdep    = 
    item->work.secretary  = item->work.telephone = NULL;
  
  item->notes.married     = item->notes.birth = item->notes.ann  = FALSE;
  item->notes.bdate       = item->notes.adate = time(NULL);
  item->notes.spouse      = item->notes.children = 
    item->notes.notes     = item->notes.pubkey   = NULL;

  item->position = -1;
}


/* does item need id? */
gboolean         
rubrica_card_item_need_id (RubricaCardItem *item)
{
  return (gboolean) (item->id == 0);
}

/* get item id */
gulong           
rubrica_card_item_get_id (RubricaCardItem *item)
{
  return (gulong) item->id;
}


/* ask id to father */
gulong 
rubrica_card_item_ask_id (RubricaCardItem *item)
{
  return rubrica_doc_get_id(item->father);
}

void 
rubrica_card_item_reject_id(RubricaCardItem *item)
{
  rubrica_doc_reject_id(item->father);
}

/* check id for all items in doc 
void 
rubrica_card_item_test_doc_id(RubricaDoc *doc)
{
  GtkTreeModel *model;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(rubrica_doc_get_view(doc)));
  gtk_tree_model_foreach(GTK_TREE_MODEL(model), 
			 rubrica_card_item_generate_id, doc);
   
}


gboolean 
rubrica_card_item_generate_id(GtkTreeModel *model, GtkTreePath *path, 
			      GtkTreeIter *iter, gpointer data)
{
  RubricaCardItem *item;
  RubricaDoc *doc = data;
  
  if (rubrica_doc_is_empty(doc))
    return TRUE;

  gtk_tree_model_get(model, iter, RUBRICA_COLUMN, &item, -1);

  if (rubrica_card_item_need_id(item))
    {
      gulong id;

      id = rubrica_doc_get_id(doc);
      rubrica_card_item_set_id(item, id);
      g_print("\nitem has id: %ld", id);
    }

  return FALSE;
}
*/



/* get item's father 
 */
RubricaDoc*      
rubrica_card_item_get_father (RubricaCardItem *item)
{
   return (RubricaDoc*) item->father;
}


/* is item deletable? 
 */
gboolean 
rubrica_card_item_is_deletable(RubricaCardItem *item)     
{
  return (gboolean) item->deletable;
}


/* is this a deleted item 
 */
gboolean 
rubrica_card_item_is_deleted(RubricaCardItem *item)     
{
  return (gboolean) item->deleted;
}


/* is this a marked item 
 */
gboolean
rubrica_card_item_is_marked(RubricaCardItem *item)     
{
  return (gboolean) item->marked;
}


/* get item's rank 
 */
RubricaRankType 
rubrica_card_item_get_rank(RubricaCardItem *item)     
{
  if (item)
    return (gboolean) item->rank;

  return RUBRICA_RANK_NORMAL;
}


/* get group list whitch item belong 
 */
RubricaGroupList
rubrica_card_item_get_groups (RubricaCardItem *item)
{
  if (item)
    return (RubricaGroupList) item->groups;

  return NULL;
}


/* get a group 
 */
RubricaGroup*
rubrica_card_item_get_group(RubricaGroupList groups)
{
  if (groups)
    return (RubricaGroup*) groups->data;

  return NULL;
}


/* get group label 
 */
gchar* 
rubrica_card_item_get_group_label(RubricaGroup* group)
{
  if (group)
    return (gchar *) group->label;

  return NULL;
}


/* get group type 
 */
RubricaGroupType
rubrica_card_item_get_group_type(RubricaGroup* group)
{
  if (group)
    return (RubricaGroupType) group->type;

  return NO_GROUP;
}

gboolean          
rubrica_card_item_belong_to_group (RubricaCardItem *item,
				   RubricaGroupType type)
{
  RubricaGroupList groups;
  RubricaGroupList lst;

  g_return_val_if_fail(item != NULL, FALSE);

  if (type == ALL_GROUP)
    return TRUE;

  lst = groups = rubrica_card_item_get_groups(item);  
  for (; lst; lst = g_list_next(lst))
    {
      RubricaGroup* group;
      
      if (!(group = rubrica_card_item_get_group(groups)))
	return FALSE;
      
      if (rubrica_card_item_get_group_type(group) == type)
	return TRUE;
    }  
  
  return FALSE;
}


gint 
rubrica_card_item_get_position(RubricaCardItem *item)
{
  g_return_val_if_fail(item != NULL, -1);

  return item->position;
}


/* set item id 
 */
void
rubrica_card_item_set_id (RubricaCardItem *item, gulong id)
{
  g_return_if_fail(item != NULL);

  item->id = (gulong) id;
}


/* set item's father to father 
 */
void             
rubrica_card_item_set_father (RubricaCardItem *item, RubricaDoc* father)
{
  g_return_if_fail(item != NULL);

  item->father = father;
}


/* set item's deletable flag 
 */
void 
rubrica_card_item_set_deletable(RubricaCardItem *item, gboolean state)
{
  g_return_if_fail(item != NULL);

  item->deletable = (gboolean) state;
}


/* set item's deleted flag 
 */ 
void 
rubrica_card_item_set_deleted(RubricaCardItem *item, gboolean state)
{
  g_return_if_fail(item != NULL);

  item->deleted = (gboolean) state;
  rubrica_doc_delete_item(item->father);
}


/* set item's marked flag 
 */
void 
rubrica_card_item_set_marked(RubricaCardItem *item, gboolean state)
{
  g_return_if_fail(item != NULL);

  item->marked = (gboolean) state;
}


/* set item's rank 
 */
void 
rubrica_card_item_set_rank (RubricaCardItem *item, RubricaRankType rank)
{
  g_return_if_fail(item != NULL);

  item->marked = (RubricaRankType) rank;
}


void 
rubrica_card_item_set_position(RubricaCardItem *item, gint pos)
{
  g_return_if_fail(item != NULL);

  item->position = pos;
}


/*   append an item in addressbook

void 
rubrica_card_item_append(RubricaDoc *doc, RubricaCardItem *item)
{
  rubrica_item_add(doc);
  rubrica_card_append_item(doc, item);
}
*/

/*   insert an item in addressbook

void 
rubrica_card_item_insert(RubricaDoc *doc, RubricaCardItem *r, gint index)
{
  rubrica_item_add(doc);
  rubrica_card_insert_item(doc, r, index);
}
*/

/*    modify a rubrica's item

void 
rubrica_card_item_modify(RubricaDoc *doc, RubricaCardItem *item, gint index)
{
  rubrica_card_modify_item(doc, item, index);

  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);
  rubrica_doc_set_empty(doc, FALSE);
}
*/

/*      get the item that will be deleted

gboolean 
rubrica_card_item_delete_selected(RubricaDoc *doc)
{
  if (rubrica_doc_is_empty_doc(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 
      
      return FALSE;
    }

  if (rubrica_card_remove_selected_item(doc))
    {  
      rubrica_item_delete(doc);
            
      return TRUE;
    }

  return FALSE;
}
 

gboolean 
rubrica_card_item_delete_last(RubricaDoc *doc)
{
  if (rubrica_doc_is_empty_doc(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return FALSE;
    }

  if(rubrica_card_remove_last_item(doc))
    {  
      rubrica_item_delete(doc);
            
      return TRUE;
    }

  return FALSE;
}



gboolean 
rubrica_card_item_delete_nth(RubricaDoc *doc, gint n)
{
  if (rubrica_doc_is_empty_doc(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return FALSE;
    }

  if(rubrica_card_remove_nth_item(doc, n))
    {  
      rubrica_item_delete(doc);
            
      return TRUE;
    }

  return FALSE;
}

*/


void 
rubrica_card_item_free_address (RubricaCardAddress* address)
{
  g_return_if_fail(address != NULL);

  rubrica_doc_free_string(address->street);
  rubrica_doc_free_string(address->number);
  rubrica_doc_free_string(address->zip);
  rubrica_doc_free_string(address->city);
  rubrica_doc_free_string(address->province);
  rubrica_doc_free_string(address->state);
  rubrica_doc_free_string(address->country);     
}


void 
rubrica_card_item_free_addresses(RubricaCardAddresses addresses)
{
  GList* link;

  for (; addresses; addresses = g_list_next(addresses))
    {
      RubricaCardAddress* address;
     
      link = addresses;
      addresses = g_list_remove_link(addresses, link);
      if (addresses->data)
	{
	  address = (RubricaCardAddress*) addresses->data;

	  rubrica_card_item_free_address(address);	  
	}
      g_list_free_1(link);
    }
}


void 
rubrica_card_item_free(RubricaCardItem *item)
{
  GList* lst;
  GList* link;

  rubrica_doc_free_string(item->card);
  rubrica_doc_free_string(item->personal.first);
  rubrica_doc_free_string(item->personal.middle);
  rubrica_doc_free_string(item->personal.last);
  rubrica_doc_free_string(item->personal.prof);
  rubrica_doc_free_string(item->personal.prefix);
  rubrica_doc_free_string(item->personal.title);

  rubrica_card_item_free_addresses(item->addresses);

  lst = item->web;
  for (; lst; lst = g_list_next(lst))
    {    
      link = lst;
      lst  = g_list_remove_link(lst, link);
      rubrica_doc_free_string(link->data);
      g_list_free_1(link);
    }
  
  lst = item->email;
  for (; lst; lst = g_list_next(lst))
    {
      link = lst;
      lst  = g_list_remove_link(lst, link);
      rubrica_doc_free_string(lst->data);  
      g_list_free_1(link);
    }

  lst = item->phone;
  for (; lst; lst = g_list_next(lst))
    {
      RubricaCardTelNum *tn;
      
      link = lst;
      lst  = g_list_remove_link(lst, link);
      if ((tn = lst->data))
	{
	  rubrica_doc_free_string(tn->type);
	  rubrica_doc_free_string(tn->number);      
	}
      g_list_free_1(link);
    }

  rubrica_doc_free_string(item->company.name);
  rubrica_doc_free_string(item->company.street);
  rubrica_doc_free_string(item->company.number);
  rubrica_doc_free_string(item->company.zip);
  rubrica_doc_free_string(item->company.city);
  rubrica_doc_free_string(item->company.province);
  rubrica_doc_free_string(item->company.country);
  rubrica_doc_free_string(item->company.web);
  rubrica_doc_free_string(item->company.email);
  rubrica_doc_free_string(item->company.operator);
  rubrica_doc_free_string(item->company.fax);
  rubrica_doc_free_string(item->company.green);
  rubrica_doc_free_string(item->company.ccare);
  rubrica_doc_free_string(item->company.notes);
  
  rubrica_doc_free_string(item->work.assigment);
  rubrica_doc_free_string(item->work.org);
  rubrica_doc_free_string(item->work.dep);
  rubrica_doc_free_string(item->work.subdep);
  rubrica_doc_free_string(item->work.secretary);
  rubrica_doc_free_string(item->work.telephone);

  rubrica_doc_free_string(item->notes.spouse);
  rubrica_doc_free_string(item->notes.children);
  rubrica_doc_free_string(item->notes.notes);
  rubrica_doc_free_string(item->notes.pubkey);

  g_free(item);
}
