/*
*  Rubrica
*  file: rubrica.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_CARD_ITEM_H
#define _RUBRICA_CARD_ITEM_H

#include <gnome.h>

#include "rubrica.h"
#include "rubrica_type.h"

typedef struct _RubricaCardItem      RubricaCardItem;
typedef struct _RubricaCardPersonal  RubricaCardPersonal;
typedef struct _RubricaCardAddress   RubricaCardAddress;
typedef struct _RubricaCardTelNum    RubricaCardTelNum;
typedef struct _RubricaCardCompany   RubricaCardCompany;
typedef struct _RubricaCardWork      RubricaCardWork;
typedef struct _RubricaCardNotes     RubricaCardNotes;

typedef GList* RubricaGroupList;
typedef GList* RubricaCardAddresses;
typedef GList* RubricaCardNet;
typedef GList* RubricaCardTelephone;

struct _RubricaCardPersonal
{
  /* name */
  gchar*   first;
  gchar*   middle;
  gchar*   last;
  gchar*   nick;

  gchar*   prof;
  gchar*   prefix;
  gchar*   title;

  gchar*   photo;    /* contact's photo (uri)     */

  gboolean birth;    /* is contact birthday know? */
  time_t   bdate;    /* contact birthday          */
};


struct _RubricaCardAddress
{
  RubricaAddressType type;

  gchar* street;
  gchar* number;
  gchar* zip;
  gchar* city;
  gchar* province;
  gchar* state;
  gchar* country;
};


struct _RubricaCardTelNum {
  gchar* number;
  gchar* type;
};

struct _RubricaCardCompany
{
  gchar* name;
  gchar* street;
  gchar* number;
  gchar* zip;
  gchar* city;
  gchar* province;
  gchar* country;
  gchar* web;
  gchar* email;
  gchar* operator;              /* company's operator */
  gchar* fax;
  gchar* green;                 /* free phone number  */
  gchar* ccare;                 /* customer care phone number */
  gchar* notes;                 
};

struct _RubricaCardWork
{
  gchar* assigment;             /* contact role        */
  gchar* org;                   /* organization        */
  gchar* dep;                   /* department          */
  gchar* subdep;                /* sub department      */
  gchar* secretary;             /* contact secretary   */
  gchar* telephone;             /* secreatry telephone */
};

struct _RubricaCardNotes
{
  gboolean married;             /* is contact married?          */
  gboolean birth;               /* is spouse birthday known ?   */
  gboolean ann;                 /* is their anniversary known ? */

  time_t bdate;                 /* spouse's birthday */
  time_t adate;                 /* anniversary       */

  gchar* spouse;                /* spouse name    */
  gchar* children;              /* children notes */
  gchar* notes;                 /* other notes    */
  gchar* pubkey;                /* public key     */
};


struct _RubricaCardItem 
{
  gulong id;                         /* card's unique id */

  RubricaDoc* father;                /* card's father    */

  gboolean deletable;                /* true if card is marked deletable */
  gboolean deleted;                  /* is this a deleted card ?         */
  gboolean marked;                   /* true if card was marked          */

  RubricaRankType  rank;             /* card's rank   */
  RubricaGroupList groups;           /* card's groups */

  gchar* card;                       /* card's name   */

  RubricaCardPersonal   personal;
  RubricaCardAddresses  addresses;
  RubricaCardNet        web;
  RubricaCardNet        email;
  RubricaCardTelephone  phone;
  RubricaCardCompany    company;
  RubricaCardWork       work;
  RubricaCardNotes      notes;

  time_t created;                    /* card's creation time    */
  time_t changed;                    /* card's last change time */
  gint position;                     /* item position in list   */
};


RubricaCardItem*  rubrica_card_item_new             (void);
void              rubrica_card_item_init            (RubricaDoc *doc, 
						     RubricaCardItem *item);

gboolean          rubrica_card_item_need_id         (RubricaCardItem *item);
gulong            rubrica_card_item_get_id          (RubricaCardItem *item);
gulong            rubrica_card_item_ask_id          (RubricaCardItem *item);
void              rubrica_card_item_reject_id       (RubricaCardItem *item);

/*
void              rubrica_card_item_test_doc_id     (RubricaDoc *doc);
gboolean          rubrica_card_item_generate_id     (GtkTreeModel *model,
						     GtkTreePath *path,   
						     GtkTreeIter *iter, 
						     gpointer data); 
*/

RubricaDoc*       rubrica_card_item_get_father      (RubricaCardItem *item);


gboolean          rubrica_card_item_is_deletable    (RubricaCardItem *item);
gboolean          rubrica_card_item_is_deleted      (RubricaCardItem *item);
gboolean          rubrica_card_item_is_marked       (RubricaCardItem *item);
RubricaRankType   rubrica_card_item_get_rank        (RubricaCardItem *item);

RubricaGroupList  rubrica_card_item_get_groups      (RubricaCardItem *item);
RubricaGroup*     rubrica_card_item_get_group       (RubricaGroupList groups);
RubricaGroup*     rubrica_card_item_get_next_group  (RubricaGroupList groups);
gchar*            rubrica_card_item_get_group_label (RubricaGroup* group);
RubricaGroupType  rubrica_card_item_get_group_type  (RubricaGroup* group);
gboolean          rubrica_card_item_belong_to_group (RubricaCardItem *item,
						     RubricaGroupType type);
gint              rubrica_card_item_get_position    (RubricaCardItem *item);


void              rubrica_card_item_set_id          (RubricaCardItem *item, 
						     gulong id);
void              rubrica_card_item_set_father      (RubricaCardItem *item,
						     RubricaDoc* father);
void              rubrica_card_item_set_deletable   (RubricaCardItem *item, 
						     gboolean bool);
void              rubrica_card_item_set_deleted     (RubricaCardItem *item,
						     gboolean state);
void              rubrica_card_item_set_marked      (RubricaCardItem *item, 
						     gboolean bool);
void              rubrica_card_item_set_rank        (RubricaCardItem *item, 
						     RubricaRankType rank);
void              rubrica_card_item_set_position    (RubricaCardItem *item,
						     gint position);


/* *********************************
   append   insert   modify   delete
*/
/*
 void rubrica_card_item_append (RubricaDoc *doc, RubricaCardItem *item); 
 void rubrica_card_item_insert (RubricaDoc *doc, RubricaCardItem *item,
			       gint index); 
 void rubrica_card_item_modify (RubricaDoc *doc, RubricaCardItem *item, 
			       gint index);


gboolean rubrica_card_item_delete_selected (RubricaDoc *doc); 
gboolean rubrica_card_item_delete_last     (RubricaDoc *doc); 
gboolean rubrica_card_item_delete_nth      (RubricaDoc *doc, gint n); 
*/


/*  Free memory
*/
void rubrica_card_item_free_addresses (RubricaCardAddresses addresses);
void rubrica_card_item_free_address   (RubricaCardAddress*  address);
void rubrica_card_item_free           (RubricaCardItem *item);
// void rubrica_card_item_free_string(gchar *str);




#endif
