/*
*  Rubrica
*  file: data_view.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <config.h>
#include <gnome.h>

#include "ui.h"
#include "rubrica.h"
#include "main.h"
#include "prefer.h"
#include "utils.h"
#include "init.h"
#include "data_view.h"

RubricaViewType  rubrica_view_type;

GtkWidget*       text_view;
GtkTextBuffer*   text_buffer;
GtkTextTagTable* tag_table;
GtkTextIter      iter;

GtkWidget*       statusbar   = NULL;
GtkWidget*       label_path  = NULL;
GtkWidget*       label_items = NULL;

gchar*           font_header = NULL;
gchar*           font_data   = NULL;

gint             items       = 0;
gint             personal    = 0;
gint             communicate = 0;
gint             notes       = 0;
gint             assigment   = 0;
gint             company     = 0;
gint             account     = 0;

gboolean         loading     = FALSE;


void 
rubrica_data_view_set_loading(gboolean bool)
{
  loading = bool;
}



void 
rubrica_data_view_inc_personal(gboolean bool)
{
  if (bool) 
    personal++;
  else 
    if (personal > 0) 
      personal--;
}

void 
rubrica_data_view_inc_communicate(gboolean bool)
{
  if (bool) 
    communicate++;
  else 
    if (communicate > 0) 
      communicate--;
}

void 
rubrica_data_view_inc_company(gboolean bool)
{
  if (bool) 
    company++;
  else
    if (company > 0) company--;
}

void 
rubrica_data_view_inc_assigment(gboolean bool)
{
  if (bool)
    assigment++;
  else 
    if (assigment > 0)
      assigment--;
}


void 
rubrica_data_view_inc_notes(gboolean bool)
{
 if (bool)
   notes++;
 else 
   if(notes > 0)
     notes--;
}

void 
rubrica_data_view_inc_account(gboolean bool)
{
  if (bool) 
    account++;
  else
    if(account > 0)
      account--;
}


/* 
   TODO:
   - add full pango support
   - add refresh of data 
   - other ?
*/


GtkWidget*
rubrica_data_view_interface_creation(void)
{
  GtkWidget *scrollwin;
  GConfClient *client;
  gchar *header_font;
  gchar *data_font;
  gchar *tmp = NULL; 
  GdkColor color; 

  client = gconf_client_get_default();

  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), 
				  GTK_POLICY_AUTOMATIC, 
				  GTK_POLICY_AUTOMATIC);  
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), 
				      GTK_SHADOW_IN);
  gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 0);
  gtk_widget_show (scrollwin);
  
  text_view   = gtk_text_view_new ();

  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  tag_table   = gtk_text_buffer_get_tag_table (GTK_TEXT_BUFFER(text_buffer));

  /* init fonts */
  header_font = gconf_client_get_string(client, RUBRICA_KEY_FONT_HEADER, NULL);
  rubrica_data_view_set_header_font(header_font);
				
  data_font = gconf_client_get_string(client, RUBRICA_KEY_FONT_DATA, NULL);
  rubrica_data_view_set_data_font(data_font);

  /* init colours */
  tmp = gconf_client_get_string(client, RUBRICA_KEY_COLOUR_HEADER, NULL); 
  if (tmp)
    {
      gdk_color_parse(tmp, &color);
      rubrica_data_view_set_header_colour(&color); 
    }
  
  tmp = gconf_client_get_string(client, RUBRICA_KEY_COLOUR_DATA, NULL);
  if (tmp)
    {
      gdk_color_parse(tmp, &color); 
      rubrica_data_view_set_data_colour(&color);  
    }

  /* init tags */
  rubrica_data_view_init_tags();

  gtk_container_add (GTK_CONTAINER (scrollwin), text_view);
  gtk_text_view_set_left_margin(GTK_TEXT_VIEW (text_view), 10);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (text_view), 10);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), FALSE);

  gtk_widget_show (text_view);
 
  g_object_unref(client);

  return (GtkWidget *) scrollwin;
}


void 
rubrica_data_view_init_tags(void)
{
  gtk_text_buffer_create_tag (text_buffer, "bold", "weight", 
			      PANGO_WEIGHT_BOLD, NULL);
  gtk_text_buffer_create_tag (text_buffer, "center", "justification", 
			      GTK_JUSTIFY_CENTER, NULL);
  gtk_text_buffer_create_tag (text_buffer, "margins", "left_margin", 10, 
			      "right_margin", 10, NULL);  
}


void 
rubrica_data_view_init_view_type(RubricaViewType view_type)
{
  rubrica_view_type = CARD_VIEW;
}


void 
rubrica_data_view_set_view_type(RubricaViewType view_type)
{
  rubrica_view_type = (RubricaViewType) view_type;
}

RubricaViewType 
rubrica_data_view_get_view_type(void)
{
  return (RubricaViewType) rubrica_view_type;
}


void 
rubrica_data_view_set_header_font(gchar *font)
{
  PangoFontDescription *font_desc;
  GtkTextTag* tag;

  if (font_header)
    g_free(font_header);

  if (font)
    {
      font_header = g_strdup(font);
      g_print("\nRubrica: setting %s as header font", font);
    }
  else
    {      
      font_header = g_strdup("Times 12");
      g_print("\nRubrica: setting Times 12 as default header font");
    }

  font_desc = pango_font_description_from_string (font_header);  
  tag       = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table), 
					 "header_font");
  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);
    
  gtk_text_buffer_create_tag (text_buffer, "header_font", 
			      "font-desc", font_desc, NULL);
  pango_font_description_free (font_desc);  
}


void 
rubrica_data_view_set_data_font(gchar *font)
{
  PangoFontDescription *font_desc;
  GtkTextTag* tag;

  if (font_data)
    g_free(font_data);
  
  if (font)
    {
      font_data = g_strdup(font);
      g_print("\nRubrica: setting %s as data font", font);
    }
  else 
    {
      font_data = g_strdup("Times 12");
      g_print("\nRubrica: setting Times 12 as default data font");
    }

  font_desc = pango_font_description_from_string (font_data);  
  tag       = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table), 
					 "data_font");
  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);

  gtk_text_buffer_create_tag (text_buffer, "data_font", 
			      "font-desc", font_desc, NULL);
  pango_font_description_free (font_desc);
}

void 
rubrica_data_view_set_data_colour(GdkColor *color) 
{
  GtkTextTag* tag; 
  
  tag = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table),
				   "data_colour"); 

  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);
  
  gtk_text_buffer_create_tag (text_buffer, "data_colour", 
			      "foreground-gdk", color, NULL); 

  g_print("\nRubrica: setting data colour");
} 

void 
rubrica_data_view_set_header_colour(GdkColor *color) 
{
  GtkTextTag* tag; 
  tag = gtk_text_tag_table_lookup (GTK_TEXT_TAG_TABLE(tag_table), 
				   "header_colour"); 

  if (tag)
    gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(tag_table), tag);
  
  gtk_text_buffer_create_tag (text_buffer, "header_colour", 
			      "foreground-gdk", color, NULL); 

  g_print("\nRubrica: setting header colour");
}


gchar *
rubrica_data_view_get_header_font(void)
{
  return (gchar *) font_header;
}

gchar *
rubrica_data_view_get_data_font(void)
{
  return (gchar *) font_data;
}


void 
rubrica_data_view_clean_view(void)
{
  rubrica_ui_clean_text_view(text_view);
}

void 
rubrica_data_view_set_statusbar(GtkWidget *sbar)
{
  statusbar = sbar;
}

GtkWidget*
rubrica_data_view_get_statusbar(void)
{
  return (GtkWidget*) statusbar;
}

GtkWidget*
rubrica_data_view_label_items_new(void)
{
  label_items = gtk_label_new(NULL);

  return label_items;
}


GtkWidget*  
rubrica_data_view_label_path_new(void)
{
  label_path = gtk_label_new(NULL);

  return label_path;
}


void 
rubrica_data_view_show_path(gboolean bool)
{
  if (bool)
    gtk_widget_show(label_path);
  else
    gtk_widget_hide(label_path);
}


void 
rubrica_data_view_set_items(gint n)
{
  if (loading) return;

  gchar* str;

  items = n;
  str = g_strdup_printf("%d", n);

  gtk_label_set_label(GTK_LABEL(label_items), str);
}

gint rubrica_data_view_get_items(void)  
{
  return (gint) items;  
}


void 
rubrica_data_view_set_path(gchar *str)
{
  gtk_label_set_text(GTK_LABEL(label_path),str);		   
}


void 
rubrica_data_view_write_header(gchar *header,  // header label
			       gchar *data)    // data (card name or NULL)
{
  gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, header, -1, 
					   "header_font",
					   "header_colour",
					   "margins", 
					   "bold", 
					   "center", NULL);
  if (data)
    {
      gchar *text;

      text = g_strdup_printf("%s\n", data);

      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, ": ", -1, 
					       "header_font",
					       "header_colour",
					       "margins", 
					       "bold", 
					       "center", NULL);       

      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, text, -1, 
					       "data_font",
					       "data_colour",
					       "margins", NULL);

      g_free(text);
    }  
}


void 
rubrica_data_view_write_item(GConfClient *client,
			     gchar *key,   // GConf key
			     gchar *str,   // header label
			     gchar *label) // data
{
  gchar *string;

  if (!label || (g_ascii_strcasecmp(label, "") == 0)) return;

  string = g_strdup_printf("\n%s: ", str);
  
  /* check if user wants to see this data */
  if (gconf_client_get_bool(client, key, NULL))
  {        
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, string, -1, 
					     "header_font",
					     "header_colour",
					     "margins",
					     "bold", NULL);  
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, label, -1, 
					     "data_font",
					     "data_colour",
					     "margins", NULL); 
  }
  
  g_free(string);
}


/*  TODO: impostare il formato della data
*/
void 
rubrica_data_view_write_date_item(GConfClient *client, 
				  gchar *key,    // GConf key
				  gchar *str,    // header label
				  gboolean bool, // known date?
				  gchar *label)  // date data
{
  if (bool)
    {
      gchar *string;
      
      string = g_strdup_printf("\n%s: ", str);
      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, string, -1, 
					       "header_font",
					       "header_colour",
					       "margins", 
					       "bold", NULL);   

      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, 
					       label, -1, 
					       "data_font",
					       "data_colour",
					       "margins", NULL);
  
      g_free(string);
    }
}


/*  write company's web and email 
*/
void 
rubrica_data_view_write_net_item(GConfClient *client, 
				 gchar *key, 
				 gchar *str,    // header label 
				 gchar *label,  // net data
				 RubricaViewNetType type)  // data type
{
  GtkTextChildAnchor *anchor;
  GtkWidget *button;
  gchar *string;
  gchar *uri;
  
  if (gconf_client_get_bool(client, key, NULL))
  {
    string = g_strdup_printf("\n%s:\n", str);
    gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, string, -1, 
					     "header_font",
					     "header_colour",
					     "margins", 
					     "bold", NULL);  
    g_free(string);
    
    if (type == RUBRICA_VIEW_EMAIL)
      {
	string = g_strdup(label);
	uri = g_strdup_printf("mailto:%s", label);
      }
    else
      {
	string = g_strdup(label);
	uri = g_strdup(label);
      }
    
      button = gnome_href_new(uri, string);      
      anchor = gtk_text_buffer_create_child_anchor(text_buffer, &iter);
      gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW(text_view), 
					 button, anchor); 
      gtk_widget_show(button);

      g_free(string);
      g_free(uri);
  }  
}

/* write web and email urls
 */
void 
rubrica_data_view_net_data(GList *net, RubricaViewNetType type)
{
  GConfClient *client;
  GtkTextChildAnchor *anchor;
  GtkWidget *button;
  GList* alias;

  if (!net) return;
  if (!net->data) return;

  client = gconf_client_get_default();

  if (type == RUBRICA_VIEW_EMAIL)
    rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_EMAIL, 
				 _("Email"), ""); 
  else
    rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_WEB, 
				 _("Web"), ""); 

  alias = net;
  for (; alias; alias = alias->next)
    {
      gchar *uri;
      gchar *str;
      
      if (type == RUBRICA_VIEW_EMAIL)
	{
	  str = g_strdup(alias->data);
	  uri = g_strdup_printf("mailto:%s", (gchar*) alias->data);
	}
      else
	{
	  str = g_strdup(alias->data);
	  uri = g_strdup(alias->data);
	}

      button = gnome_href_new(uri, str);      
      gtk_text_buffer_insert_with_tags_by_name(text_buffer, &iter, "\n\t", 
					       -1, "margins", NULL);
      
      anchor = gtk_text_buffer_create_child_anchor(text_buffer, &iter);
      gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW(text_view), 
					 button, anchor);      
      gtk_widget_show(button);

      g_free(uri);
      g_free(str);
    }
}


void  
rubrica_data_view_show_photo(gchar* photo_path)
{
  GdkPixbuf *pixbuf = NULL;
  GdkPixbuf *scaled;
  
  pixbuf = gdk_pixbuf_new_from_file (photo_path, NULL);
  
  if (!pixbuf) 
    {
      g_warning("Errore nel caricamento dell'immagine");
      
      return;
    }
  
  scaled = gdk_pixbuf_scale_simple (pixbuf, 128, 128, GDK_INTERP_BILINEAR);
  g_object_unref (pixbuf);
  pixbuf = scaled;   

  gtk_text_buffer_insert_pixbuf (text_buffer, &iter, pixbuf);
}



void 
rubrica_data_view_show_card_item(RubricaCardItem *item)
{
  GConfClient *client;
  GList* alias;
  
  if (!item) return;

  client = gconf_client_get_default();
  rubrica_ui_clean_text_view(text_view);

  gtk_text_buffer_get_iter_at_offset(text_buffer, &iter, 0);

  rubrica_data_view_write_header(_("Card"), item->card);
  rubrica_data_view_show_photo(item->personal.photo);
  
  if (personal)
    {
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_FIRST_NAME, 
				   _("First Name"), item->personal.first);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_MIDDLE_NAME, 
				   _("Middle Name"), item->personal.middle);
// TODO show nickname      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_LAST_NAME, 
				   _("Last Name"), item->personal.last);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_PROFESSION, 
				   _("Profession"), item->personal.prof);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_NAME_PREFIX, 
				   _("Name Prefix"), item->personal.prefix);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_TITLE, 
				   _("Title"), item->personal.title);
      rubrica_data_view_write_date_item(client, RUBRICA_KEY_SCD_BIRTHDAY, 
					_("BirthDay"), 
					item->personal.birth,
					RUBRICA_DATE(item->personal.bdate, 
						     RT_MDY));

      alias = item->addresses;
      for (; alias; alias = g_list_next(alias))
	{
	  RubricaCardAddress* address;

	  if ((address = alias->data))
	    {
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_STREET, 
					   _("Street"), address->street);
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_STREET_NUM, 
					   _("Street Number"),address->number);
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_ZIP_CODE, 
					   _("Zip Code"), address->zip);
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_CITY, 
					   _("City"), address->city);
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_PROVINCE, 
					   _("Province"), address->province);
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_STATE, 
					   _("State"), address->state);
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COUNTRY, 
					   _("Country"), address->country);
	    }
	}
    }
  
  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_WEB, NULL) || 
      gconf_client_get_bool(client, RUBRICA_KEY_SCD_EMAIL, NULL))
    {
      if (item->web || item->email)
	{
	  rubrica_data_view_write_header(_("\n\nNet Data"), NULL);
	  
	  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_WEB, NULL))
	    rubrica_data_view_net_data(item->web, RUBRICA_VIEW_WEB);
	  
	  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_EMAIL, NULL))
	    rubrica_data_view_net_data(item->email, RUBRICA_VIEW_EMAIL);
	}
    }


  if (gconf_client_get_bool(client, RUBRICA_KEY_SCD_TELEPHONE, NULL))
    {
      if (item->phone && item->phone->data)
	{
	  rubrica_data_view_write_header(_("\n\nTelephone Data"), NULL);
	  
	  alias = item->phone;
	  for (; alias; alias = alias->next)
	    {
	      RubricaCardTelNum *tn;
	      gchar *str;
	      
	      tn = (RubricaCardTelNum *) alias->data;
	      str = g_strdup_printf("%s\t%s", tn->number, tn->type);
	      
	      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_TELEPHONE, 
					   _("Telephone"), str);     
	      
	      g_free(str);
	    }
	}
    }
 
  if (company)
    {
      rubrica_data_view_write_header(_("\n\nCompany Data"), NULL);

      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY, 
				   _("Company"), item->company.name);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_STREET, 
				   _("Street"), item->company.street);
      rubrica_data_view_write_item(client, 
				   RUBRICA_KEY_SCD_COMPANY_STREET_NUM, 
				   _("Street Number"), item->company.number);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_ZIP_CODE,
				   _("Zip Code"), item->company.zip);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_CITY, 
				   _("City"), item->company.city);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_PROVINCE, 
				   _("Province"), item->company.province);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_COUNTRY, 
				   _("Country"), item->company.country);
      rubrica_data_view_write_net_item(client, RUBRICA_KEY_SCD_COMPANY_WEB, 
				       _("Web"), item->company.web, 
				       RUBRICA_VIEW_WEB);
      rubrica_data_view_write_net_item(client,
				       RUBRICA_KEY_SCD_COMPANY_EMAIL, 
				       _("Email"), item->company.email, 
				       RUBRICA_VIEW_EMAIL);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_OPERATOR, 
				   _("Operator"), item->company.operator);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_FAX, 
				   _("Fax"), item->company.fax);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_GREEN, 
				   _("Green"), item->company.green);
      rubrica_data_view_write_item(client,
				   RUBRICA_KEY_SCD_CUSTOMER_CARE, 
				   _("Customer Care"), 
				   item->company.ccare);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_COMPANY_NOTE, 
				   _("Note"), item->company.notes);
      
    }

  if (assigment)
    {
      rubrica_data_view_write_header(_("\n\nWork Data\n"), NULL); 
	
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_ASSIGMENT, 
				   _("Assigment"), item->work.assigment);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_ORG, 
				   _("Organization"), item->work.org);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_DEP, 
				   _("Department"), item->work.dep);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SUB_DEP, 
				   _("Sub Department"), item->work.subdep);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SECRETARY, 
				   _("Secretary"), item->work.secretary);
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SECR_PHONE, 
				   _("Secretary Telephone"), 
				   item->work.telephone);
    }

  if (notes)
    {
      rubrica_data_view_write_header(_("\n\nNotes"), NULL);
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_MARRIED,
				   _("Contact"), 
				   item->notes.married ? 
				   _("is married") : _("is not married"));
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_SPOUSE, 
				   _("Spouse"), item->notes.spouse);
      
      rubrica_data_view_write_date_item(client, 
					RUBRICA_KEY_SCD_SPOUSE_BIRTH, 
					_("Spouse birthday"),
					item->notes.birth,
					RUBRICA_DATE(item->notes.bdate, 
						     RT_MDY));
      
      rubrica_data_view_write_date_item(client, RUBRICA_KEY_SCD_ANNIVERSARY,
					_("Marriage anniversary"), 
					item->notes.ann,
					RUBRICA_DATE(item->notes.adate, 
						     RT_MDY));
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_CHILDREN, 
				   _("Children note"), item->notes.children);
        
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_NOTE, _("Note"), 
				   item->notes.notes);
      
      rubrica_data_view_write_item(client, RUBRICA_KEY_SCD_PUBLIC_KEY, 
				   _("Public key"), item->notes.pubkey);
    }
}


/*
void 
rubrica_data_view_show_account_item(RubricaAccountItem *item)
{
  GConfClient *client;
  
  client = gconf_client_get_default();
  rubrica_ui_clean_text_view(text_view);
  gtk_text_buffer_get_iter_at_offset(text_buffer, &iter, 0);

  rubrica_data_view_write_header(_("Account"), item->name);
  rubrica_data_view_write_item(client, RUBRICA_KEY_SAD_LOGIN, 
			       _("Login"), item->login);
  rubrica_data_view_write_item(client, RUBRICA_KEY_SAD_PASSWORD, 
			       _("Password"),item->password);
  rubrica_data_view_write_item(client, RUBRICA_KEY_SAD_WEB, 
			       _("Web"), item->web);
  rubrica_data_view_write_item(client, RUBRICA_KEY_SAD_EMAIL, 
			       _("Email"), item->email);
  rubrica_data_view_write_item(client, RUBRICA_KEY_SAD_DESC, 
			       _("Account description"), item->short_desc);    
  rubrica_data_view_write_item(client, RUBRICA_KEY_SAD_LONG_DESC,
			       _("Long descriotion"), item->long_desc);
}
*/
