/*
*  Rubrica
*  file: file_card.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <stdio.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <time.h>


#include "rubrica.h"
#include "file.h"
#include "file_card.h"
#include "card.h"
#include "form_card.h"
#include "prefer.h"
#include "log.h"
#include "data_view.h"
#include "utils.h"

/*  RubricaDoc file (addressbook type) structure:
    
    <Rubrica>                                      root
       <Card id="." rank=".." ...>                 root child
          <Groups>                                 card child 
	     <Group>...</Group>                    groups children
	     ...
	  </Groups>       
	  <Data>                                   card child
	     <FirstName>...</FirstName>            data child 
	     <MiddleName>...</MiddleName>           "     "
	     <LastName>...</LastName>               "     "
	     ....
	  </Data>
	  <Addresses>                              card child
	     <Address>                             addresses children
	        <Street number="..">...</Street>   address child
		...
	     </Address>
	     ...
	  </Addresses>
	  <WebAddresses>                           card child
	     <Web>...</Web>                        webaddresses children
	     ...
	  </WebAddresses>
          <EmailAddresses>                         card child
	     <Email>...</Email>                    emailaddresses children
	     ...
	  </EmailAddresses>
	  <Company>
	     <CompanyName>...</CompanyName>
	     ...
	  </Company>
	  <Work>
	     <Department>...</Department>
	  </Work>
	  <Notes>
	     <Spouse></Spouse>
	     ...
	  </Notes>
       </Card>
       <Card>
       ....
       </Card>
    </Rubrica>


  Rubrica
     |
     | child
     |
     v   next        next       next 
   Card-------->Card------>....------>Card
     |
     | child
     |
     v   next                
   Groups-->Data->Addresses->WebAddresses->EmailAddresses->Company->Work->Notes
     |        \       |           |              |            |      |      |
     | child  | child | child     |              |            |      |      |
     |        |       v           v              v            v      v      v
     |        |     ...         ...            ...          ...    ...    ...
     v        v
   Group    FirstName--->MiddleName--->...
*/


void
rubrica_file_card_write_file(GtkWidget *ok_button, gpointer data)
{
  RubricaDoc *doc;
  GtkWidget *fs;
  gchar *tmp;
  gchar *fname = NULL;

  fs  = (GtkWidget *) data;
  doc = g_object_get_data(G_OBJECT(fs), "rubrica");

  tmp = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

  /*      if file hasn't extension or extension isn'r .rub
   */
  if (!rubrica_file_has_rub_extension(tmp)) 
    fname = g_strdup_printf("%s.rub",tmp);  
  else 
    fname = g_strdup(tmp);
  
  /*    ERROR if file was not selected, or can't open the file
  */  
  if (!fname || !rubrica_file_card_write_xmlfile(doc, fname)) 
    {
      gchar *msg;
      
      if (fname)
	msg = g_strdup_printf(_("I can't save the file:\n%s"), fname); 
      else
	msg = g_strdup_printf(_("I can't save a file without name.")); 
      
      rubrica_dialog_error(msg);

      g_free(msg);
    }
  else /* file has been saved */
    {      
      gchar *path = NULL;
      gchar *name = NULL;
      gchar *newname = NULL;

      name = g_path_get_basename(fname);
      if (rubrica_preferences_get_have_default_folder())  
	path  = rubrica_preferences_get_default_folder();
      else	
	path  = g_path_get_dirname(fname);
      
      newname = g_strdup_printf("%s/%s", path, name);
          
      /* set the saved file as last opened/showed
      */
      rubrica_preferences_set_last_book(newname);
      rubrica_doc_set_path(doc, path);
      rubrica_doc_set_name(doc, name);
      rubrica_doc_set_modified(doc, FALSE);
          
      g_free(newname);
      g_free(name);
      g_free(path);
    }

  gtk_widget_destroy(fs);
  g_free(fname);
}

gboolean
rubrica_file_card_write_xmlfile(RubricaDoc *doc, gchar *name)
{
  xmlDocPtr xmldoc;     /* pointer to xml doc */
  gchar *path = NULL, *fname;
  gchar *msg;
  
  g_return_val_if_fail(name != NULL, FALSE);

  path  = g_path_get_dirname(name);
  fname = g_strdup(name);

  xmldoc = rubrica_doc_get_xml_tree(doc);

  /* make new xml doc

  xmldoc = xmlNewDoc("1.0"); 
   */  
  /* make a new rubrica's root node 
 
  fileformat = g_strdup_printf("%d", RUBRICA_FILE_FORMAT); 

  xmldoc->children = xmlNewDocNode(xmldoc, NULL, "Rubrica", NULL); 
  xmlSetProp(xmldoc->children, "version", VERSION); 
  xmlSetProp(xmldoc->children, "fileformat", fileformat);  
  xmlSetProp(xmldoc->children, "doctype", "AddressBook");  

  g_free(fileformat); 
  */
  /* browse addressbook and write data in xml tree 

  len = rubrica_doc_get_items(doc); 
  for (i = 0; i < len; i++)  
    {
      RubricaCardItem *item; 

      item = (RubricaCardItem *) rubrica_doc_get_nth_item(doc, i); 
      
      if (item) 
	rubrica_file_card_build_xmlnode(xmldoc->children, item); 
      else 
	g_print("\nRubrica error *** rubrica_file_write_xmlfile -- " 
		"try to write a NULL item"); 
    } 
  */  

  /* test if exist a file with requested name
     if asked make backup of old file
     otherwise delete the old file and write the new one
  */
  if (g_file_test(fname, G_FILE_TEST_EXISTS))
    {
      if (rubrica_preferences_get_do_backup())
	rubrica_file_do_backup(doc, fname);
      else
	{
	  remove(fname);
	}
    }
  
  /* compress doc?
  */
  if (rubrica_preferences_get_compress())
    {
      xmlSetCompressMode(rubrica_preferences_get_compress_rate());
      xmlSetDocCompressMode(xmldoc, rubrica_preferences_get_compress_rate());
    }
  else
    {
      xmlSetDocCompressMode(xmldoc, 0);     
      xmlSetCompressMode(0);
    }
  
  /*  write xml tree
  */
  if (xmlSaveFormatFile(fname, xmldoc, 1) == -1) 
    {
      msg = g_strdup_printf(_("An error occurred during file saving.\n"
			      "I can't save the file %s"), fname);
      rubrica_log_write_log(msg);
      rubrica_dialog_error(msg);

      xmlFreeDoc(xmldoc);
      g_free(msg);
      g_free(path);
      g_free(fname);

      return FALSE;
    }  

  /* if we are here the addressbook has been writen 
   */
  msg = g_strdup_printf("\naddressbook saved as %s in folder: %s\n", 
			name, path);
  rubrica_log_write_log(msg);
  
  rubrica_data_view_set_path(path);

  /* xmlFreeDoc do a segfault, why? */
  //  xmlFreeDoc(doc);    
  g_free(fname);
  g_free(path);
  g_free(msg);

  return TRUE;
}

void 
rubrica_file_card_build_xmlnode(xmlNodePtr xmlnode, RubricaCardItem *item)
{

}



/* 
 *  read the addressbook file 
 */
gboolean 
rubrica_file_card_real_load_xmlfile(RubricaDoc *doc, gchar *fname, 
				    RubricaErrorType *err)
{
  xmlDocPtr xmldoc = NULL;
  gint fileformat  = 0;
  gchar* msg;

  if((xmldoc = rubrica_file_open_doc(fname)) == NULL)
    return FALSE;


  if(!rubrica_file_is_rubrica_doc(xmldoc, fname))
    return FALSE;

  /* get the file format
   */
  fileformat = rubrica_file_get_fileformat(xmldoc);
  if (fileformat < RUBRICA_FILE_FORMAT)  /* this is an old rubrica file */
    {
      /* close opened file */
      xmlFreeDoc(xmldoc);
  
      if (!rubrica_file_import_old_file(fname))
	return FALSE;

      if((xmldoc = rubrica_file_open_doc(fname)) == NULL)
	return FALSE;
      
      if(!rubrica_file_is_rubrica_doc(xmldoc, fname))
	return FALSE;     
    }

  /* parse doc, extract address from file and put them in memory 
   */
  rubrica_file_card_parse_doc(doc, xmldoc, err);
  
  /* correggere id */
//  rubrica_card_item_test_id(doc);
  
  msg = g_strdup_printf(_("document %s succesfully opened"), fname);
  g_print("\n%s", msg);  
  g_free(msg);

  /* addressbook has been parsed. Free memory */
  xmlFreeDoc(xmldoc);
  
  return TRUE;  
}


void 
rubrica_file_card_parse_doc(RubricaDoc *doc, xmlDocPtr xmldoc, 
			    RubricaErrorType *err)
{  
  xmlNodePtr node;
  gchar *doctype;
  gint n = 0;
      
  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  e' doc->children->children == xmlnew (salvato)
     
     nodo (= doc->children) contiene "Rubrica"
     nodo->children (doc->children->children) e' vuoto
     nodo->children->next (doc->children->children->next) 
     e' il primo nodo utile ("Card")     
  */

  doctype = rubrica_file_get_document_tpye(xmldoc);

  node = xmldoc->children;
  if (xmlIsBlankNode(node))
    node = node->next;
  
  if (node && node->name && g_ascii_strcasecmp(node->name, "Rubrica") == 0) 
    {
      if (g_ascii_strcasecmp(doctype, "AddressBook") == 0)
	{
	  node = node->children;
	  
	  if (xmlIsBlankNode(node))
	    node = node->next;
	  
	  /* node is the head of a Card's list */
	  rubrica_data_view_set_loading(TRUE);
	  while (node)
	    {
	      rubrica_file_card_load_from_xmlnode(doc, node);  
	      n++;
	      
	      /* parse next Card */
	      node = node->next;
	      if(xmlIsBlankNode(node))
		node = node->next;
	    }
	  rubrica_data_view_set_loading(FALSE);
	}

      /* altri formati andranno qui 
	 support to other file format goes here
      */
    }
  else
    *err = RE_XML_DOC_NO_RUBRICA_HEADER;

  *err = RE_NO_ERROR;
  
  rubrica_data_view_set_items(n);
  g_print("\nRubrica reads %d card%s", n, (n > 1 ? "s" : ""));
  
}


void
rubrica_file_card_load_from_xmlnode(RubricaDoc *doc, xmlNodePtr xmlcard)
{
  RubricaCardItem *item;  
  xmlNodePtr child;

  /* gli xmlnode sono nodi cardxml da cui estrarre i nodi figli */

  item = rubrica_card_item_new();
// DEBUG(rubrica, item);  

  /*      Get data in Card node
  */
  if(xmlIsBlankNode(xmlcard))
    xmlcard = xmlcard->next;
  rubrica_file_card_load_card_data(item, xmlcard);

  /*      Get data in Card childres   */

  /*    first child: group list
   */     
  child = xmlcard->children;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_groups_data(item, child);  

  /*    next child: personal data
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_personal_data(item, child);

  /*    next child: addresses
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_address_data(item, child);  

  /*    next child: web 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_web_data(item, child);


  /*    next child: email 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_email_data(item, child);

  
  /*    next child: telephone 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_telephone_data(item, child);

  
  /*    next child: company 
   */
  child = child->next; 
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_company_data(item, child);

  
  /*    next child: work 
   */
  child = child->next; 
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_work_data(item, child);


  /*    next child: note 
   */
  child = child->next;
  if(xmlIsBlankNode(child))
    child = child->next;
  rubrica_file_card_load_note_data(item, child);

  g_print("\n\t\tAPPENDING %s", item->card);
  rubrica_card_append_item(doc, item);
  g_print("\n\t\tELEMENTO APPESO");
}


void 
rubrica_file_card_load_card_data(RubricaCardItem *item, xmlNodePtr node)
{
  gchar *tmp = NULL;  
  gulong val = 0L;

  if (node &&                                      /* se il node e' valido e */
      node->name &&                                /* se il node ha nome e  */
      g_ascii_strcasecmp(node->name, "Card") == 0) /* il nome e' Card  */
    {
      item->card = ((tmp = xmlGetProp(node, "name")) ? g_strdup(tmp) : "");
      
      g_print("\nreading card: %s", tmp);
      if (tmp) 
	g_free(tmp);
      
      item->deletable = ((g_ascii_strcasecmp(xmlGetProp(node, "deletable"), 
					     "true") == 0) ? TRUE : FALSE);
      item->deleted   = ((g_ascii_strcasecmp(xmlGetProp(node, "deleted"), 
					     "yes") == 0) ? TRUE : FALSE);
      item->rank      = (RubricaRankType) atoi(xmlGetProp(node, "rank"));
      
      if (xmlHasProp(node, "id"))
	{
	  tmp = xmlGetProp(node, "id") ? tmp : "0";
	  val = atol(tmp);
	  g_free(tmp);
	}      
      if (val == 0L)
	item->id = rubrica_card_item_ask_id(item);
      else
	item->id = val; 

      item->created = ((tmp = xmlGetProp(node, "created")) ? 
		       atol(tmp) : time(NULL));
      if (tmp) g_free(tmp);
      
      item->changed = ((tmp = xmlGetProp(node, "last_change")) ? 
		       atol(tmp) : time(NULL));
      if (tmp) g_free(tmp);      
    }
  else
    {
      /* TODO 
	 
      segnalare l'errore, far scegliere all'utente se 
      continuare a caricare automaticamente il file 
      o vuole un caricamento assistito
      */	 
    }
}


void 
rubrica_file_card_load_groups_data(RubricaCardItem *item, xmlNodePtr xmlgroups)
{
  xmlNodePtr child;

  if (xmlgroups && xmlgroups->name && 
      g_ascii_strcasecmp(xmlgroups->name, "Groups") == 0)
    {
      child = xmlgroups->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      
      while (child)
	{
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  
	  if (child)
	    {
	      RubricaGroup* group;
	      gchar* tmp;

	      group = (RubricaGroup*) g_malloc(sizeof(RubricaGroup));
	      if (!group) return;
	      
	      group->label = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");
	      group->type  = rubrica_form_card_get_option_type(group->label);  
	      item->groups = g_list_append(item->groups, group);
	      
	      child = child->next;
	    }
	}
    }
}


void 
rubrica_file_card_load_personal_data(RubricaCardItem *item, xmlNodePtr xmldata)
{
  gchar *tmp = NULL;
  xmlNodePtr child;
    
  if (xmldata && xmldata->name &&
      g_ascii_strcasecmp(xmldata->name, "Data") == 0)  
    {
      /* carico i figli di datixml
       */   
      child = xmldata->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      item->personal.first = ((tmp = xmlNodeGetContent(child)) ?
			      g_strdup(tmp) : ""); 
      tmp = xmlGetProp(child, "know_birthday");      
      item->personal.birth = (g_ascii_strcasecmp(tmp, "true") == 0) ? TRUE : FALSE;
      item->personal.bdate = ((tmp = xmlGetProp(child, "birthday")) ? 
			      atol(tmp) : time(NULL));

//      g_print("\nnome letto %s", item->personal.first);

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
//      g_print("\nmiddle");
      item->personal.middle = ((tmp = xmlNodeGetContent(child)) ? 
			       g_strdup(tmp) : "");
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;      
//      g_print("\nlast");	
      item->personal.last = ((tmp  = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");
      
      child = child->next;  
      if (xmlIsBlankNode(child))
	child = child->next;    
//      g_print("\nnickname");	
      item->personal.nick = ((tmp = xmlNodeGetContent(child)) ?
			     g_strdup(tmp) : "");

      child = child->next;  
      if (xmlIsBlankNode(child))
	child = child->next;    
//      g_print("\nprofession");	
      item->personal.prof = ((tmp = xmlNodeGetContent(child)) ?
			     g_strdup(tmp) : "");
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
//      g_print("\nprefix");	
      item->personal.prefix = ((tmp = xmlNodeGetContent(child)) ? 
			       g_strdup(tmp) : "");
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
//      g_print("\ntitle");	
      item->personal.title = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");
    }
  else 
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Data");
      
      item->personal.first = item->personal.middle = item->personal.last 
	= item->personal.prefix = item->personal.title = "";
      item->personal.bdate = time(NULL);
    }
}  


void 
rubrica_file_card_load_address_data(RubricaCardItem *item, xmlNodePtr xmladd)
{
  xmlNodePtr xmladdress;
  xmlNodePtr child;
  gchar *tmp;
  
  if (xmladd &&
      xmladd->name &&
      g_ascii_strcasecmp(xmladd->name, "Addresses") == 0)
    {
      /* child is the head of an <address> list */
      xmladdress = xmladd->children;  
      if (xmlIsBlankNode(xmladdress))
	xmladdress = xmladdress->next;
      
      while(xmladdress)
	{
	  RubricaCardAddress* address;

	  address = (RubricaCardAddress*) g_malloc(sizeof(RubricaCardAddress));
	  
	  tmp = xmlGetProp(xmladdress, "type");
	  address->type = RUBRICA_STR2ADDRESS_TYPE(tmp);

	  /* get fist <address> child */
	  child = xmladdress->children;  
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  address->street = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");
	  address->number = (tmp = xmlGetProp(child, "number") ? 
			     g_strdup(tmp) : "");
	  
/*	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->number = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");

	  
	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->zip = ((tmp = xmlNodeGetContent(child)) ? 
			  g_strdup(tmp) : "");
*/	  
	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->city = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");
	  address->zip = (tmp = xmlGetProp(child, "zip") ? 
			  g_strdup(tmp) : "");

	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->province = ((tmp = xmlNodeGetContent(child)) ? 
			       g_strdup(tmp) : "");
	  
	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->state = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");
	  
	  child = child->next;
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  address->country = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");

	  item->addresses = g_list_append(item->addresses, address);
	  
	  xmladdress = xmladdress->next;
	  if (xmlIsBlankNode(xmladdress))
	    xmladdress = xmladdress->next;
	}
    }
  else 
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Address");
      
      item->addresses = NULL;
    }
}

  
void 
rubrica_file_card_load_web_data(RubricaCardItem *item, xmlNodePtr node)
{
  GList *web = NULL;
  xmlNodePtr child;
  gchar *buffer;
  
  if (node &&
      node->name &&
      g_ascii_strcasecmp(node->name, "WebAddresses") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      while(child)
	{
	  gchar *tmp;
	  
	  if (xmlIsBlankNode(child))
	    child = child->next;
    	  
	  tmp = xmlNodeGetContent(child);
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      web = g_list_append(web, buffer);
	      child = child->next;
	    }	  
	}
      item->web = web;
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"WebAddresses");
      
      item->web = NULL;
    }
}


void 
rubrica_file_card_load_email_data(RubricaCardItem *item, xmlNodePtr node)
{
  GList *email = NULL;
  xmlNodePtr child;
  gchar *buffer;

  if (node &&
      node->name &&
      g_ascii_strcasecmp(node->name, "EmailAddresses") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      while(child)
	{
	  gchar *tmp;
	  
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  
	  tmp = xmlNodeGetContent(child);
	  
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      email = g_list_append(email, buffer);
	      child = child->next;
	    }	
	}
      item->email = email;
    }  
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"EmailAddresses");
      
      item->email = NULL;
    }
}


void
rubrica_file_card_load_telephone_data(RubricaCardItem *item, xmlNodePtr node)
{
  GList *tel = NULL;
  xmlNodePtr child;
  RubricaCardTelNum *tn;

  if (node &&
      node->name &&
      g_ascii_strcasecmp(node->name, "TelephoneNumbers") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      while(child)
	{
	  gchar *tmp1=NULL, *tmp2=NULL;
	  
	  if (child)
	    {
	      if ((tmp1= xmlNodeGetContent(child)) &&
		  (g_ascii_strcasecmp(tmp1, "") != 0))
		{	
		  tn = g_malloc(sizeof(RubricaCardTelNum));
		  
		  if (xmlHasProp(child, "type"))
		    tmp2 = xmlGetProp(child, "type");
		  else 
		    tmp2 = "???";
		  
		  tn->number = g_strdup(tmp1);
		  tn->type   = g_strdup(tmp2);	  	     
		  
		  tel = g_list_append(tel, tn);
		  
		  child = child->next;
		  if(child && xmlIsBlankNode(child))
		    child = child->next;	      
		}
	      else
		{
		  
		  child = child->next;
		  if(child && xmlIsBlankNode(child))
		    child = child->next;
		}
	    }
	} 
      
      item->phone = tel;
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"TelephoneNumbers");
      
      item->phone = NULL;
    }
}


void 
rubrica_file_card_load_company_data(RubricaCardItem *item, xmlNodePtr node)
{
  xmlNodePtr child;
  gchar *tmp;

  if (node &&
      node->name && 
      g_ascii_strcasecmp(node->name, "Company") == 0)
    {
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.name = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.street = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.number = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.zip = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.city = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.province = ((tmp = xmlNodeGetContent(child)) ? 
				g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.country = ((tmp = xmlNodeGetContent(child)) ? 
			       g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.web = ((tmp = xmlNodeGetContent(child)) ?
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.email = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.operator = ((tmp = xmlNodeGetContent(child)) ? 
				g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.fax = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.green = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.ccare =((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->company.notes = ((tmp = xmlNodeGetContent(child)) ? 
			     g_strdup(tmp) : "");
    }
  else   
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Company");
      
      item->company.name = item->company.street = item->company.number 
	= item->company.zip = item->company.city  = item->company.province
	= item->company.country = item->company.web = item->company.email
	= item->company.operator = item->company.fax = item->company.green 
	= item->company.ccare = item->company.notes = "";
    }
}


void 
rubrica_file_card_load_work_data(RubricaCardItem *item, xmlNodePtr node)
{
  xmlNodePtr child;
  gchar *tmp;

  if (node &&
      node->name && 
      g_ascii_strcasecmp(node->name, "Work") == 0)
    {     
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.assigment = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.org = ((tmp = xmlNodeGetContent(child)) ? g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.dep = ((tmp = xmlNodeGetContent(child)) ? g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.subdep = ((tmp = xmlNodeGetContent(child)) ? 
			g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.secretary = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->work.telephone = ((tmp = xmlNodeGetContent(child)) ? 
			      g_strdup(tmp) : "");
       
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Work");

      item->work.assigment    = item->work.org       = 
	item->work.dep        = item->work.subdep    = 
	item->work.secretary  = item->work.telephone = "";
    }
}


void 
rubrica_file_card_load_note_data(RubricaCardItem *item, xmlNodePtr node)
{
  xmlNodePtr child;
  gchar *tmp;

  if (node &&
      node->name &&
      g_ascii_strcasecmp(node->name, "Notes") == 0)
    {      
      tmp = xmlGetProp(node, "married");
      item->notes.married = ((g_ascii_strcasecmp(tmp, "true") == 0) ? TRUE : FALSE);

      tmp = xmlGetProp(node, "know_birthday");
      item->notes.birth = ((g_ascii_strcasecmp(tmp, "true") == 0) ? TRUE : FALSE);

      tmp = xmlGetProp(node, "know_anniversary");
      item->notes.ann = ((g_ascii_strcasecmp(tmp, "true") == 0) ? TRUE : FALSE);
      
      child = node->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->notes.spouse = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");

      if (item->notes.birth)
	item->notes.bdate = ((tmp = xmlGetProp(child, "birthday")) ? 
			     atol(tmp) : time(NULL));
      else
	item->notes.bdate = time(NULL);

      if (item->notes.ann)    
	item->notes.adate = ((tmp= xmlGetProp(child, "anniversary")) ? 
				atol(tmp) : time(NULL));
      else
	item->notes.adate = time(NULL);	

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->notes.children = ((tmp = xmlNodeGetContent(child)) ? 
			   g_strdup(tmp) : "");

      child = child->next;
       if (xmlIsBlankNode(child))
	child = child->next;
       item->notes.notes = ((tmp = xmlNodeGetContent(child)) ? 
			 g_strdup(tmp) : "");

      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
      item->notes.pubkey = ((tmp = xmlNodeGetContent(child)) ? 
			    g_strdup(tmp) : "");
    } 
  else
    {
      g_warning(_("May be this file is corrupted. Can't read %s field"), 
		"Notes");

      item->notes.bdate   = item->notes.adate    = time(NULL);
      item->notes.married = item->notes.birth    = item->notes.ann = FALSE;
      item->notes.spouse  = item->notes.children =
	item->notes.notes = item->notes.pubkey   = "";
    }
}





