/*
*  Rubrica
*  file: form_int.c
*
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>
#include <glade/glade.h>

#include "rubrica.h"
#include "rubrica_type.h"
#include "form_cb.h"
#include "data_view.h"
#include "form_card.h"
#include "ui.h"
#include "card.h"
#include "card_item.h"
#include "utils.h"
#include "dialog.h"
#include "log.h"

RubricaFormCard* form = NULL;
time_t created, changed; 

GList *grouplist   = NULL;
GList *addresslist = NULL;
GList *netlist     = NULL;
GList *tellist     = NULL;


RubricaOptionType options[] = {
  {"",                 N_("no group"),      NO_GROUP},
  {"favorites.png",    N_("favorites"),     FAVORITES_GROUP},
  {"hot-contacts.png", N_("hot contact"),   HOT_GROUP},
  {"friends.png",      N_("friends"),       FRIENDS_GROUP},
  {"heart.png",        N_("heart"),         HEART_GROUP},
  {"home.png",         N_("relatives"),     RELATIVES_GROUP},
  {"holiday.png",      N_("holidays"),      HOLIDAYS_GROUP},
  {"hobby.png",        N_("hobbies"),       HOBBIES_GROUP},
  {"football.png",     N_("sport"),         SPORT_GROUP},
  {"money.png",        N_("bussines"),      BUSSINES_GROUP},
  {"company2.png",     N_("company"),       COMPANY_GROUP},
  {"genius.png",       N_("work"),          WORK_GROUP},
  {"customer.png",     N_("customer"),      CUSTOMER_GROUP},
  {"strategies.png",   N_("strategies"),    STRATEGIES_GROUP},
  {"penguin.png",      N_("programmers"),   PROGRAMMERS_GROUP},
  {"freebsd.png",      N_("BSD friends"),   BSD_GROUP},
  {"pj.png",           N_("journal"),       PLUTO_GROUP},
  {NULL, NULL}
};


RubricaOptionType rank[] = {
  {"none.png",    N_("None"),       RUBRICA_RANK_NONE},
  {"few.png",     N_("Few"),        RUBRICA_RANK_FEW},
  {"normal.png",  N_("Normal"),     RUBRICA_RANK_NORMAL},
  {"hi.png",      N_("Hight"),      RUBRICA_RANK_HIGHT},
  {"very-hi.png", N_("Very hight"), RUBRICA_RANK_VERY_HIGHT},
  {NULL, NULL}  
};


RubricaOptionType addtype[] = {
  {"", N_("preferred"),     RUBRICA_ADDRESS_PREF},
  {"", N_("home"),          RUBRICA_ADDRESS_HOME},
  {"", N_("work"),          RUBRICA_ADDRESS_WORK},
  {"", N_("parcel"),        RUBRICA_ADDRESS_PARCEL},
  {"", N_("postal"),        RUBRICA_ADDRESS_POSTAL},
  {"", N_("international"), RUBRICA_ADDRESS_INTL},
  {"", N_("domestic"),      RUBRICA_ADDRESS_DOM},
  {NULL, NULL}  
};



RubricaFormCard* 
rubrica_form_card_get_form(void) 
{
  return (RubricaFormCard *) form; 
}

/*
gboolean 
rubrica_form_card_changed(void)
{
  return (gboolean) form->modified;
}


void 
rubrica_form_card_set_changed(gboolean bool)
{
  form->modified = bool;
}


RubricaCardType 
rubrica_form_card_get_form_type(void)
{
  return (RubricaCardType) form->type;
}

void 
rubrica_form_card_set_form_type(RubricaCardType type)
{
  form->type = type;
}
*/


gchar*
rubrica_form_card_get_telephone(void)
{
  return (gchar *) g_strdup(form->cur_tel);
}


void 
rubrica_form_card_set_telephone(gchar *str)
{  
  if (form->cur_tel)
    g_free(form->cur_tel);

  form->cur_tel = g_strdup(str);
}


gchar*    
rubrica_form_card_addtype2str(RubricaAddressType type)
{
  gint len;

  len = sizeof(addtype) / sizeof(addtype[1]);

  if (type > len)
    return N_("unknowk");
  
  return addtype[type].label;
}

RubricaAddressType 
rubrica_form_card_str2addtype (gchar* str)
{
  RubricaOptionType* add = addtype;
  
  while (add)
    {
      if (g_ascii_strcasecmp(str, add->label) == 0)
	return add->type;
	  
      add++;
    }

  return RUBRICA_ADDRESS_UNKNOWN;
}


gchar*
rubrica_form_card_get_option_label(gint index)
{
  gint option_len;

  if (index < 0)
    return g_strdup("");

  option_len = rubrica_utils_option_len(options);

  if (index <= option_len)
    return g_strdup(options[index].label);

  return g_strdup("");
}


gchar*
rubrica_form_card_get_option_pixmap(gint index)
{
  gint option_len;  

  option_len = rubrica_utils_option_len(options);

  if ((index > 0) && (index < option_len))
    return (gchar *) options[index].pixmap;
  
  return NULL;
}

RubricaGroupType 
rubrica_form_card_get_option_type(gchar *label)
{
  gint option_len, i;
  
  g_return_val_if_fail(label != NULL, NO_GROUP);

  if (strcmp(label, "") == 0)
    return NO_GROUP;

  i = 0;
  option_len = rubrica_utils_option_len(options);

  while(i < option_len)
    {
      if (strcmp(options[i].label, label) == 0)
	return (RubricaGroupType) options[i].type;
      
      i++;
    }

  return (RubricaGroupType) NO_GROUP;
}


void 
rubrica_form_card_populate_model(GtkTreeModel* model)
{
  GtkTreeIter iter;  
  RubricaOptionType* op = options;

  while (op->label)
    {
      GdkPixbuf* pixbuf;

      pixbuf = rubrica_ui_pixbuf_new(op->pixmap);

      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 GROUP_BELONG_COLUMN, FALSE, 
			 GROUP_PIXMAP_COLUMN, pixbuf, 
			 GROUP_LABEL_COLUMN,  op->label,
			 GROUP_TYPE_COLUMN,   op->type,
			 -1);

      op++;
    }
  
}

GtkTreeModel *
rubrica_form_card_create_model(RubricaModelType type)
{
  GtkListStore *model = NULL;
  
  switch (type)
    {
    case GROUP_MODEL:
      model = gtk_list_store_new(GROUP_LAST_COLUMN,
				 G_TYPE_INT,          // check (belong) column
				 GDK_TYPE_PIXBUF,     // group pixbuf  
				 G_TYPE_STRING,       // group label
				 G_TYPE_INT);         // group type
      break;

    case ADDRESS_MODEL:
      model = gtk_list_store_new(LAST_ADDRESS_COLUMN, 
				 G_TYPE_STRING,      // street
				 G_TYPE_STRING,      // street number
				 G_TYPE_STRING,      // address type
				 G_TYPE_POINTER);    // pointer to address
      break;
    case NET_MODEL:
      model = gtk_list_store_new(LAST_NET_COLUMN, 
				 G_TYPE_STRING);     // web/email url
      break;

    case TELEPHONE_MODEL:
      model = gtk_list_store_new(LAST_TEL_COLUMN, 
				 G_TYPE_STRING,      // telephone number
				 G_TYPE_STRING);     // telephone type
      break;
      
    default:
      break;
    }

  return GTK_TREE_MODEL(model);
}



void 
rubrica_form_card_add_columns(GtkTreeView *view, RubricaColumnsType type)
{
  GtkTreeViewColumn* column;
  GtkCellRenderer *renderer;
  GtkTreeModel *model;
    
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));; 

  switch(type)
    {
    case GROUP_COLUMNS:      
      renderer = gtk_cell_renderer_toggle_new();
      gtk_tree_view_insert_column_with_attributes(view, -1, 
						  _("Belongs"), 
						  renderer, "active", 
						  GROUP_BELONG_COLUMN,
						  NULL); 
      g_signal_connect(G_OBJECT(renderer), "toggled", 
		       G_CALLBACK(rubrica_form_card_toggle_group), model); 
      
      column = gtk_tree_view_column_new();
      gtk_tree_view_column_set_title(column, _("Groups"));

      renderer = gtk_cell_renderer_pixbuf_new();
      gtk_tree_view_column_pack_start(column, renderer, FALSE);
      gtk_tree_view_column_set_attributes (column, renderer, 
					   "pixbuf", GROUP_PIXMAP_COLUMN,
					   NULL);
      
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_column_pack_start(column, renderer, TRUE);
      gtk_tree_view_column_set_attributes (column, renderer, 
					   "text", GROUP_LABEL_COLUMN,
					   NULL);
      
      gtk_tree_view_append_column (GTK_TREE_VIEW(view), column);
      break;

    case ADDRESS_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, /* tree view */
						   -1,  _("Street"),
						   renderer, "text",      
						   ADDRESS_STREET_COLUMN,  
						   NULL);

      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  _("Number"),
						   renderer, "text",      
						   ADDRESS_NUMBER_COLUMN,  
						   NULL);

      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  _("Address type"),
						   renderer, "text",      
						   ADDRESS_TYPE_COLUMN,  
						   NULL);      
      break;

    case WEB_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  _("Web URL"),
						   renderer, "text",      
						   URI_COLUMN,  
						   NULL);       
      break;

    case EMAIL_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, -1, _("Email URL"),
						   renderer, "text", 
						   URI_COLUMN, NULL);
      break;

    case TELEPHONE_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, -1, _("Number"), 
						   renderer, "text", 
						   NUMBER_COLUMN, 
						   NULL);   
      
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, -1, _("Type"), 
						   renderer, "text", 
						   TYPE_COLUMN, 
						   NULL);     
      break;

    default:
      break;
    }
}



RubricaFormCard *
rubrica_form_card_create_form(void)
{
  GladeXML*  xml;  

  xml = glade_xml_new (PACKAGE_DATA_DIR"/rubrica/ui/card.glade", 
		       NULL, NULL);
  
  if (!xml)
    {
      gchar* msg;

      msg = g_strdup_printf("Can't load gui: %s%s", PACKAGE_DATA_DIR, 
			    "/rubrica/ui/card.glade");
      
      rubrica_log_write_log(msg); 
      rubrica_dialog_error(msg);
      
      g_free(msg);
      
      return NULL;
    }
  
  form = (RubricaFormCard*) g_malloc(sizeof(RubricaFormCard));
  if (!form)
    {
      g_error("\nOut of memory. Can't allocate enougth memory");
    }

  form->dialog           = glade_xml_get_widget(xml, "CardForm");
  form->notebook         = glade_xml_get_widget(xml, "notebook");

  form->card             = glade_xml_get_widget(xml, "card");
  form->delete           = glade_xml_get_widget(xml, "check_del");
  form->rank             = glade_xml_get_widget(xml, "rank");
  form->group            = glade_xml_get_widget(xml, "grouptree");

  form->personal.first   = glade_xml_get_widget(xml, "fn");
  form->personal.middle  = glade_xml_get_widget(xml, "sn");
  form->personal.last    = glade_xml_get_widget(xml, "ln");
  form->personal.nick    = glade_xml_get_widget(xml, "nick");
  form->personal.prof    = glade_xml_get_widget(xml, "prof");
  form->personal.prefix  = glade_xml_get_widget(xml, "prefix");
  form->personal.title   = glade_xml_get_widget(xml, "title");
  form->personal.birth   = glade_xml_get_widget(xml, "birth");
  form->personal.date    = glade_xml_get_widget(xml, "birth_date");
  form->personal.photo   = glade_xml_get_widget(xml, "photo");

  form->address.type     = glade_xml_get_widget(xml, "address_type");
  form->address.street   = glade_xml_get_widget(xml, "st");
  form->address.number   = glade_xml_get_widget(xml, "num");
  form->address.zip      = glade_xml_get_widget(xml, "zip");
  form->address.city     = glade_xml_get_widget(xml, "city");
  form->address.province = glade_xml_get_widget(xml, "prov");
  form->address.state    = glade_xml_get_widget(xml, "state");
  form->address.country  = glade_xml_get_widget(xml, "country");
  form->address.tree     = glade_xml_get_widget(xml, "addresses_tree");
  form->address.add      = glade_xml_get_widget(xml, "add_add");
  form->address.mod      = glade_xml_get_widget(xml, "add_mod");
  form->address.del      = glade_xml_get_widget(xml, "add_del");

  form->net.web          = glade_xml_get_widget(xml, "web");
  form->net.wtree        = glade_xml_get_widget(xml, "web_tree");
  form->net.wadd         = glade_xml_get_widget(xml, "web_add");
  form->net.wmod         = glade_xml_get_widget(xml, "web_mod");
  form->net.wdel         = glade_xml_get_widget(xml, "web_del");

  form->net.email        = glade_xml_get_widget(xml, "email");
  form->net.etree        = glade_xml_get_widget(xml, "email_tree");
  form->net.eadd         = glade_xml_get_widget(xml, "email_add");
  form->net.emod         = glade_xml_get_widget(xml, "email_mod");
  form->net.edel         = glade_xml_get_widget(xml, "email_del");

  form->tel.number       = glade_xml_get_widget(xml, "tel");
  form->tel.home         = glade_xml_get_widget(xml, "home");
  form->tel.work         = glade_xml_get_widget(xml, "work");
  form->tel.fax          = glade_xml_get_widget(xml, "fax");
  form->tel.cell         = glade_xml_get_widget(xml, "cell");
  form->tel.other        = glade_xml_get_widget(xml, "other");
  form->tel.tree         = glade_xml_get_widget(xml, "tel_tree");
  form->tel.add          = glade_xml_get_widget(xml, "tel_add");
  form->tel.mod          = glade_xml_get_widget(xml, "tel_mod");
  form->tel.del          = glade_xml_get_widget(xml, "tel_del");

  form->company.name     = glade_xml_get_widget(xml, "cname");
  form->company.street   = glade_xml_get_widget(xml, "cst");
  form->company.number   = glade_xml_get_widget(xml, "cn");
  form->company.zip      = glade_xml_get_widget(xml, "czip");
  form->company.city     = glade_xml_get_widget(xml, "ccity");
  form->company.province = glade_xml_get_widget(xml, "cprov");
  form->company.country  = glade_xml_get_widget(xml, "ccount");
  form->company.web      = glade_xml_get_widget(xml, "cweb");
  form->company.email    = glade_xml_get_widget(xml, "cemail");
  form->company.operator = glade_xml_get_widget(xml, "cop");
  form->company.fax      = glade_xml_get_widget(xml, "cfax");
  form->company.green    = glade_xml_get_widget(xml, "cgreen");
  form->company.ccare    = glade_xml_get_widget(xml, "ccare");
  form->company.notes    = glade_xml_get_widget(xml, "cnote");

  form->work.role        = glade_xml_get_widget(xml, "role");
  form->work.dep         = glade_xml_get_widget(xml, "dep");
  form->work.sub         = glade_xml_get_widget(xml, "subdep");
  form->work.org         = glade_xml_get_widget(xml, "org");
  form->work.secretary   = glade_xml_get_widget(xml, "secr");
  form->work.phone       = glade_xml_get_widget(xml, "secphone");

  form->notes.married    = glade_xml_get_widget(xml, "married");
  form->notes.birth      = glade_xml_get_widget(xml, "sbirth");
  form->notes.bdate      = glade_xml_get_widget(xml, "spouse_birth");
  form->notes.ann        = glade_xml_get_widget(xml, "have_anniv");
  form->notes.adate      = glade_xml_get_widget(xml, "anniversary");
  form->notes.spouse     = glade_xml_get_widget(xml, "spouse");
  form->notes.children   = glade_xml_get_widget(xml, "child");
  form->notes.notes      = glade_xml_get_widget(xml, "note");
  form->notes.pubkey     = glade_xml_get_widget(xml, "pubkey");

  form->ok               = glade_xml_get_widget(xml, "okbutton");
  form->cancel           = glade_xml_get_widget(xml, "cancelbutton");
  form->cur_tel          = g_strdup("home");

  rubrica_form_card_init_form();

  return form;
}


void 
rubrica_form_card_init_form(void)
{
  GtkTreeModel *model;
  GtkTreeSelection *selection;

  rubrica_ui_option_menu_new(form->rank,  rank);
  rubrica_ui_option_menu_new(form->address.type, addtype);

  RUBRICA_SET_RANK (form->rank, RUBRICA_RANK_NORMAL);
  RUBRICA_SET_ADDRESS_TYPE(form->address.type, RUBRICA_ADDRESS_HOME);

  /* configure check button and related data widget */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->personal.birth), FALSE);
  gtk_widget_set_sensitive(form->personal.date, FALSE);
  
  g_signal_connect(G_OBJECT(form->personal.birth), "toggled", 
		   G_CALLBACK(rubrica_form_card_option_check_toggled),
		   form->personal.date); 
  /*    GROUPS
   */
  model = rubrica_form_card_create_model(GROUP_MODEL);
  rubrica_form_card_populate_model(model);
  gtk_tree_view_set_model(GTK_TREE_VIEW(form->group), GTK_TREE_MODEL(model));

  rubrica_form_card_add_columns(GTK_TREE_VIEW(form->group), GROUP_COLUMNS);
  g_object_unref(model);

  /*    ADDRESSES 
   */  
  /* make unsensitive the address's modify and remove buttons */
  gtk_widget_set_sensitive(GTK_WIDGET(form->address.mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->address.del), FALSE); 

  /* address model/view */
  model = rubrica_form_card_create_model(ADDRESS_MODEL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(form->address.tree), 
			  GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->address.tree));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  rubrica_form_card_add_columns(GTK_TREE_VIEW(form->address.tree), 
				ADDRESS_COLUMNS);

  /* address selection callback 
   */
  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(rubrica_form_card_addresses_list_activated), 
		   form);
  g_object_unref(model);

  /*    WEB URLS
   */  
  /* make unsensitive the web's modify and remove buttons */
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.wmod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.wdel), FALSE); 

  /* web urls model/view */
  model = rubrica_form_card_create_model(NET_MODEL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(form->net.wtree), 
			  GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->net.wtree));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  rubrica_form_card_add_columns(GTK_TREE_VIEW(form->net.wtree), WEB_COLUMNS);

  /* web selection callback 
   */
  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(rubrica_form_card_web_list_activated), form);
  g_object_unref(model);
  
  /*    EMAIL URLS
   */
  /* make unsensitive the email's modify and remove buttons */
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.emod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.edel), FALSE); 

  /* email urls model/view */
  model = rubrica_form_card_create_model(NET_MODEL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(form->net.etree), 
			  GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->net.etree));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  rubrica_form_card_add_columns(GTK_TREE_VIEW(form->net.etree), EMAIL_COLUMNS);
  
  /* email selection callback 
   */
  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(rubrica_form_card_email_list_activated), form);
  g_object_unref(model);


  /*    TELEPHONE NUMBERS
  */
  /* make unsensitive the telephone's modify and remove buttons */
  gtk_widget_set_sensitive(GTK_WIDGET(form->tel.mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->tel.del), FALSE);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (form->tel.home), TRUE);

  /* telephone model/view */
  model = rubrica_form_card_create_model(TELEPHONE_MODEL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(form->tel.tree), 
			  GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->tel.tree));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE); 

  rubrica_form_card_add_columns(GTK_TREE_VIEW(form->tel.tree), 
				TELEPHONE_COLUMNS);

  /* telephone selection callback 
   */
  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(rubrica_form_card_telephone_list_activated),
		   form);
  g_object_unref(model);


  /* notes */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->notes.married), FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->notes.birth),   FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->notes.ann),     FALSE);

  gtk_widget_set_sensitive(form->notes.bdate, FALSE);
  gtk_widget_set_sensitive(form->notes.adate, FALSE);
  gtk_widget_set_sensitive(form->notes.birth, FALSE);
  gtk_widget_set_sensitive(form->notes.ann,   FALSE);


  /*    OK/CANCEL CALLBACKS
   */
  g_signal_connect(G_OBJECT(form->dialog), "destroy", 
		   G_CALLBACK(rubrica_form_card_close_form), NULL); 

  /*
  g_signal_connect(G_OBJECT(form->ok), "clicked",
		   G_CALLBACK(rubrica_form_card_add_item), NULL);
  */    

  g_signal_connect(G_OBJECT(form->cancel), "clicked",
		   G_CALLBACK(rubrica_form_card_close_form), NULL);

  /*    callbacks for:
	address's entries,
	web's entry , 
	email's entry  
   */
  g_signal_connect (G_OBJECT (form->address.street), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.street); 
  g_signal_connect (G_OBJECT (form->address.number), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.number); 
  g_signal_connect (G_OBJECT (form->address.zip), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.zip); 
  g_signal_connect (G_OBJECT (form->address.city), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.city); 
  g_signal_connect (G_OBJECT (form->address.province), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.province); 
  g_signal_connect (G_OBJECT (form->address.state), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.state); 
  g_signal_connect (G_OBJECT (form->address.country), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->address.country); 

  g_signal_connect (G_OBJECT (form->net.web), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->net.eadd); 
  g_signal_connect (G_OBJECT (form->net.email), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->net.eadd); 

  /*    bottoni address
  */
  g_signal_connect (G_OBJECT (form->address.add), "clicked", 
		    G_CALLBACK (rubrica_form_card_address_add_clicked), form); 
  g_signal_connect (G_OBJECT (form->address.mod), "clicked", 
		    G_CALLBACK (rubrica_form_card_address_mod_clicked), form); 
  g_signal_connect (G_OBJECT (form->address.del), "clicked", 
		    G_CALLBACK (rubrica_form_card_address_del_clicked), form); 

  /*    bottoni web
  */
  g_signal_connect (G_OBJECT (form->net.wadd), "clicked", 
		    G_CALLBACK (rubrica_form_card_web_add_clicked), form); 
  g_signal_connect (G_OBJECT (form->net.wmod), "clicked", 
		    G_CALLBACK(rubrica_form_card_web_mod_clicked), form); 
  g_signal_connect (G_OBJECT (form->net.wdel), "clicked", 
		    G_CALLBACK (rubrica_form_card_web_del_clicked), form); 

  /*    bottoni mail
  */
  g_signal_connect (G_OBJECT (form->net.eadd), "clicked",
		    G_CALLBACK (rubrica_form_card_email_add_clicked), form);
  g_signal_connect (G_OBJECT (form->net.emod), "clicked",
		    G_CALLBACK (rubrica_form_card_email_mod_clicked), form);
  g_signal_connect (G_OBJECT (form->net.edel), "clicked",
		    G_CALLBACK (rubrica_form_card_email_del_clicked), form);

  /*    Telephone entry/buttons callback
  */
  g_signal_connect (G_OBJECT (form->tel.number), "activate", 
		    G_CALLBACK (rubrica_form_card_entry_activate),
		    form->tel.add);

  g_signal_connect (G_OBJECT (form->tel.add), "clicked", 
		    G_CALLBACK (rubrica_form_card_tel_add_clicked), form); 
  g_signal_connect (G_OBJECT (form->tel.mod), "clicked", 
		    G_CALLBACK (rubrica_form_card_tel_mod_clicked), form); 
  g_signal_connect (G_OBJECT (form->tel.del), "clicked", 
		    G_CALLBACK (rubrica_form_card_tel_del_clicked), form); 

  /*    Notes Callback
  */
  g_signal_connect(G_OBJECT(form->notes.married), "toggled", 
		   G_CALLBACK(rubrica_form_card_married_check_toggled), form); 

  g_signal_connect(G_OBJECT(form->notes.birth), "toggled", 
		   G_CALLBACK(rubrica_form_card_option_check_toggled), 
		   form->notes.bdate); 

  g_signal_connect(G_OBJECT(form->notes.ann), "toggled", 
		   G_CALLBACK(rubrica_form_card_option_check_toggled), 
		   form->notes.adate); 
}




void 
rubrica_form_card_clean_form(void)
{
  GtkTreeModel *model;
  
  gtk_entry_set_text(GTK_ENTRY(form->card), "");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->delete), FALSE);

  RUBRICA_SET_RANK (form->rank, RUBRICA_RANK_NORMAL);
  RUBRICA_SET_ADDRESS_TYPE(form->address.type, RUBRICA_ADDRESS_HOME);

  gtk_entry_set_text(GTK_ENTRY(form->personal.first) , "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.middle), "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.last),   "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.nick),   "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.prof),   "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.prefix), "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.title),  "");
  gtk_entry_set_text(GTK_ENTRY(form->personal.photo),  "");

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->personal.birth), FALSE);

  gnome_date_edit_set_time(GNOME_DATE_EDIT(form->personal.date), time(NULL));
  gtk_widget_set_sensitive(form->personal.date, FALSE);
  
  gtk_entry_set_text(GTK_ENTRY(form->address.street),   "");
  gtk_entry_set_text(GTK_ENTRY(form->address.number),   "");
  gtk_entry_set_text(GTK_ENTRY(form->address.zip),      "");
  gtk_entry_set_text(GTK_ENTRY(form->address.city),     "");
  gtk_entry_set_text(GTK_ENTRY(form->address.province), "");
  gtk_entry_set_text(GTK_ENTRY(form->address.state),    "");
  gtk_entry_set_text(GTK_ENTRY(form->address.country),  "");

  gtk_entry_set_text(GTK_ENTRY(form->net.web),   "");
  gtk_entry_set_text(GTK_ENTRY(form->net.email), "");

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->group));
  gtk_tree_model_foreach(GTK_TREE_MODEL(model), 
			 rubrica_form_card_uncheck_all, NULL);

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->address.tree));
  gtk_list_store_clear(GTK_LIST_STORE(model));

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.wtree));
  gtk_list_store_clear(GTK_LIST_STORE(model));

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.etree));
  gtk_list_store_clear(GTK_LIST_STORE(model));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.home), TRUE);
  gtk_entry_set_text(GTK_ENTRY(form->tel.number), "");

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->tel.tree));
  gtk_list_store_clear(GTK_LIST_STORE(model));

  gtk_entry_set_text(GTK_ENTRY(form->company.name),     "");
  gtk_entry_set_text(GTK_ENTRY(form->company.street),   "");
  gtk_entry_set_text(GTK_ENTRY(form->company.number),   "");
  gtk_entry_set_text(GTK_ENTRY(form->company.zip),      "");
  gtk_entry_set_text(GTK_ENTRY(form->company.city),     "");
  gtk_entry_set_text(GTK_ENTRY(form->company.province), "");
  gtk_entry_set_text(GTK_ENTRY(form->company.country),  "");
  gtk_entry_set_text(GTK_ENTRY(form->company.web),      "");
  gtk_entry_set_text(GTK_ENTRY(form->company.email),    "");
  gtk_entry_set_text(GTK_ENTRY(form->company.operator), "");
  gtk_entry_set_text(GTK_ENTRY(form->company.fax),      "");
  gtk_entry_set_text(GTK_ENTRY(form->company.green),    "");
  gtk_entry_set_text(GTK_ENTRY(form->company.ccare),    "");
  rubrica_ui_clean_text_view(form->company.notes);

  gtk_entry_set_text(GTK_ENTRY(form->work.role),      "");
  gtk_entry_set_text(GTK_ENTRY(form->work.dep),       "");
  gtk_entry_set_text(GTK_ENTRY(form->work.sub),       "");
  gtk_entry_set_text(GTK_ENTRY(form->work.org),       "");
  gtk_entry_set_text(GTK_ENTRY(form->work.secretary), "");
  gtk_entry_set_text(GTK_ENTRY(form->work.phone),     "");

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->notes.married), FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->notes.birth),   FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->notes.ann),     FALSE);

  gnome_date_edit_set_time(GNOME_DATE_EDIT(form->notes.bdate), time(NULL));
  gnome_date_edit_set_time(GNOME_DATE_EDIT(form->notes.adate), time(NULL));

  gtk_widget_set_sensitive(form->notes.birth, FALSE);
  gtk_widget_set_sensitive(form->notes.ann,   FALSE);
  gtk_widget_set_sensitive(form->notes.bdate, FALSE);
  gtk_widget_set_sensitive(form->notes.adate, FALSE);

  gtk_entry_set_text(GTK_ENTRY(form->notes.spouse), "");
  rubrica_ui_clean_text_view(form->notes.children);
  rubrica_ui_clean_text_view(form->notes.notes);
  rubrica_ui_clean_text_view(form->notes.pubkey);

  /* set notebook page to first one
   */
  gtk_notebook_set_current_page(GTK_NOTEBOOK(form->notebook), 0);
}



/*   funzioni per manipolare i dati contenuti nel widget

     estrae i dati contenuti nella finestra di dialogo e 
     li salva nella rubrica
*/
RubricaCardItem *
rubrica_form_card_get_item(RubricaCardType type)
{
  RubricaCardItem *item;
  RubricaCardAddress* address;

  item = rubrica_card_item_new();  
  if (!item) 
    {
      gchar *msg;

      msg = g_strdup("\nrubrica_form_card_get_item()\ncan't allocate memory.");

      rubrica_log_write_log(msg);
      rubrica_dialog_error(msg);
      g_free(msg);
	
      return NULL;
    }
  if (type == OLD_CARD)
    rubrica_card_item_reject_id(item);

  address = (RubricaCardAddress *) g_malloc(sizeof(RubricaCardAddress));

  /*    Card data 
   */
  item->card      = rubrica_form_card_get_data(form->card);
  item->deletable = RUBRICA_GET_CHECK_STATUS(form->delete);
  item->rank      = RUBRICA_GET_RANK(form->rank);

  item->personal.first  = rubrica_form_card_get_data(form->personal.first);
  item->personal.middle = rubrica_form_card_get_data(form->personal.middle);
  item->personal.last   = rubrica_form_card_get_data(form->personal.last);
  item->personal.nick   = rubrica_form_card_get_data(form->personal.nick);
  item->personal.prof   = rubrica_form_card_get_data(form->personal.prof);
  item->personal.prefix = rubrica_form_card_get_data(form->personal.prefix);
  item->personal.title  = rubrica_form_card_get_data(form->personal.title);
  item->personal.photo  = rubrica_form_card_get_data(form->personal.photo);

  item->personal.birth = RUBRICA_GET_CHECK_STATUS(form->personal.birth);
  item->personal.bdate = RUBRICA_GET_DATE(form->personal.date);

  /*    Company data
  */
  item->company.name     = rubrica_form_card_get_data(form->company.name);
  item->company.street   = rubrica_form_card_get_data(form->company.street); 
  item->company.number   = rubrica_form_card_get_data(form->company.number); 
  item->company.zip      = rubrica_form_card_get_data(form->company.zip); 
  item->company.city     = rubrica_form_card_get_data(form->company.city); 
  item->company.province = rubrica_form_card_get_data(form->company.province); 
  item->company.country  = rubrica_form_card_get_data(form->company.country); 
  item->company.web      = rubrica_form_card_get_data(form->company.web); 
  item->company.email    = rubrica_form_card_get_data(form->company.email);
  item->company.operator = rubrica_form_card_get_data(form->company.operator); 
  item->company.fax      = rubrica_form_card_get_data(form->company.fax); 
  item->company.green    = rubrica_form_card_get_data(form->company.green);
  item->company.ccare    = rubrica_form_card_get_data(form->company.ccare); 
  item->company.notes    = rubrica_form_card_get_text(form->company.notes);

  /*     Work data
  */
  item->work.assigment    = rubrica_form_card_get_data(form->work.role);
  item->work.org          = rubrica_form_card_get_data(form->work.org); 
  item->work.dep          = rubrica_form_card_get_data(form->work.dep);
  item->work.subdep       = rubrica_form_card_get_data(form->work.sub);
  item->work.secretary    = rubrica_form_card_get_data(form->work.secretary);
  item->work.telephone    = rubrica_form_card_get_data(form->work.phone);


  /*    Personal notes
  */
  item->notes.married = RUBRICA_GET_CHECK_STATUS(form->notes.married);
  item->notes.birth   = RUBRICA_GET_CHECK_STATUS(form->notes.birth);
  item->notes.bdate   = item->notes.birth ? 
    RUBRICA_GET_DATE(form->notes.bdate) : time(NULL);
  
  item->notes.ann = RUBRICA_GET_CHECK_STATUS(form->notes.ann);
  item->notes.adate = item->notes.ann ?
    RUBRICA_GET_DATE(form->notes.adate) : time(NULL);

  item->notes.spouse   = rubrica_form_card_get_data(form->notes.spouse);
  item->notes.children = rubrica_form_card_get_text(form->notes.children);
  item->notes.notes    = rubrica_form_card_get_text(form->notes.notes);
  item->notes.pubkey   = rubrica_form_card_get_text(form->notes.pubkey);

  /* estrazione dei dati dalle liste, web, mail e telefonica 
   */
  
  /* group */
  item->groups = rubrica_form_card_get_all_data(form->group, GROUP_TREE);

  /* Addresses */ 
  item->addresses = rubrica_form_card_get_all_data(form->address.tree,
						   ADDRESS_TREE);

  /* web */
  item->web = rubrica_form_card_get_all_data(form->net.wtree, NET_TREE);
  
  /* mail */
  item->email = rubrica_form_card_get_all_data(form->net.etree, NET_TREE);
  
  /* telephone */
  item->phone = rubrica_form_card_get_all_data(form->tel.tree,
					       PHONE_TREE);
  
  return (RubricaCardItem *) item;   
}

/*
    visualizza i dati contenuti nell'elemento di 
    rubrica, nella finestra di dialogo
*/
RubricaErrorType 
rubrica_form_card_insert_item(RubricaCardItem *item, gint index)
{
  GList* alias;
  GtkTreeModel *model;
  GtkTreeIter iter;

  created     = item->created;
  changed     = item->changed;
  form->index = index;

  /*     Card data 
  */  
  gtk_entry_set_text(GTK_ENTRY(form->card), item->card);
  RUBRICA_SET_CHECK_STATUS(form->delete, item->deletable);
  RUBRICA_SET_RANK(form->rank, item->rank);

  /*    Personal data
  */
  gtk_entry_set_text(GTK_ENTRY(form->personal.first),  item->personal.first);
  gtk_entry_set_text(GTK_ENTRY(form->personal.middle), item->personal.middle);
  gtk_entry_set_text(GTK_ENTRY(form->personal.last),   item->personal.last);
  gtk_entry_set_text(GTK_ENTRY(form->personal.nick),   item->personal.nick);
  gtk_entry_set_text(GTK_ENTRY(form->personal.prof),   item->personal.prof);
  gtk_entry_set_text(GTK_ENTRY(form->personal.prefix), item->personal.prefix);
  gtk_entry_set_text(GTK_ENTRY(form->personal.photo),  item->personal.photo);

  RUBRICA_SET_CHECK_STATUS(form->personal.birth, item->personal.birth);
  RUBRICA_SET_DATE(form->personal.date, item->personal.bdate);

  if (!item->personal.birth)
    gtk_widget_set_sensitive(form->personal.date, FALSE);
  
  /* company */
  gtk_entry_set_text(GTK_ENTRY(form->company.name),    item->company.name);
  gtk_entry_set_text(GTK_ENTRY(form->company.street),  item->company.street);
  gtk_entry_set_text(GTK_ENTRY(form->company.number),  item->company.number);
  gtk_entry_set_text(GTK_ENTRY(form->company.zip),     item->company.zip);
  gtk_entry_set_text(GTK_ENTRY(form->company.city),    item->company.city);
  gtk_entry_set_text(GTK_ENTRY(form->company.province),item->company.province);
  gtk_entry_set_text(GTK_ENTRY(form->company.country), item->company.country);
  gtk_entry_set_text(GTK_ENTRY(form->company.web),     item->company.web);
  gtk_entry_set_text(GTK_ENTRY(form->company.email),   item->company.email);
  gtk_entry_set_text(GTK_ENTRY(form->company.operator),item->company.operator);
  gtk_entry_set_text(GTK_ENTRY(form->company.fax),     item->company.fax);
  gtk_entry_set_text(GTK_ENTRY(form->company.green),   item->company.green);
  gtk_entry_set_text(GTK_ENTRY(form->company.ccare),   item->company.ccare);
  rubrica_ui_set_text_in_view(form->company.notes,     item->company.notes);

  /* work */
  gtk_entry_set_text(GTK_ENTRY(form->work.role),       item->work.assigment);
  gtk_entry_set_text(GTK_ENTRY(form->work.org),        item->work.org);
  gtk_entry_set_text(GTK_ENTRY(form->work.dep),        item->work.dep);
  gtk_entry_set_text(GTK_ENTRY(form->work.sub),        item->work.subdep);
  gtk_entry_set_text(GTK_ENTRY(form->work.secretary),  item->work.secretary);
  gtk_entry_set_text(GTK_ENTRY(form->work.phone),      item->work.telephone);

  /* notes */
  RUBRICA_SET_CHECK_STATUS(form->notes.married, item->notes.married);
  RUBRICA_SET_CHECK_STATUS(form->notes.birth,   item->notes.birth);
  RUBRICA_SET_CHECK_STATUS(form->notes.ann,     item->notes.ann);

  gtk_entry_set_text(GTK_ENTRY(form->notes.spouse), item->notes.spouse);
  rubrica_ui_set_text_in_view(form->notes.children, item->notes.children);
  rubrica_ui_set_text_in_view(form->notes.notes,    item->notes.notes);
  rubrica_ui_set_text_in_view(form->notes.pubkey,   item->notes.pubkey);
  
  gtk_widget_set_sensitive(form->notes.spouse,      item->notes.married);
  gtk_widget_set_sensitive(form->notes.birth,       item->notes.married);
  gtk_widget_set_sensitive(form->notes.ann,         item->notes.married);
  gtk_widget_set_sensitive(form->notes.bdate,       item->notes.birth);
  gtk_widget_set_sensitive(form->notes.adate,       item->notes.ann);


  /*    groups
  */
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->group));
  alias = item->groups;
  for (; alias; alias = g_list_next(alias))
    {
      RubricaGroup* group;
      
      if ((group = rubrica_card_item_get_group(alias)))
	{
	  RubricaGroupType type;

	  type = rubrica_card_item_get_group_type(group);
	  gtk_tree_model_foreach(GTK_TREE_MODEL(model),
				 rubrica_form_card_enable_group,
				 GINT_TO_POINTER(type));
	}
    }

  /*    address
  */
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->address.tree));
  alias = item->addresses;
  for(; alias; alias = g_list_next(alias))
    {
      RubricaCardAddress* address;

      if ((address = alias->data))
	{	  
	  gchar *type = addtype[address->type].label;

	  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
	  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			     ADDRESS_STREET_COLUMN, address->street,
			     ADDRESS_NUMBER_COLUMN, address->number,
			     ADDRESS_TYPE_COLUMN,   type,
			     ADDRESS_DATA_COLUMN,   address,
			     -1); 
	}
    }


  /*    web 
   */
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.wtree));
  alias = item->web;
  for (; alias; alias = g_list_next(alias))
    {     
      gchar *str;

      str = (gchar*) alias->data ? alias->data : "";

      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, URI_COLUMN, str, -1); 
    } 
  

  /*    email
   */
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.etree));
  alias = item->email;
  for (; alias; alias = g_list_next(alias))
    {
      gchar *str; 
      
      str = (gchar*) alias->data ? alias->data : "";

      gtk_list_store_append(GTK_LIST_STORE(model), &iter); 
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, URI_COLUMN, str, -1);
    }  
  
  /*    Telefono
   */
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->tel.tree));
  alias = item->phone;
  for (; alias; alias = g_list_next(alias))
    {      
      RubricaCardTelNum *tn; 

      if ((tn = alias->data))
	{
	  gtk_list_store_append(GTK_LIST_STORE(model), &iter); 
	  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			     NUMBER_COLUMN, tn->number, -1);      
	  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			     TYPE_COLUMN, tn->type, -1);	  
	}
    }  

  return RE_NO_ERROR;
}

void 
rubrica_form_card_close_form(void)
{
  gtk_widget_hide(GTK_WIDGET(form->dialog));
}


RubricaCardItem* 
rubrica_form_card_add_item(void)
{
  RubricaDoc *rubrica;
  RubricaCardItem *item;
  
  rubrica = rubrica_folder_get_current_document();
    
  item = rubrica_form_card_get_item(NEW_CARD);
  if (item)
    {
      item->created = item->changed = time(NULL); 
      rubrica_doc_append_item(rubrica, item); 
    }
  
  rubrica_form_card_close_form();

  return (RubricaCardItem*) item;
}

RubricaCardItem*
rubrica_form_card_modify_item(RubricaDoc *doc, RubricaCardItem* old)
{
  RubricaFormCard* form = NULL;
  RubricaCardItem* item;
  gulong id;
  gint response;

  if ((form = rubrica_form_card_get_form()) == NULL)
    form = rubrica_form_card_create_form();  
  
  id = rubrica_card_item_get_id(old);
  rubrica_form_card_clean_form();
  rubrica_form_card_insert_item(old, doc->index);
  
  response = gtk_dialog_run(GTK_DIALOG(form->dialog));  
  switch(response)
    {
    case GTK_RESPONSE_OK:
      item = rubrica_form_card_get_item(OLD_CARD);
      item->id = id;
      item->created = created;
      item->changed = time(NULL);
  
      rubrica_form_card_close_form();      

      return (RubricaCardItem *) item;
//      rubrica_doc_modify_item(rubrica, item);
//      rubrica_card_modify_item(rubrica, item, form->index);
      break;
      
    case GTK_RESPONSE_CLOSE:
      rubrica_form_card_close_form();      
      break;
      
    default: 
      break;
    }

  return NULL;
}


/**
 * rubrica_form_card_get_data
 * @entrywidget: the text widget pointer
 * 
 * extract text stored into the entry widget. 
 *
 * return:  
 */
gchar*
rubrica_form_card_get_data(GtkWidget *entry)
{
  g_return_val_if_fail(GTK_IS_ENTRY(entry), g_strdup(""));

  if (!gtk_entry_get_text(GTK_ENTRY(entry))) 
    return g_strdup("");

  return g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));     
}


/**
 * rubrica_form_card_get_text
 * @textwidget: the text widget pointer
 * 
 * extract text stored into the text widget. 
 *
 * return: an allocated UTF-8 string 
 */
gchar*
rubrica_form_card_get_text(GtkWidget *text)
{
  GtkTextIter start, end;
  GtkTextBuffer *text_buffer;
  gchar *ret_val;  /* an allocated UTF-8 string */
  
  text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (text));
  
  gtk_text_buffer_get_bounds(text_buffer, &start, &end);
  ret_val = gtk_text_buffer_get_text(text_buffer, &start, &end, FALSE);

  return ret_val;
}

gboolean
rubrica_form_card_enable_group(GtkTreeModel *model, GtkTreePath *path,
			       GtkTreeIter *iter,   gpointer data)
{
  RubricaGroupType type;
  
  gtk_tree_model_get(model, iter, GROUP_TYPE_COLUMN, &type, -1);
  
  if (type == (RubricaGroupType) data)    
    {
      gtk_list_store_set(GTK_LIST_STORE(model), iter,  
			 GROUP_BELONG_COLUMN, TRUE, -1); 
    }

  return FALSE;  
}




/*
void rubrica_form_card_clicked_modify(GtkWidget* button, gpointer data)
{
  RubricaCardItem *old, *new;
  RubricaFormCard* form = (RubricaFormCard*) data;
  
  old =  (RubricaCardItem) form->item;

  new = rubrica_form_card_get_item();
  new->created = created;
  new->changed = time(NULL);

  item  = rubrica_doc_get_item(rubrica, id);
      
  rubrica_card_modify_item(rubrica, item, form->index);

  rubrica_form_card_close();
}
*/


/*
 */
GList*
rubrica_form_card_get_all_data(GtkWidget *tree, RubricaTreeType type)
{
  GtkTreeModel *model;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
  grouplist = addresslist = netlist = tellist = NULL;  /* global variables */

  switch (type)
    {
    case GROUP_TREE:
      gtk_tree_model_foreach(GTK_TREE_MODEL(model),
			     rubrica_form_card_extract_group_data, NULL);
      
      return grouplist;
      break;

    case CARD_TREE:
      break;

    case ACCOUNT_TREE:
      break;

    case ADDRESS_TREE:
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), 
			     rubrica_form_card_extract_address_data, NULL);  

      return addresslist;
      break;

    case NET_TREE:
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), 
			     rubrica_form_card_extract_net_data, NULL);  

      return netlist;
      break;

    case PHONE_TREE:
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), 
			     rubrica_form_card_extract_phone_data, NULL);

      return tellist;
      break;

    default:
      break;
    }
  
  return NULL;
}

gboolean 
rubrica_form_card_uncheck_all(GtkTreeModel *model, GtkTreePath *path,
			      GtkTreeIter *iter, gpointer data)
{
  gtk_list_store_set(GTK_LIST_STORE(model), iter, 
		     GROUP_BELONG_COLUMN, FALSE, -1); 

  return FALSE; 
}

gboolean 
rubrica_form_card_extract_group_data(GtkTreeModel *model, GtkTreePath *path,
				     GtkTreeIter *iter, gpointer data)
{
  RubricaGroupType type;
  gchar *str;
  gboolean belong;
  
  gtk_tree_model_get(model, iter, 
		     GROUP_BELONG_COLUMN, &belong, 
		     GROUP_LABEL_COLUMN,  &str,
		     GROUP_TYPE_COLUMN,   &type,
		     -1);
  if (belong)
    {
      RubricaGroup* group;

      group = (RubricaGroup *) g_malloc(sizeof(RubricaGroup));
      if (!group) return FALSE;

      group->label = g_strdup(str);
      group->type  = type;

      grouplist = g_list_append(grouplist, group);
    }
  
  return FALSE;
}

gboolean 
rubrica_form_card_extract_net_data(GtkTreeModel *model, GtkTreePath *path,
				   GtkTreeIter *iter, gpointer data)
{
  gchar *str;

  gtk_tree_model_get(model, iter, URI_COLUMN, &str, -1);
  netlist = g_list_append(netlist, str);
  
  return FALSE;
}

gboolean 
rubrica_form_card_extract_address_data(GtkTreeModel *model, GtkTreePath *path,
				       GtkTreeIter *iter, gpointer data)
{
  RubricaCardAddress* address;

  gtk_tree_model_get(model, iter, ADDRESS_DATA_COLUMN, &address, -1);

  addresslist = g_list_append(addresslist, address);

  return FALSE;
}

gboolean 
rubrica_form_card_extract_phone_data(GtkTreeModel *model, GtkTreePath *path,
				     GtkTreeIter *iter, gpointer data)
{
  RubricaCardTelNum *tt;
  gchar *number;
  gchar *type;

  tt = (RubricaCardTelNum *) g_malloc(sizeof(RubricaCardTelNum));
  if (!tt) return FALSE;

  gtk_tree_model_get(model, iter, 
		     NUMBER_COLUMN, &number, 
		     TYPE_COLUMN,   &type, 
		     -1);
  
  tt->number = g_strdup(number);
  tt->type = g_strdup(type);
  tellist = g_list_append(tellist, tt);

  return FALSE;
}

 
void 
rubrica_form_card_toggle_group(GtkCellRendererToggle *cell,  
			       gchar *path_str, gpointer data) 
{ 
  GtkTreeModel *model; 
  GtkTreeIter iter; 
  GtkTreePath *path; 
  gboolean status;      

  model  = (GtkTreeModel *) data;
  path   = gtk_tree_path_new_from_string(path_str); 

  gtk_tree_model_get_iter (model, &iter, path); 
  gtk_tree_model_get (model, &iter, GROUP_BELONG_COLUMN, &status, -1); 

  status ^= 1; 
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,  
		     GROUP_BELONG_COLUMN, status, -1); 
}








































