/*
*  Rubrica
*  file: modulo_int.h
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef __MODULO_INTERFACCIA_H
#define __MODULO_INTERFACCIA_H

#include <gnome.h>
#include "rubrica_type.h"
#include "card_item.h"

#define RUBRICA_GET_GROUP(x)         (RubricaCategoriesType) gtk_option_menu_get_history(GTK_OPTION_MENU(x))
#define RUBRICA_GET_RANK(x)          (RubricaRankType)       gtk_option_menu_get_history(GTK_OPTION_MENU(x))
#define RUBRICA_GET_ADDRESS_TYPE(x)  (RubricaAddressType)    gtk_option_menu_get_history(GTK_OPTION_MENU(x))

#define RUBRICA_SET_GROUP(menu,pos)       gtk_option_menu_set_history(GTK_OPTION_MENU(menu), pos)
#define RUBRICA_SET_RANK(menu, pos)       gtk_option_menu_set_history(GTK_OPTION_MENU(menu), pos)
#define RUBRICA_SET_ADDRESS_TYPE(menu, x) gtk_option_menu_set_history(GTK_OPTION_MENU(menu), x)

#define RUBRICA_GET_ENTRY_STRING(x)  g_strdup(gtk_entry_get_text(GTK_ENTRY(x)))
#define RUBRICA_GET_CHECK_STATUS(x)  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(x))
#define RUBRICA_GET_DATE(x)          gnome_date_edit_get_time(GNOME_DATE_EDIT(x))

#define RUBRICA_SET_CHECK_STATUS(check, value)  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), value)
#define RUBRICA_SET_DATE(widget, date)          gnome_date_edit_set_time(GNOME_DATE_EDIT(widget), date)



typedef struct _RubricaFormCard         RubricaFormCard;

typedef struct _RubricaPersonalForm     RubricaPersonalForm;
typedef struct _RubricaAddressForm      RubricaAddressForm;
typedef struct _RubricaNetForm          RubricaNetForm;
typedef struct _RubricaTelephoneForm    RubricaTelephoneForm;
typedef struct _RubricaCompanyForm      RubricaCompanyForm;
typedef struct _RubricaWorkForm         RubricaWorkForm;
typedef struct _RubricaNotesForm        RubricaNotesForm;


struct _RubricaPersonalForm 
{
  GtkWidget *first;       /* first name     */
  GtkWidget *middle;      /* middle name    */
  GtkWidget *last;        /* last name      */ 
  GtkWidget *nick;        /* nickname       */
  GtkWidget *prof;        /* profession     */
  GtkWidget *prefix;      /* name prefix    */
  GtkWidget *title;       /* title          */
  GtkWidget *photo;       /* photo          */
  GtkWidget *birth;       /* know birthday? */
  GtkWidget *date;        /* bithday        */
};

struct _RubricaAddressForm
{
  GtkWidget* type;      /* address type */
  GtkWidget* street;    
  GtkWidget* number;
  GtkWidget* zip;
  GtkWidget* city;
  GtkWidget* province;
  GtkWidget* state;
  GtkWidget* country;

  GtkWidget* tree;      /* addresses tree        */
  GtkWidget* add;       /* add address button    */
  GtkWidget* mod;       /* modify address button */
  GtkWidget* del;       /* delete address button */
};

struct _RubricaNetForm 
{
  GtkWidget *web;       /* web entry          */
  GtkWidget *wtree;     /* web tree view      */
  GtkWidget *wadd;      /* web add button     */
  GtkWidget *wmod;      /* web modify button  */
  GtkWidget *wdel;      /* web delete button  */

  GtkWidget *email;     /* email entry         */
  GtkWidget *etree;     /* email tree view     */
  GtkWidget *eadd;      /* email add button    */
  GtkWidget *emod;      /* email modify button */
  GtkWidget *edel;      /* email delete button */
};


struct _RubricaTelephoneForm
{
  GtkWidget *number;

  GSList *group;
  GtkWidget *home;
  GtkWidget *work;
  GtkWidget *fax;
  GtkWidget *cell;
//  GtkWidget *pager;
  GtkWidget *other;    

  GtkWidget *tree;
  GtkWidget *add;  
  GtkWidget *mod;
  GtkWidget *del;
};

struct _RubricaCompanyForm
{
  GtkWidget *name;
  GtkWidget *street;
  GtkWidget *number;
  GtkWidget *zip;
  GtkWidget *city;    
  GtkWidget *province;
  GtkWidget *country;
  GtkWidget *web;
  GtkWidget *email;
  GtkWidget *operator;
  GtkWidget *fax;
  GtkWidget *green;
  GtkWidget *ccare;
  GtkWidget *notes;
};

struct _RubricaWorkForm
{
  GtkWidget *role;       
  GtkWidget *dep;
  GtkWidget *sub;
  GtkWidget *org;
  GtkWidget *secretary;     /* secretary/contact in company       */
  GtkWidget *phone;         /* secretary/contact telephone number */
};

struct _RubricaNotesForm
{
  GtkWidget *married;       /* check button - he/she is marries?        */

  GtkWidget *birth;         /* check button - his/her birthday is known */
  GtkWidget *bdate;         /* his/her birthday */

  GtkWidget *ann;           /* check bt - is known their anniversary date? */
  GtkWidget *adate;         /* anniversary date */
   
  GtkWidget *spouse;        /* contact spouse's name (given name) */

  GtkWidget *children;      /* text - children notes              */
  GtkWidget *notes;         /* text - his/her hobbies, notes,...  */
  GtkWidget *pubkey;        /* text - his/her public key          */
};


struct _RubricaFormCard {
  GtkWidget* dialog;

  GtkWidget* notebook;
  GtkWidget* ok;
  GtkWidget* cancel;

  GtkWidget* card;
  GtkWidget* delete;
  GtkWidget* rank;
  GtkWidget* group;  

  RubricaPersonalForm  personal;
  RubricaAddressForm   address;
  RubricaNetForm       net;
  RubricaTelephoneForm tel;
  RubricaCompanyForm   company;
  RubricaWorkForm      work;
  RubricaNotesForm     notes;
  
  RubricaCardItem* item;
  gint             index;        /* index of item */
  gchar*           cur_tel;      /* current telephone id */
//  gboolean         modified;     /* modified form */
};


gchar*           rubrica_form_card_get_option_label  (gint index);
gchar*           rubrica_form_card_get_option_pixmap (gint index);
RubricaGroupType rubrica_form_card_get_option_type   (gchar *label);

GtkTreeModel*    rubrica_form_card_create_model      (RubricaModelType type);
void             rubrica_form_card_populate_model    (GtkTreeModel *model);
void             rubrica_form_card_add_columns       (GtkTreeView *view, 
						      RubricaColumnsType type);

RubricaFormCard* rubrica_form_card_create_form       (void);

void             rubrica_form_card_init_form         (void);
RubricaFormCard* rubrica_form_card_get_form          (void);
void             rubrica_form_card_clean_form        (void);

// gboolean         rubrica_form_card_changed           (void);
// void             rubrica_form_card_set_changed       (gboolean bool);
// void             rubrica_form_card_set_form_type     (RubricaCardType type);
gchar*           rubrica_form_card_get_telephone     (void);
void             rubrica_form_card_set_telephone     (gchar *str);

RubricaCardItem* rubrica_form_card_get_item          (RubricaCardType type);
RubricaErrorType rubrica_form_card_insert_item       (RubricaCardItem *item,
						      gint index);

void             rubrica_form_card_close_form        (void);
RubricaCardItem* rubrica_form_card_add_item          (void);
RubricaCardItem* rubrica_form_card_modify_item       (RubricaDoc *doc, 
						      RubricaCardItem* item);

gchar*           rubrica_form_card_get_data          (GtkWidget *entry);
gchar*           rubrica_form_card_get_text          (GtkWidget *text);
   
gboolean         rubrica_form_card_enable_group      (GtkTreeModel *model, 
						      GtkTreePath *path,
						      GtkTreeIter *iter,   
						      gpointer data);

GList*           rubrica_form_card_get_all_data      (GtkWidget *tree,
						      RubricaTreeType type);

gboolean         rubrica_form_card_uncheck_all       (GtkTreeModel *model,
						      GtkTreePath *path,
						      GtkTreeIter *iter, 
						      gpointer data);

gboolean         rubrica_form_card_extract_group_data (GtkTreeModel *model, 
						       GtkTreePath *path,
						       GtkTreeIter *iter, 
						       gpointer data);

gboolean         rubrica_form_card_extract_net_data   (GtkTreeModel *model,
						       GtkTreePath *path,
						       GtkTreeIter *iter, 
						       gpointer data);
gboolean         rubrica_form_card_extract_phone_data (GtkTreeModel *model, 
						       GtkTreePath *path,
						       GtkTreeIter *iter,
						       gpointer data);

gboolean         rubrica_form_card_extract_address_data (GtkTreeModel *model,
							 GtkTreePath *path,
							 GtkTreeIter *iter,
							 gpointer data);

void             rubrica_form_card_toggle_group (GtkCellRendererToggle *cell,  
						 gchar *path_str,
						 gpointer data);

gchar*             rubrica_form_card_addtype2str (RubricaAddressType type);
RubricaAddressType rubrica_form_card_str2addtype (gchar* str);

#define RUBRICA_ADDRESS_TYPE(type)       rubrica_form_card_addtype2str(type)
#define RUBRICA_STR2ADDRESS_TYPE(str)    rubrica_form_card_str2addtype(str)


#endif
