/*
*  Rubrica
*  file: form_cb.c
*
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <config.h>
#include <gnome.h>

#include "form_card.h"
#include "form_cb.h"
#include "utils.h"

void 
rubrica_form_card_entry_activate(GtkEditable *editable, gpointer data)
{  
  g_signal_emit_by_name(G_OBJECT(data), "clicked");
}


void 
rubrica_form_card_option_check_toggled(GtkToggleButton *togglebutton, 
				       gpointer data)
{
  gtk_widget_set_sensitive(GTK_WIDGET(data),
			   RUBRICA_GET_CHECK_STATUS(togglebutton));
}


void 
rubrica_form_card_married_check_toggled(GtkToggleButton *togglebutton, 
					gpointer data)
{
  RubricaFormCard *form = (RubricaFormCard *) data;
  gboolean married;
  gboolean birthday;
  gboolean anniversary;
  
  married     = RUBRICA_GET_CHECK_STATUS(togglebutton);
  birthday    = RUBRICA_GET_CHECK_STATUS(form->notes.birth);
  anniversary = RUBRICA_GET_CHECK_STATUS(form->notes.ann);
  
  gtk_widget_set_sensitive(GTK_WIDGET(form->notes.birth), married);
  gtk_widget_set_sensitive(GTK_WIDGET(form->notes.ann), married);

  if (married)
    {
      gtk_widget_set_sensitive(GTK_WIDGET(form->notes.bdate), birthday);
      gtk_widget_set_sensitive(GTK_WIDGET(form->notes.adate), 
			       anniversary);  
    }
  else
    {
      gtk_widget_set_sensitive(GTK_WIDGET(form->notes.bdate), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(form->notes.adate), FALSE);
    }
}


/*  Address
*/
void rubrica_form_card_addresses_list_activated(GtkTreeSelection *selection, 
						gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormCard *form = (RubricaFormCard *) data;
  RubricaCardAddress* address;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->address.tree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, ADDRESS_DATA_COLUMN, &address, -1);

      RUBRICA_SET_ADDRESS_TYPE(form->address.type, address->type);
      gtk_entry_set_text(GTK_ENTRY(form->address.street),   address->street);
      gtk_entry_set_text(GTK_ENTRY(form->address.number),   address->number);
      gtk_entry_set_text(GTK_ENTRY(form->address.zip),      address->zip);
      gtk_entry_set_text(GTK_ENTRY(form->address.city),     address->city);
      gtk_entry_set_text(GTK_ENTRY(form->address.province), address->province);
      gtk_entry_set_text(GTK_ENTRY(form->address.state),    address->state);
      gtk_entry_set_text(GTK_ENTRY(form->address.country),  address->country);

      
      gtk_widget_set_sensitive(form->address.mod, TRUE);
      gtk_widget_set_sensitive(form->address.del, TRUE);
    }
}

void
rubrica_form_card_address_add_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormCard *form = (RubricaFormCard *) data;
  RubricaCardAddress* address;
  gchar* addtype;
  
  gtk_widget_set_sensitive(GTK_WIDGET(form->address.mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->address.del), FALSE);

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->address.tree));
  g_return_if_fail(GTK_IS_TREE_MODEL(model));
  
  address = (RubricaCardAddress *) g_malloc(sizeof(RubricaCardAddress));
  
  address->type     = RUBRICA_GET_ADDRESS_TYPE(form->address.type);
  address->street   = RUBRICA_GET_ENTRY_STRING(form->address.street);
  address->number   = RUBRICA_GET_ENTRY_STRING(form->address.number);
  address->zip      = RUBRICA_GET_ENTRY_STRING(form->address.zip);
  address->city     = RUBRICA_GET_ENTRY_STRING(form->address.city);
  address->province = RUBRICA_GET_ENTRY_STRING(form->address.province);
  address->state    = RUBRICA_GET_ENTRY_STRING(form->address.state);
  address->country  = RUBRICA_GET_ENTRY_STRING(form->address.country);

  addtype = rubrica_form_card_addtype2str(address->type);

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     ADDRESS_STREET_COLUMN, address->street,
		     ADDRESS_NUMBER_COLUMN, address->number,
		     ADDRESS_TYPE_COLUMN,   addtype,
		     ADDRESS_DATA_COLUMN,   address,
		     -1);

  RUBRICA_SET_ADDRESS_TYPE(form->address.type, RUBRICA_ADDRESS_PREF);
  gtk_entry_set_text(GTK_ENTRY(form->address.street),   "");
  gtk_entry_set_text(GTK_ENTRY(form->address.number),   "");
  gtk_entry_set_text(GTK_ENTRY(form->address.zip),      "");
  gtk_entry_set_text(GTK_ENTRY(form->address.city),     "");
  gtk_entry_set_text(GTK_ENTRY(form->address.province), "");
  gtk_entry_set_text(GTK_ENTRY(form->address.state),    "");
  gtk_entry_set_text(GTK_ENTRY(form->address.country),  "");
}


void
rubrica_form_card_address_mod_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaCardAddress* oldaddress;
  RubricaCardAddress* address;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar* addtype;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->address.tree));
  g_return_if_fail(GTK_IS_TREE_MODEL(model));

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->address.tree));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gint i;
    GtkTreePath *path;

    address = (RubricaCardAddress *) g_malloc(sizeof(RubricaCardAddress));
    
    address->type     = RUBRICA_GET_ADDRESS_TYPE(form->address.type);
    address->street   = RUBRICA_GET_ENTRY_STRING(form->address.street);
    address->number   = RUBRICA_GET_ENTRY_STRING(form->address.number);
    address->zip      = RUBRICA_GET_ENTRY_STRING(form->address.zip);
    address->city     = RUBRICA_GET_ENTRY_STRING(form->address.city);
    address->province = RUBRICA_GET_ENTRY_STRING(form->address.province);
    address->state    = RUBRICA_GET_ENTRY_STRING(form->address.state);
    address->country  = RUBRICA_GET_ENTRY_STRING(form->address.country);
    
    addtype = rubrica_form_card_addtype2str(address->type);

    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
		       ADDRESS_DATA_COLUMN, &oldaddress, -1);
    rubrica_card_item_free_address(oldaddress);

    path = gtk_tree_model_get_path(model, &iter);
    i = gtk_tree_path_get_indices(path)[0];

    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_path_free(path);
    
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, i);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		       ADDRESS_STREET_COLUMN, address->street,
		       ADDRESS_NUMBER_COLUMN, address->number,
		       ADDRESS_TYPE_COLUMN,   addtype,
		       ADDRESS_DATA_COLUMN,   address, -1);
    
    RUBRICA_SET_ADDRESS_TYPE(form->address.type, RUBRICA_ADDRESS_PREF);
    gtk_entry_set_text(GTK_ENTRY(form->address.street),   "");
    gtk_entry_set_text(GTK_ENTRY(form->address.number),   "");
    gtk_entry_set_text(GTK_ENTRY(form->address.zip),      "");
    gtk_entry_set_text(GTK_ENTRY(form->address.city),     "");
    gtk_entry_set_text(GTK_ENTRY(form->address.province), "");
    gtk_entry_set_text(GTK_ENTRY(form->address.state),    "");
    gtk_entry_set_text(GTK_ENTRY(form->address.country),  "");		       
    
    gtk_widget_set_sensitive(GTK_WIDGET(form->address.mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->address.del), FALSE);   
  }  
}


void
rubrica_form_card_address_del_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;
  RubricaCardAddress* oldaddress;
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->address.tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->address.tree));
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
		       ADDRESS_DATA_COLUMN, &oldaddress, -1);
    rubrica_card_item_free_address(oldaddress);

    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
   
    RUBRICA_SET_ADDRESS_TYPE(form->address.type, RUBRICA_ADDRESS_PREF);
    gtk_entry_set_text(GTK_ENTRY(form->address.street),   "");
    gtk_entry_set_text(GTK_ENTRY(form->address.number),   "");
    gtk_entry_set_text(GTK_ENTRY(form->address.zip),      "");
    gtk_entry_set_text(GTK_ENTRY(form->address.city),     "");
    gtk_entry_set_text(GTK_ENTRY(form->address.province), "");
    gtk_entry_set_text(GTK_ENTRY(form->address.state),    "");
    gtk_entry_set_text(GTK_ENTRY(form->address.country),  "");

    gtk_widget_set_sensitive(GTK_WIDGET(form->address.mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->address.del), FALSE);   
  }  
}


/*  Web
*/
void rubrica_form_card_web_list_activated(GtkTreeSelection *selection, 
					  gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar *str;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.wtree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, URI_COLUMN, &str, -1);
      
      gtk_entry_set_text(GTK_ENTRY(form->net.web), str);
      
      gtk_widget_set_sensitive(form->net.wmod, TRUE);
      gtk_widget_set_sensitive(form->net.wdel, TRUE);
      g_free(str);
    }
}

void
rubrica_form_card_web_add_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *str;
  RubricaFormCard *form = (RubricaFormCard *) data;
  
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.wmod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.wdel), FALSE);

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.wtree));
  str   = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->net.web)));

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     URI_COLUMN, str, -1);

  gtk_entry_set_text(GTK_ENTRY(form->net.web), "");
}


void
rubrica_form_card_web_mod_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar *str = NULL;

  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.wtree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->net.wtree));
  str       = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->net.web)));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gint i;
    GtkTreePath *path;
    
    path = gtk_tree_model_get_path(model, &iter);
    i    = gtk_tree_path_get_indices(path)[0];
    
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_path_free(path);
    
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, i);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		       URI_COLUMN, str, -1);

    gtk_entry_set_text(GTK_ENTRY(form->net.web), "");

    gtk_widget_set_sensitive(GTK_WIDGET(form->net.wmod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->net.wdel), FALSE);   
  }  
}


void
rubrica_form_card_web_del_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;
  
  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.wtree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->net.wtree));
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
   
    gtk_entry_set_text(GTK_ENTRY(form->net.web), "");

    gtk_widget_set_sensitive(GTK_WIDGET(form->net.wmod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->net.wdel), FALSE);   
  }  
}

/*  Email
*/
void rubrica_form_card_email_list_activated(GtkTreeSelection *selection, 
					    gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar *str;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.etree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, URI_COLUMN, &str, -1);
      
      gtk_entry_set_text(GTK_ENTRY(form->net.email), str);
      
      gtk_widget_set_sensitive(form->net.emod, TRUE);
      gtk_widget_set_sensitive(form->net.edel, TRUE);
      g_free(str);
    }
}

void
rubrica_form_card_email_add_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *str;
  RubricaFormCard *form = (RubricaFormCard *) data;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.etree));
  str   = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->net.email)));

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     URI_COLUMN, str, -1);

  gtk_entry_set_text(GTK_ENTRY(form->net.email), "");
  
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.emod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->net.edel), FALSE);  
}


void
rubrica_form_card_email_mod_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar *str = NULL;

  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.etree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->net.etree));
  str       = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->net.email)));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gint i;
    GtkTreePath *path;
    
    path = gtk_tree_model_get_path(model, &iter);
    i    = gtk_tree_path_get_indices(path)[0];
    
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_path_free(path);
    
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, i);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		       URI_COLUMN, str, -1);

    gtk_entry_set_text(GTK_ENTRY(form->net.email), "");

    gtk_widget_set_sensitive(GTK_WIDGET(form->net.emod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->net.edel), FALSE);   
  }  
}


void
rubrica_form_card_email_del_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->net.etree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->net.etree));
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
   
    gtk_entry_set_text(GTK_ENTRY(form->net.email), "");

    gtk_widget_set_sensitive(GTK_WIDGET(form->net.emod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->net.edel), FALSE);   
  }  
}



/* Telephone
 */
void rubrica_form_card_telephone_list_activated(GtkTreeSelection *selection, 
						gpointer data)
{
  RubricaTelephoneType type;
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar *numb_str;
  gchar *type_str;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->tel.tree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, NUMBER_COLUMN, &numb_str, -1);
      gtk_tree_model_get(model, &iter, TYPE_COLUMN,   &type_str, -1); 
      
      gtk_entry_set_text(GTK_ENTRY(form->tel.number), numb_str);
     
      type = rubrica_utils_telephone2int(type_str);

      switch (type)
	{
	case TELEPHONE_HOME:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.home),TRUE);
	  break;
	case TELEPHONE_WORK:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.work),TRUE);
	  break;
	case TELEPHONE_FAX:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.fax), TRUE);
	  break;
	case TELEPHONE_CELLPHONE:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.cell),TRUE);
	  break; 
	case TELEPHONE_OTHER:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.other),
				       TRUE);
	  break; 

	default:
	  break;
	}      
      
      gtk_widget_set_sensitive(form->tel.mod, TRUE);
      gtk_widget_set_sensitive(form->tel.del, TRUE);
      g_free(numb_str);
      g_free(type_str);
    }
}


void
rubrica_form_card_tel_add_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *numb_str = NULL;
  gchar *type_str = NULL;
  RubricaFormCard *form = (RubricaFormCard *) data;

  model    = gtk_tree_view_get_model(GTK_TREE_VIEW(form->tel.tree));
  numb_str = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->tel.number)));

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.home)))
    type_str = _("home");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.work)))
    type_str = _("work");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.fax)))
    type_str = _("fax");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.cell)))
    type_str = _("cellphone");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.other)))
    type_str = _("other");

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     NUMBER_COLUMN, numb_str, 
		     TYPE_COLUMN,   type_str,
		     -1);

  gtk_entry_set_text(GTK_ENTRY(form->tel.number), "");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.home), TRUE);
    
  gtk_widget_set_sensitive(GTK_WIDGET(form->tel.mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->tel.del), FALSE);  
}


void
rubrica_form_card_tel_mod_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;
  gchar *numb_str = NULL;
  gchar *type_str = NULL;


  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->tel.tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->tel.tree));
  numb_str  = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->tel.number)));

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.home)))
    type_str = _("home");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.work)))
    type_str = _("work");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.fax)))
    type_str = _("fax");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.cell)))
    type_str = _("cellphone");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->tel.other)))
    type_str = _("other");

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gint i;
    GtkTreePath *path;
    
    path = gtk_tree_model_get_path(model, &iter);
    i    = gtk_tree_path_get_indices(path)[0];
    
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_path_free(path);
    
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, i);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		       NUMBER_COLUMN, numb_str, 
		       TYPE_COLUMN,   type_str,
		       -1);

    gtk_entry_set_text(GTK_ENTRY(form->tel.number),"");

    gtk_widget_set_sensitive(GTK_WIDGET(form->tel.mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->tel.del), FALSE);   
  }  
}


void
rubrica_form_card_tel_del_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormCard *form = (RubricaFormCard *) data;

  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->tel.tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->tel.tree));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    
    gtk_entry_set_text(GTK_ENTRY(form->tel.number),"");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->tel.home), TRUE);

    gtk_widget_set_sensitive(GTK_WIDGET(form->tel.mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->tel.del), FALSE);   
  }  
}
