/*
*  Rubrica
*  file: init.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "init.h"
#include "utils.h"
#include "sort.h"
#include "prefer.h"
#include "data_view.h"
#include "dictionary.h"


void rubrica_init(void)
{
  g_print("\nRubrica: begin rubrica initialization...");

  rubrica_utils_add_pixmap_directory(RUBRICA_UI_DIR);
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR);
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR"/account");
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR"/card");
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR"/groups");
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR"/preferences");
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR"/ui");
  rubrica_utils_add_pixmap_directory(RUBRICA_PIXMAP_DIR"/rank");


  rubrica_init_gconf();
  rubrica_load_data_view_configuration();
  rubrica_check_keys();
  rubrica_utils_update_time();
  rubrica_init_sort();

  g_print("\nRubrica: rubrica initialization done");
}


void 
rubrica_init_gconf(void)
{
  GConfClient *client;

  g_print("\nRubrica: adding gconf directories");
  
  client = gconf_client_get_default();

  /*    keys
  */
  gconf_client_add_dir(client, "/apps/rubrica", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/file", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/general", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/show",   
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

  /*    show columns keys
  */
  gconf_client_add_dir(client, "/apps/rubrica/show/card/column", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/show/account/column", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

  /*    data keys
   */
  gconf_client_add_dir(client, "/apps/rubrica/show/card/data/personal", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/show/card/data/communicate", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/show/card/data/job",    
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/show/card/data/notes",  
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

  gconf_client_add_dir(client, "/apps/rubrica/show/account/data",
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

  /*    translation keys
  */
  gconf_client_add_dir(client, "/apps/rubrica/translation", 
		       GCONF_CLIENT_PRELOAD_NONE, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/dictionary",  
		       GCONF_CLIENT_PRELOAD_NONE, NULL);

  g_print("\nRubrica: gconf directories added");
}



void rubrica_load_data_view_configuration(void)
{
  GConfClient *client;

  g_print("\nRubrica: loading rubrica configuration");
  
  client = gconf_client_get_default();

  if (!client)
    {
      g_error("\nCant' get GConf client. "
	      "Please verify if GConf is running on your system");
    }

  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_FIRST_NAME);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_MIDDLE_NAME);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_LAST_NAME);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_NICKNAME);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_PROFESSION);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_NAME_PREFIX);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_TITLE);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_BIRTHDAY);
//  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_BIRTHKNOW);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_STREET);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_STREET_NUM);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_ZIP_CODE);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_CITY);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_PROVINCE);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_STATE);
  rubrica_load_personal_key_val(client, RUBRICA_KEY_SCD_COUNTRY);

  rubrica_load_communicate_key_val(client, RUBRICA_KEY_SCD_EMAIL);
  rubrica_load_communicate_key_val(client, RUBRICA_KEY_SCD_WEB);
  rubrica_load_communicate_key_val(client, RUBRICA_KEY_SCD_TELEPHONE);

  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_STREET);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_STREET_NUM);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_ZIP_CODE);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_CITY);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_PROVINCE);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_COUNTRY);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_WEB);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_EMAIL);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_OPERATOR);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_FAX);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_GREEN);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_CUSTOMER_CARE);
  rubrica_load_company_key_val(client, RUBRICA_KEY_SCD_COMPANY_NOTE);

  rubrica_load_assigment_key_val(client, RUBRICA_KEY_SCD_ASSIGMENT);
  rubrica_load_assigment_key_val(client, RUBRICA_KEY_SCD_ORG);
  rubrica_load_assigment_key_val(client, RUBRICA_KEY_SCD_DEP);
  rubrica_load_assigment_key_val(client, RUBRICA_KEY_SCD_SUB_DEP);
  rubrica_load_assigment_key_val(client, RUBRICA_KEY_SCD_SECRETARY);
  rubrica_load_assigment_key_val(client, RUBRICA_KEY_SCD_SECR_PHONE);

  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_MARRIED);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_SPOUSE);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_SPOUSE_BIRTH);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_ANNIVERSARY);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_HAS_CHILDREN);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_CHILDREN);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_HOBBIES);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_NOTE);
  rubrica_load_notes_key_val(client, RUBRICA_KEY_SCD_PUBLIC_KEY);

  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_NAME);
  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_LOGIN);
  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_PASSWORD);
  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_WEB);
  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_EMAIL);
  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_DESC);
  rubrica_load_account_key_val(client, RUBRICA_KEY_SAD_LONG_DESC);  

  g_object_unref(client);

  g_print("\nRubrica: rubrica configuration loaded"); 
}


/* NOTE:
   This function will be used to upgrade rubrica's GConf keys
   examples: from 1.0.x to 1.x the old keys RUBRICA_KEY_XXX_BOOK have 
   been moved to RUBRICA_KEY_XXX_DOC. So I need to copy old values 
   in new keys.
*/
void rubrica_check_keys(void)
{
  GConfClient *client;

  g_print("\nRubrica: checking rubrica's gconf keys...");

  client = gconf_client_get_default();

  if (rubrica_preferences_upgrade_keys())
    {
      /* need to upgrade: 
	 // RUBRICA_KEY_LAST_BOOK ==> RUBRICA_KEY_LAST_DOC
	 RUBRICA_KEY_HAVE_BOOK ==> RUBRICA_KEY_HAVE_DOC
	 RUBRICA_KEY_BOOK      ==> RUBRICA_KEY_DOC
      */
      gboolean bool;
      gchar* doc;

      if (gconf_client_dir_exists(client, RUBRICA_KEY_HAVE_BOOK, NULL))
	{
	  bool = rubrica_preferences_get_have_default_addressbook();
	  rubrica_preferences_set_have_default_doc(bool);
	}
      else
	rubrica_preferences_set_have_default_doc(FALSE);
      
      if (gconf_client_dir_exists(client, RUBRICA_KEY_BOOK, NULL))
	{
	  doc = rubrica_preferences_get_book();
	  rubrica_preferences_set_doc(doc);
	}
      else
	rubrica_preferences_set_doc("");	    
    }

  g_print("\nRubrica: keys check done.");
}


void rubrica_init_sort(void)
{
  GConfClient *client;

  g_print("\nRubrica: begin sort initialization...");

  client = gconf_client_get_default();  

  gconf_client_notify_add(client, RUBRICA_KEY_SORT_TYPE, 
			  rubrica_sort_notify_sort_type_changes,
			  NULL, NULL, NULL);

  gconf_client_notify_add(client, RUBRICA_KEY_SORT_DIRECTION, 
			  rubrica_sort_notify_sort_direction_changes,
			  NULL, NULL, NULL);
  g_object_unref(client);

  g_print("\nRubrica: sort initialization done...");
}


void rubrica_init_dictionary(void)
{
  gchar *dicthome = NULL;
  gchar *home, *homedir;
  RubricaLanguageType language;
  
  g_print("\nRubrica: begin dictionary initialization...");

  home     = g_strdup(g_get_home_dir());
  homedir  = g_strdup_printf("%s/.rubrica/dictionary", home);
  dicthome = rubrica_preferences_get_dictionary_home();
  
  if (!dicthome ||
      (g_ascii_strcasecmp(dicthome, homedir) != 0))
    {
      dicthome = g_strdup_printf("%s/.rubrica/dictionary", home);
      rubrica_preferences_set_dictionary_home(dicthome);
    }
  
  /*      test is user want to use a default dictionary 
   */
  if (user_wants_default_dictionary())
    {
      if(user_wants_italian_dictionary())
	language = ITALIAN;
      else
	language = ENGLISH;
    }
  else
    language = USER;
  
  /*      test if this is first rubrica's run 
   */
  if (rubrica_run_first_time())
    rubrica_dictionary_init(ITALIAN);
  else
    rubrica_dictionary_init(language);  

  g_print("\nRubrica: dictionary initialization done...");
}

gboolean 
rubrica_start_with_user_addressbook(void)
{
  return rubrica_preferences_get_have_default_addressbook();
}


gchar*
rubrica_get_book(void)
{
  gchar *file;
  
  file = g_strdup(rubrica_preferences_get_book());
  
  return (gchar *)(file ? file : NULL);
}



void 
rubrica_load_personal_key_val(GConfClient* client, gchar* key)
{
  GConfValue  *value;
  gboolean bool;

  value = gconf_client_get(client, key, NULL);
  if (value != NULL && value->type == GCONF_VALUE_BOOL)
    {
      bool = gconf_client_get_bool(client, key, NULL);
      rubrica_data_view_inc_personal(bool);
    }

  gconf_client_notify_add(client, key, rubrica_personal_key_changes, key,
			  NULL, NULL);
}


void
rubrica_load_communicate_key_val(GConfClient* client, gchar* key)
{
  GConfValue  *value;
  gboolean bool;

  value = gconf_client_get(client, key, NULL);
  if (value != NULL && value->type == GCONF_VALUE_BOOL)
    {
      bool = gconf_client_get_bool(client, key, NULL);
      rubrica_data_view_inc_communicate(bool);
    }

  gconf_client_notify_add(client, key, rubrica_communicate_key_changes, key,
			  NULL, NULL);
}


void 
rubrica_load_company_key_val(GConfClient* client, gchar* key)
{
  GConfValue  *value;
  gboolean bool;

  value = gconf_client_get(client, key, NULL);
  if (value != NULL && value->type == GCONF_VALUE_BOOL)
    {
      bool = gconf_client_get_bool(client, key, NULL);
      rubrica_data_view_inc_company(TRUE);
    }

  gconf_client_notify_add(client, key, rubrica_company_key_changes, key,
			  NULL, NULL);
}


void 
rubrica_load_assigment_key_val(GConfClient* client, gchar* key)
{
  GConfValue  *value;
  gboolean bool;

  value = gconf_client_get(client, key, NULL);
  if (value != NULL && value->type == GCONF_VALUE_BOOL)
    {
      bool = gconf_client_get_bool(client, key, NULL);
      rubrica_data_view_inc_assigment(bool);
    }

  gconf_client_notify_add(client, key, rubrica_assigment_key_changes, key,
			  NULL, NULL);
}


void 
rubrica_load_notes_key_val(GConfClient* client, gchar* key)
{
  GConfValue  *value;
  gboolean bool;

  value = gconf_client_get(client, key, NULL);
  if (value != NULL && value->type == GCONF_VALUE_BOOL)
    {
      bool = gconf_client_get_bool(client, key, NULL);
      rubrica_data_view_inc_notes(bool);
    }

  gconf_client_notify_add(client, key, rubrica_notes_key_changes, key,
			  NULL, NULL);
}


void 
rubrica_load_account_key_val(GConfClient* client, gchar* key)
{
  GConfValue  *value;
  gboolean bool;

  value = gconf_client_get(client, key, NULL);
  if (value != NULL && value->type == GCONF_VALUE_BOOL)
    {
      bool = gconf_client_get_bool(client, key, NULL);
      rubrica_data_view_inc_account(TRUE);
    }

  gconf_client_notify_add(client, key, rubrica_account_key_changes, key,
			  NULL, NULL);
}


void 
rubrica_personal_key_changes(GConfClient *client, guint cnxn_id, 
				  GConfEntry *entry, gpointer key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, (gchar *) key, NULL);
  rubrica_data_view_inc_personal(bool);
}


void 
rubrica_communicate_key_changes(GConfClient *client, guint cnxn_id, 
				     GConfEntry *entry, gpointer key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, (gchar *) key, NULL);
  rubrica_data_view_inc_communicate(bool);  
}


void 
rubrica_company_key_changes(GConfClient *client, guint cnxn_id, 
				 GConfEntry *entry, gpointer key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, (gchar *) key, NULL);
  rubrica_data_view_inc_company(bool);  
}

void 
rubrica_assigment_key_changes(GConfClient *client, guint cnxn_id, 
				   GConfEntry *entry, gpointer key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, (gchar *) key, NULL);
  rubrica_data_view_inc_assigment(bool);      
}

void 
rubrica_notes_key_changes(GConfClient *client, guint cnxn_id, 
			       GConfEntry *entry, gpointer key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, (gchar *) key, NULL);
  rubrica_data_view_inc_notes(bool);  
}


void 
rubrica_account_key_changes(GConfClient *client, guint cnxn_id, 
				 GConfEntry *entry, gpointer key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, (gchar *) key, NULL);
  rubrica_data_view_inc_account(bool);  
}
