/*
*  Rubrica
*  file: rubrica.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "rubrica.h"
#include "data_view.h"
#include "main.h"
#include "dialog.h"
#include "card.h"
#include "ui.h"
#include "callbacks.h"
#include "form_card.h"
#include "utils.h"


GList*         folders = NULL;

RubricaFolder* active_folder;
gboolean       have_buffer = FALSE;

gpointer buffer = NULL;
gpointer rtemp  = NULL;

gint position = -1;


/*   RubricaDocTab
*/
  

/*
  crea un box da inserire come tab nella pagina del notebook.
  Contiene una lampada rossa, una verde, 
  un'etichetta con il nome del file e un bottone per
  chiudere la pagina del notebook.
*/
GtkWidget*
rubrica_doc_tab_new(RubricaDoc* doc, gchar* name)
{  
  RubricaDocTab* doc_tab = NULL;
  GtkWidget* image;
  gchar* str;

  if (!name)
    g_warning("\nRubrica ** rubrica_doc_tab_new name is NULL");

  doc_tab = (RubricaDocTab*) g_malloc(sizeof(RubricaDocTab));
  if (!doc_tab)
    g_error("\nCan't allocate enougth memory");

  image = rubrica_ui_image_new("close.png");

  doc_tab->ttips = gtk_tooltips_new();
  doc_tab->box   = gtk_hbox_new(FALSE, 2);
  doc_tab->green = rubrica_ui_image_new("ledgreen.png");
  doc_tab->red   = rubrica_ui_image_new("ledred.png");
  doc_tab->label = gtk_label_new(name);
  doc_tab->close = gtk_button_new();

  gtk_container_add(GTK_CONTAINER(doc_tab->close), image);  
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->green, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->red,   FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->label, TRUE,  FALSE, 1);
  gtk_box_pack_start(GTK_BOX(doc_tab->box), doc_tab->close, FALSE, FALSE, 1);

  gtk_button_set_relief(GTK_BUTTON(doc_tab->close), GTK_RELIEF_NONE);
  
  gtk_widget_show(doc_tab->box);
  gtk_widget_show(doc_tab->green);
  gtk_widget_hide(doc_tab->red);
  gtk_widget_show(doc_tab->label);
  gtk_widget_show(doc_tab->close);
  rubrica_utils_add_tooltip(doc_tab->ttips);

  str = g_strdup_printf(_("close the addressbook %s"), name);
  gtk_tooltips_set_tip(GTK_TOOLTIPS(doc_tab->ttips), doc_tab->close, 
		       str, NULL);
  g_free(str);

  doc->tab = doc_tab;

  g_signal_connect(G_OBJECT(doc_tab->close), "clicked",
		   G_CALLBACK(on_close_activate), doc);  


  return doc_tab->box;
}


/*  modifica la stringa visualizzata sull'etichetta
*/
void 
rubrica_doc_tab_set_label(RubricaDoc* doc, gchar *label)
{
  gchar* str;

  str = g_strdup_printf(_("close the addressbook %s"), label);
  gtk_tooltips_set_tip(GTK_TOOLTIPS(doc->tab->ttips), doc->tab->close, 
		       str, NULL);
  
  gtk_label_set_text(GTK_LABEL(doc->tab->label), label);

  g_free(str);
}


/*  spegne la luce rossa (errore/modifica), 
    accende la luce verde (ok)
*/  
void 
rubrica_doc_tab_enable_green(RubricaDoc* doc)
{
  gtk_widget_hide(doc->tab->red);
  gtk_widget_show(doc->tab->green);
}


/*  spegne la luce verde
    accende la luce rosssa
*/
void 
rubrica_doc_tab_enable_red(RubricaDoc* doc)
{
  gtk_widget_hide(doc->tab->green);
  gtk_widget_show(doc->tab->red);
}



/*   RubricaFolder
*/

/*  crea un nuovo folder
    create a new folder
*/
RubricaFolder *
rubrica_folder_new(RubricaFolderType type)
{
  RubricaFolder* folder;

  folder = (RubricaFolder*) g_malloc(sizeof(RubricaFolder));

  if (!folder)
    {
      g_error("\nRubrica: Out of memory. I can't allocate memory") ;
    }
  
  folder->type       = type;     /* folder type                     */

  folder->docs       = NULL;     /* documents list                  */
  folder->current    = NULL;     /* current document                */
  folder->documents  = 0;        /* documents number in this folder */

  folder->container  = NULL;     /* notebook container              */
  folder->label      = NULL;     /* container's label               */
  folder->notebook   = NULL;     /* notebook, document container    */

  folders = g_list_append(folders, folder);

  return folder;
}


void 
rubrica_folder_free(RubricaFolder* folder)
{
  g_object_unref(folder->container);

//  TODO: rimuovere elemento dalla lista
}

void 
rubrica_folder_interface_new(GtkWidget *box, 
			     GtkWidget *label,
			     RubricaFolderType type)
{
  RubricaFolder *folder;

  folder = rubrica_folder_new(type);

  /* build notebook container */
  folder->container = gtk_hbox_new(FALSE, 0);

  /* container label */
  folder->label = label;   

  /* the notebook */
  folder->notebook  = gtk_notebook_new();
  gtk_container_add(GTK_CONTAINER(folder->container), folder->notebook);
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK (folder->notebook), GTK_POS_BOTTOM);
  gtk_notebook_set_scrollable(GTK_NOTEBOOK (folder->notebook), TRUE);

  gtk_box_pack_start(GTK_BOX(box), folder->container, TRUE, TRUE, 0);
  
  if (type == ADDRESSBOOK_FOLDER)
    gtk_widget_show_all(folder->container);
  else
    {
      /*
      GtkWidget* lbl;
      
      lbl = gtk_label_new("<b>Accounts</b>");
      gtk_label_set_use_markup(GTK_LABEL(lbl), TRUE);
      gtk_widget_show(lbl);
      folder->label = lbl;
      */
      
      // TODO: set folder->label to <b>Accounts</b>
      gtk_widget_hide_all(folder->container);
    }
    
  g_signal_connect(G_OBJECT(folder->notebook), "switch-page",  
		   G_CALLBACK(document_notebook_switch_pages_cb), 
		   (gpointer) folder);  
}


GList* 
rubrica_folder_get_all_folders(void)
{
  return (GList*) folders;
}


RubricaFolder* 
rubrica_folder_get_first_folder()      
{
  return (RubricaFolder*) g_list_first(folders);
}


RubricaFolder* 
rubrica_folder_get_last_folder()
{
  return (RubricaFolder*) g_list_last(folders);
}


RubricaFolder* 
rubrica_folder_get_next_folder()
{
  return (RubricaFolder*) g_list_next(active_folder);
}


RubricaFolder* 
rubrica_folder_get_previous_folder()
{
  return (RubricaFolder*) g_list_previous(active_folder);
}


RubricaFolder* 
rubrica_folder_get_folder(RubricaFolderType type)
{
  GList* flds;
  RubricaFolder* folder;

  flds = folders;
  for (; flds; flds = g_list_next(flds))
    {
      folder = flds->data;

      if (folder->type == type)
	return folder;
    }

  return NULL;
}


void 
rubrica_folder_init_active_folder(RubricaFolderType type)
{  
  active_folder = (RubricaFolder*) g_list_nth_data(folders, type); 
}

void 
rubrica_folder_set_active_folder(RubricaFolderType type)
{
  RubricaFolder* next_folder;

  next_folder = (RubricaFolder*) g_list_nth_data(folders, type);  
  gtk_widget_hide_all(active_folder->container);
  gtk_widget_show_all(next_folder->container);
  active_folder = next_folder;
}


RubricaFolderType 
rubrica_folder_get_active_folder (void)
{
  return (RubricaFolderType) active_folder->type;
}


RubricaFolderType 
rubrica_folder_get_type (RubricaFolder* folder)
{
  if (!folder)
    folder = active_folder;

  return (RubricaFolderType) folder->type;
}


GList*
rubrica_folder_get_documents (RubricaFolder* folder)
{
  if (!folder)  
    folder = active_folder;
  
  return (GList*) folder->docs;
}


gint 
rubrica_folder_get_docs_number (RubricaFolder* folder)
{
  if (!folder)  
    folder = active_folder;

  return (gint) folder->documents;
}


gint 
rubrica_folder_get_page(RubricaFolder* folder)
{
  if (!folder)
    folder = active_folder;

  return gtk_notebook_get_current_page(GTK_NOTEBOOK(folder->notebook));
}


RubricaDoc* 
rubrica_folder_get_current_document(void)
{
  RubricaDoc* doc;

  doc = (RubricaDoc*) active_folder->current;

  if (rubrica_is_valid_doc(doc))
    return (RubricaDoc*) doc; 
  
  return NULL;
}


RubricaDoc* 
rubrica_folder_get_nth_document(RubricaFolder* folder, gint n)
{
  RubricaDoc* doc;

  if (!folder)
    folder = active_folder;

  doc = (RubricaDoc*) g_list_nth_data(folder->docs, n);

  if (rubrica_is_valid_doc(doc))
    return (RubricaDoc*) doc; 
  
  return NULL;
}


void 
rubrica_folder_set_doc_as_current(RubricaFolder* folder, RubricaDoc* doc)
{
  if (!folder)
    folder = active_folder;

  folder->current = doc;
}


void 
rubrica_folder_append_doc(RubricaFolder* folder, RubricaDoc* doc,
			  GtkWidget *child, GtkWidget *box)
{
  if (!folder)
    folder = active_folder;

  folder->docs = g_list_append(folder->docs, doc);
  gtk_notebook_append_page(GTK_NOTEBOOK(folder->notebook), child, box);
  gtk_notebook_set_current_page(GTK_NOTEBOOK(folder->notebook), -1);

  doc->page = gtk_notebook_get_current_page(GTK_NOTEBOOK(folder->notebook));
  folder->documents++;
}


void rubrica_folder_remove_doc(RubricaDoc* doc)
{
  RubricaFolder* folder;
  GList* removed = NULL;
  gint page;

  if (!doc)
    return;

  folder  = doc->father;
  page    = doc->page;
  removed = g_list_remove(folder->docs, doc);

  if (removed)
    {
      rubrica_doc_free_memory(doc);
      g_list_free_1(removed);
      gtk_notebook_remove_page(GTK_NOTEBOOK(folder->notebook), page);
      folder->documents--;
    }
  else
    {
      g_warning("\nremoved == NULL");
    }
}


/*  RubricaDoc
*/
RubricaDoc* 
rubrica_doc_new(RubricaDocType doctype)
{
  RubricaDoc *doc = NULL;

  doc = (RubricaDoc *) g_malloc(sizeof(RubricaDoc));

  if (!doc)
    g_error(_("Out of memory! I can't allocate enough memory for rubrica"));
  
  doc->id        = 1L;
  doc->type      = doctype;

  doc->docs      = NULL;      /* items list                         */
  doc->xmldoc    = NULL;      /* ptr to xml doc                     */

  doc->new       = TRUE;      /* true if this is a new addressbook  */
  doc->modified  = FALSE;     /* true if rubrica is modified */
  doc->empty     = TRUE;      /* true if rub list is empty   */

  doc->name      = NULL;      /* addressbook's name */ 
  doc->path      = NULL;      /* addressbook's path */
  doc->tmpfile   = NULL;      /* temp file's name, used in autosave */

  doc->items     = 0;         /* number of item in current document */
  doc->index     = -1;        /* number of the item currently show  */
  doc->list_path = NULL;      /* currently shown item's path (GtkTreePath) */

  doc->father    = NULL;      /* folder that own this document */
  doc->view      = NULL;      /* the tree_view */
  doc->child     = NULL;      /* notebook page child */
  doc->tab       = NULL;      /* notebook tab */

  return (RubricaDoc*) doc;
}

gboolean 
rubrica_is_valid_doc(RubricaDoc*doc)
{
  if (!doc || !doc->father || !doc->view) 
    return FALSE;

  switch (doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
    case ACCOUNT_DOC_TYPE:
    case BLOCK_NOTES_DOC_TYPE:
      return TRUE;
      break;
      
    default:
      break;
    }
  
  return FALSE;
}


/*  return the first available id
*/
gulong
rubrica_doc_get_id(RubricaDoc *doc)
{
  gint id;

  if (!doc)
    return 0L;

  id = doc->id;
  if (id <= doc->items)
    id = doc->id = doc->items+1;
  
  doc->id++;

  return id;
}


RubricaDocType 
rubrica_doc_get_type(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
      
      return UNKNOWN_DOC;
    }

  return (RubricaDocType) doc->type;
}

/*  is New ?
 */
gboolean 
rubrica_doc_is_new(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return FALSE;
    }

  return (gboolean) doc->new;
}


/*  is Empty ?
*/         
gboolean 
rubrica_doc_is_empty(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return FALSE;
    }

  return (gboolean) doc->empty;
}


/*  is Modified ?
*/
gboolean 
rubrica_doc_is_modified(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return FALSE;
    }

  return (gboolean) doc->modified;
}


/*  Get rubrica's path
*/
gchar*
rubrica_doc_get_path(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return NULL;
    }

  return (gchar *) doc->path;
}


/*  Get rubrica's file name
*/
gchar*
rubrica_doc_get_name(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return NULL;
    }

  return (gchar *) doc->name;
}


gchar*
rubrica_doc_get_tmpfile(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return NULL;
    }

  return (gchar *) doc->tmpfile;  
}


/*  Get rubrica's items
*/
gint 
rubrica_doc_get_items(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return 0;
    }

  return (gint) doc->items;
}


/*  Get the index of currently displayed item
*/
gint 
rubrica_doc_get_index(RubricaDoc *doc)
{
  if (!doc)
    {
      g_warning("\nThis isn't a valid rubrica document");
	
      return 0;
    }

  return (gint) doc->index;
}


gchar*
rubrica_doc_get_list_path(RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, NULL);

  return (gchar*) doc->list_path;
}

RubricaFolder*    
rubrica_doc_get_father (RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, NULL);

  return (RubricaFolder*) doc->father;
}

/*  Get GtkTreeView
*/
GtkWidget*
rubrica_doc_get_view(RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, NULL);

  return (GtkWidget *) doc->view; 
}


/* reject id
 */
void           
rubrica_doc_reject_id (RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);

  doc->id--;
}


/*  setta:
    true se la lista e' nuova
    false altrimenti  
 */
void 
rubrica_doc_set_new(RubricaDoc *doc, gboolean state)
{
  g_return_if_fail(doc != NULL);

  doc->new = (gboolean) state;
}


/*  setta a:
    true se la lista e' vuota
    false altrimenti  
 */
void 
rubrica_doc_set_empty(RubricaDoc *doc, gboolean state)
{
  g_return_if_fail(doc != NULL);

  doc->empty = state;
}

/*  setta a:
    true se la lista e' modificata
    false altrimenti  
 */
void 
rubrica_doc_set_modified(RubricaDoc *doc, gboolean modify)
{
  GtkWidget *app;
  gchar *title, *name;
  
  g_return_if_fail(doc != NULL);

  app  = rubrica_get_app();
  name = rubrica_doc_get_name(doc);

  if (modify) 
    {
      title = g_strdup_printf(_("Rubrica -- %s changed --"), name);
      gtk_window_set_title(GTK_WINDOW(app), title);
      rubrica_doc_tab_enable_red(doc);
      g_free(title);
    }
  else      
    {
      gtk_window_set_title(GTK_WINDOW(app), "Rubrica");
      rubrica_doc_tab_enable_green(doc);
    }
  
  doc->modified = modify;
}


/*  imposta la path a str
*/
void 
rubrica_doc_set_path(RubricaDoc *doc, gchar *str)
{
  g_return_if_fail(doc != NULL);

  if (doc->path)
    g_free(doc->path);

  if (str)
    doc->path = g_strdup(str);
  else
    doc->path = NULL;   
}


/*  imposta a str il nuovo nome della rubrica
*/
void 
rubrica_doc_set_name(RubricaDoc *doc, gchar *str)
{
  g_return_if_fail(doc != NULL);

  if (doc->name)
    g_free(doc->name);

  if (str)
    doc->name = g_strdup(str);
  else
    doc->name = NULL;

  rubrica_doc_tab_set_label(doc, str);
}


/*  imposta il nome del file temporaneo
*/
void 
rubrica_doc_set_tmpfile(RubricaDoc *doc, gchar *str)
{
  g_return_if_fail(doc != NULL);

  if (doc->tmpfile)
    g_free(doc->tmpfile);

  
  doc->tmpfile = str ? g_strdup(str) : NULL;
}


/*  setta il numero di elementi a n
 */
void 
rubrica_doc_set_items(RubricaDoc *doc, gint n)
{
  g_return_if_fail(doc != NULL);

  doc->items = n;
}

void 
rubrica_doc_inc_items(RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);

  doc->items++;
}

void 
rubrica_doc_dec_items(RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);

  doc->items--;
}



/*  salva l'indice dell'elemento mostrato attualmente
    store the index of item currently showed
 */
void 
rubrica_doc_set_index(RubricaDoc *doc, gint index)
{
  g_return_if_fail(doc != NULL);

  doc->index = index;
}


void  
rubrica_doc_set_list_path(RubricaDoc *doc, gchar *path)
{
  g_return_if_fail(doc != NULL);

  if (doc->list_path != NULL)
    g_free(doc->list_path);
  
  doc->list_path = g_strdup(path);
}

void 
rubrica_doc_set_father(RubricaDoc *doc, RubricaFolder* father)
{
  g_return_if_fail(doc != NULL);

  doc->father = father;
}

void 
rubrica_doc_set_view(RubricaDoc *doc, GtkTreeView* treeview)
{
  g_return_if_fail(doc != NULL);

  doc->view = treeview;
}

void 
rubrica_doc_set_child(RubricaDoc *doc, GtkWidget* child)
{
  g_return_if_fail(doc != NULL);

  doc->child = child;
}


void 
rubrica_doc_set_doc_tab(RubricaDoc *doc, RubricaDocTab* tab)
{
  g_return_if_fail(doc != NULL);

  doc->tab = tab;
}


gpointer 
rubrica_doc_item_new(RubricaDocType type)
{
  switch (type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      return rubrica_card_item_new();
      break;

    case ACCOUNT_DOC_TYPE:
      return NULL;
      break;

    default:
      return NULL;
      break;
    }
}


void 
rubrica_doc_append_item(RubricaDoc *doc, gpointer item)
{
  g_return_if_fail(doc != NULL);

  doc->docs = g_list_append(doc->docs, item);
  
  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_empty(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);

  rubrica_doc_inc_items(doc);
  rubrica_data_view_set_items(doc->items);  

  switch (doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      rubrica_card_append_item(doc, item);  
      break;
      
    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break;
    }
}


void
rubrica_doc_delete_item (RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);

  rubrica_doc_dec_items(doc); 
  rubrica_data_view_set_items(doc->items);

  if (rubrica_doc_get_items(doc) == 0)
    rubrica_doc_set_empty(doc, TRUE);
  else
    rubrica_doc_set_empty(doc, FALSE);

  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_modified(doc, TRUE);
}

void
rubrica_doc_replace_item (RubricaDoc *doc, gpointer old, gpointer new)
{
  GList* node;

  node = g_list_find(doc->docs, old);
  node->data = new;  
}

gboolean 
rubrica_doc_modify_item (RubricaDoc *doc, gpointer item)
{
  gint pos;
  
  g_return_val_if_fail(doc  != NULL, FALSE);
  g_return_val_if_fail(item != NULL, FALSE);
    
  switch(doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:      
      if ((pos = ((RubricaCardItem*)item)->position) != -1)
	{
	  RubricaCardItem* new;

	  new = rubrica_form_card_modify_item(doc, item);
	  
	  if (new)
	    {
	      rubrica_doc_replace_item(doc, item, new);
	      rubrica_card_modify_item(doc, new, doc->index);

	      rubrica_card_item_free(item);		  
	      
	      rubrica_doc_set_new(doc, FALSE);
	      rubrica_doc_set_modified(doc, TRUE);
	      rubrica_doc_set_empty(doc, FALSE);
	      
	      return TRUE;
	    }	
	}
      break;
      
    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break;
    }
  
  return FALSE;
}



GList* 
rubrica_doc_get_doc_list(RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, NULL);

  return doc->docs;
}


xmlDocPtr      
rubrica_doc_get_xml_tree (RubricaDoc* doc)
{
  g_return_val_if_fail(doc != NULL, NULL);

  if (!doc->xmldoc)
    rubrica_doc_build_xml_tree(doc);

  return doc->xmldoc;
}


gpointer
rubrica_doc_get_item(RubricaDoc *doc, gulong id)
{
  GList* alias;
  
  switch(doc->type)
    {
      case ADDRESSBOOK_DOC_TYPE:
	alias = doc->docs;
	for (; alias; alias = g_list_next(alias))
	  {
	    /* NOTA g_list_index indicizza a partire da zero
	       controllare la corrispondenza di indici con 
	       la lista delle schede
	    */
	    RubricaCardItem* item;

	    item = (RubricaCardItem*) alias->data;
	    
	    if(item && (item->id == id))
	      {
		item->position = g_list_index(doc->docs, item);
		
		return item;
	      }
	  }
	break;

    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break;
    }

  return NULL;
}

gpointer 
rubrica_doc_get_nth_item(RubricaDoc *doc, gulong n)
{
  g_return_val_if_fail(doc != NULL, NULL);
  g_return_val_if_fail(n >= 0, NULL);
  g_return_val_if_fail(n < doc->items, NULL);

  return g_list_nth_data(doc->docs, n);
}

gpointer
rubrica_doc_get_last_item(RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, NULL);
  
  return g_list_nth_data(doc->docs, g_list_length(doc->docs));
}


gboolean 
rubrica_doc_delete_selected_item(RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, FALSE);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 
      
      return FALSE;
    }

  switch(doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      if (rubrica_card_remove_selected_item(doc))
	{  
	  rubrica_doc_delete_item(doc);
	  
	  return TRUE;
	}
      break;
      
    case ACCOUNT_DOC_TYPE:
      break;

    default:
      break;
    }

  return FALSE;
}
 

gboolean 
rubrica_doc_delete_last_item(RubricaDoc *doc)
{
  g_return_val_if_fail(doc != NULL, FALSE);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return FALSE;
    }

  switch(doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      if(rubrica_card_remove_last_item(doc))
	{  
	  RubricaCardItem *item;

	  item = rubrica_doc_get_last_item(doc);
	  rubrica_card_item_set_deleted(item, TRUE);
	  
	  return TRUE;
	}      
      break;

    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break;      
    }
  
  return FALSE;
}



gboolean 
rubrica_doc_delete_nth_item(RubricaDoc *doc, gint n)
{
  g_return_val_if_fail(doc != NULL, FALSE);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_message(_("Address book is empty. I can't continue\n"),
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 

      return FALSE;
    }

  switch(doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      if(rubrica_card_remove_nth_item(doc, n))
	{  
	  RubricaCardItem *item;

	  item = rubrica_doc_get_nth_item(doc, n);
	  rubrica_card_item_set_deleted(item, TRUE);
	  
	  return TRUE;
	}
      break;
      
    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break;     
    }
  
  return FALSE;
}


void           
rubrica_doc_build_xml_tree (RubricaDoc* doc)
{
  xmlDocPtr xmldoc;     /* pointer to xml doc */
  GList* itms;
  gchar *fileformat;

  xmldoc = xmlNewDoc("1.0");  
  fileformat = g_strdup_printf("%d", RUBRICA_FILE_FORMAT);

  xmldoc->children = xmlNewDocNode(xmldoc, NULL, "Rubrica", NULL);
  xmlSetProp(xmldoc->children, "version", RUBRICA_VERSION);
  xmlSetProp(xmldoc->children, "fileformat", fileformat); 
  g_free(fileformat);  
  
  if (doc->type == ADDRESSBOOK_DOC_TYPE)
    {
      xmlSetProp(xmldoc->children, "doctype", "AddressBook"); 

      /* browse addressbook and write data in xml tree 
       */
      itms = doc->docs;  
      for (; itms; itms = itms->next)
	{
	  RubricaCardItem *item;
	  
	  item = (RubricaCardItem *) itms->data;
	  
	  if (item)
	    rubrica_doc_build_card_node(xmldoc->children, item);
	}
    }
  else
    {
      xmlSetProp(xmldoc->children, "doctype", "AccountDoc"); 

      /* TODO: build account tree */
    }

  doc->xmldoc = xmldoc;
}

void 
rubrica_doc_build_card_node(xmlNodePtr xmlnode, gpointer data)
{
  RubricaCardItem *item;
  xmlNodePtr cardxml;     /* card node */

  xmlNodePtr groupxml;    /* group node                     */
  xmlNodePtr datixml;     /* personal data node             */
  xmlNodePtr addressxml;  /* addresses node, address father */ 
  xmlNodePtr adsxml;      /* address node, addresses child  */
  xmlNodePtr webxml;      /* web node       */
  xmlNodePtr mailxml;     /* email node     */
  xmlNodePtr telxml;      /* telephone node */
  xmlNodePtr companyxml;  /* company node   */
  xmlNodePtr workxml;     /* work node      */
  xmlNodePtr notexml;     /* note node      */
  xmlNodePtr node;

  RubricaGroupList groups;
  RubricaGroup* group;
  GList* alias;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(data != NULL);

  item = (RubricaCardItem*) data;
  groups = rubrica_card_item_get_groups(item);
  group  = groups->data;

  /* card xmlnode: 
     is a Rubrica's child 
     is the father of groups, address, webaddresses, etc
  */
  cardxml = xmlNewTextChild(xmlnode, NULL, "Card", NULL);

  /* set card property:
     name:        card name
     deletable:   is a deletable card
     deleted:     card was deleted (when deleted, card is only marked as 
                  deleted real deleting is done during addressbook 
                  optimization)
     rank:        card rank
     id:          card id
     created:     time when card was created
     last_change: last time card was changed
  */
  xmlSetProp(cardxml, "name",        item->card      ? item->card : "");
  xmlSetProp(cardxml, "deletable",   item->deletable ? "true" : "false");
  xmlSetProp(cardxml, "deleted",     item->deleted   ? "yes"  : "no");
  xmlSetProp(cardxml, "rank",        g_strdup_printf("%d",  item->rank));
  xmlSetProp(cardxml, "id",          g_strdup_printf("%ld", item->id));  
  xmlSetProp(cardxml, "created",     g_strdup_printf("%ld", item->created));  
  xmlSetProp(cardxml, "last_change", g_strdup_printf("%ld", item->changed));

  /*    Groups
  */
  groupxml = xmlNewTextChild(cardxml, NULL, "Groups", NULL);
  for (; groups; groups = g_list_next(groups))
    if (groups && groups->data)
      xmlNewTextChild(groupxml, NULL, "Group", 
		      rubrica_card_item_get_group_label(groups->data));  
  
  /*    Personal 
   */
  datixml = xmlNewTextChild(cardxml, NULL, "Data", NULL);

  node = xmlNewTextChild(datixml, NULL, "FirstName", item->personal.first);
  xmlSetProp(node, "know_birthday", (item->personal.birth ? "true" : "false"));
  xmlSetProp(node, "birthday", g_strdup_printf("%ld", item->personal.bdate));
  xmlNewTextChild(datixml, NULL, "MiddleName", item->personal.middle);
  xmlNewTextChild(datixml, NULL, "LastName",   item->personal.last);
  xmlNewTextChild(datixml, NULL, "Nickname",   item->personal.nick);
  xmlNewTextChild(datixml, NULL, "Profession", item->personal.prof);
  xmlNewTextChild(datixml, NULL, "NamePrefix", item->personal.prefix);
  xmlNewTextChild(datixml, NULL, "Title",      item->personal.title);

  /*    Addresses 
   */
  addressxml = xmlNewTextChild(cardxml, NULL, "Addresses", NULL);
  alias = item->addresses;
  for (; alias; alias = g_list_next(alias))
    {
      RubricaCardAddress* address;

      if ((address = alias->data))
	{
	  adsxml = xmlNewTextChild(addressxml, NULL, "Address", NULL);
	  xmlSetProp(adsxml, "type", RUBRICA_ADDRESS_TYPE(address->type));

	  node = xmlNewTextChild(adsxml, NULL, "Street", address->street);
	  xmlSetProp(node, "number", address->number);
	  node = xmlNewTextChild(adsxml, NULL, "City",  address->city);
	  xmlSetProp(node, "zip", address->zip);
	  xmlNewTextChild(adsxml, NULL, "Province",     address->province);
	  xmlNewTextChild(adsxml, NULL, "State",        address->state);
	  xmlNewTextChild(adsxml, NULL, "Country",      address->country);
	}
    }

  /*    Net 
   */
  webxml = xmlNewTextChild(cardxml, NULL, "WebAddresses", NULL);
  alias = item->web;
  for (; alias; alias = g_list_next(alias))
    if (alias && alias->data)
      xmlNewTextChild(webxml, NULL, "Web", alias->data);
  
  mailxml = xmlNewTextChild(cardxml, NULL, "EmailAddresses", NULL);
  alias = item->email;
  for (; alias; alias = g_list_next(alias))
    if (alias && alias->data)
      xmlNewTextChild(mailxml, NULL, "Email", alias->data);
  
  /*    Telephone 
   */
  telxml = xmlNewTextChild(cardxml, NULL, "TelephoneNumbers", NULL);
  alias = item->phone;
  for (; alias; alias = g_list_next(alias))
    if (alias && alias->data)
      {
	RubricaCardTelNum *tn = (RubricaCardTelNum*) alias->data;
	
	node = xmlNewTextChild(telxml, NULL, "Telephone", tn->number);
	xmlSetProp(node, "type", tn->type);		  
      }

  /*    Company 
  */
  companyxml = xmlNewTextChild(cardxml, NULL, "Company", NULL);

  xmlNewTextChild(companyxml, NULL, "CompanyName",   item->company.name);
  node = xmlNewTextChild(companyxml, NULL, "Street", item->company.street);
  xmlSetProp(node, "number", item->company.number);
  node = xmlNewTextChild(companyxml, NULL, "City",   item->company.city);
  xmlSetProp(node, "zip", item->company.zip);
  xmlNewTextChild(companyxml, NULL, "Province",      item->company.province);
  xmlNewTextChild(companyxml, NULL, "Country",       item->company.country);
  xmlNewTextChild(companyxml, NULL, "Web",           item->company.web);
  xmlNewTextChild(companyxml, NULL, "Email",         item->company.email);
  xmlNewTextChild(companyxml, NULL, "Operator",      item->company.operator);
  xmlNewTextChild(companyxml, NULL, "Fax",           item->company.fax);
  xmlNewTextChild(companyxml, NULL, "Green",         item->company.green);
  xmlNewTextChild(companyxml, NULL, "CustomerCare",  item->company.ccare);
  xmlNewTextChild(companyxml, NULL, "Notes",         item->company.notes);
  
  /*    Work 
   */
  workxml = xmlNewTextChild(cardxml, NULL, "Work", NULL);

  xmlNewTextChild(workxml, NULL, "Assigment",      item->work.assigment);
  xmlNewTextChild(workxml, NULL, "Organization",   item->work.org);
  xmlNewTextChild(workxml, NULL, "Department",     item->work.dep);
  xmlNewTextChild(workxml, NULL, "SubDepartment",  item->work.subdep);
  xmlNewTextChild(workxml, NULL, "SecretaryName",  item->work.secretary);
  xmlNewTextChild(workxml, NULL, "SecretaryPhone", item->work.telephone);


  /*    Notes 
  */
  notexml = xmlNewTextChild(cardxml, NULL, "Notes", NULL);

  xmlSetProp(notexml, "married",       item->notes.married ? "true" : "false");
  xmlSetProp(notexml, "know_birthday", item->notes.birth   ? "true" : "false");
  xmlSetProp(notexml, "know_anniversary", item->notes.ann  ? "true" : "false");

  node = xmlNewTextChild(notexml, NULL, "SpouseName", item->notes.spouse);
  xmlSetProp(node, "birthday",    RUBRICA_REAL_DATE(item->notes.bdate));
  xmlSetProp(node, "anniversary", RUBRICA_REAL_DATE(item->notes.adate));

  xmlNewTextChild(notexml, NULL, "Child",      item->notes.children);
  xmlNewTextChild(notexml, NULL, "OtherNotes", item->notes.notes);
  xmlNewTextChild(notexml, NULL, "PublicKey",  item->notes.pubkey);  
}


void 
rubrica_doc_copy_item(RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_error(_("Address book is empty. I can't continue"));
      
      return;
    }

  switch (doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      buffer = rubrica_card_item_copy(doc);
      break;
      
    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break; 
    }
}

void 
rubrica_doc_cut_item(RubricaDoc *doc)
{
  g_return_if_fail(doc != NULL);

  if (rubrica_doc_is_empty(doc))
    {
      rubrica_dialog_error(_("Address book is empty. I can't continue"));
      
      return;
    }

  switch (doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      if (buffer) 
	{ 
	  rubrica_card_item_free(buffer);
	  buffer = NULL;
	}
      
      buffer = rubrica_card_item_cut(doc);
      break;
      
    case ACCOUNT_DOC_TYPE:
      break;
      
    default:
      break; 
    }  
}


void 
rubrica_doc_paste_item(RubricaDoc *doc)
{
  gint index;

  g_return_if_fail(doc != NULL);

  if (!buffer)
    {
      rubrica_dialog_message(_("I can't paste anything if you "
			       "haven't previously cut or copied"), 
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);

      return;
    }
  
  rubrica_card_item_set_father(buffer, doc);
  index = rubrica_doc_get_index(doc);
  if (index  == -1)
    rubrica_card_append_item(doc, buffer);
  else
    rubrica_card_insert_item(doc, buffer, index);

  buffer = NULL;
}


void 
rubrica_doc_free_memory(RubricaDoc *doc)
{
  GtkTreeModel *model;

  g_return_if_fail(doc != NULL);

  switch (doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      model = gtk_tree_view_get_model(GTK_TREE_VIEW
				      (rubrica_doc_get_view(doc)));
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), 
			     rubrica_card_free_memory, doc);
      break;

    case ACCOUNT_DOC_TYPE:      
      break;

    default:
      break;
    }
}

void 
rubrica_doc_free_string(gchar *str)
{
  if (str) g_free(str);
}


/*  utility functions
*/
gboolean rubrica_doc_others_docs_are_modified(void)
{  
  RubricaDoc *doc;
  gint cards, n;
  gboolean bool = FALSE;

  cards = g_list_length(rubrica_folder_get_documents(NULL));
  
  for (n = 0; n < cards; n++)
    {
      doc = rubrica_folder_get_nth_document(NULL, n);
      
      bool |=  rubrica_doc_is_modified(doc);   
    }


/*
  cards = rubrica_card_get_docs();

  for (n = 0; n < cards; n++)
    {
      doc = rubrica_get_current_notebook_nth_document(n);

      bool |=  rubrica_doc_is_modified_doc(doc);   
    }
*/

  return bool;  
}


void 
rubrica_doc_close(RubricaDoc *doc)
{    
  GtkWidget *dialog;
  gint scelta;
  gchar *mes;

  g_return_if_fail(doc != NULL);

  if (rubrica_doc_is_modified(doc))
    {
      mes = g_strdup_printf(_("Warning. %s\n"
			      "Changes not saved will be lost.\n\n"
			      "Do you want continue?"), 
			    _("Addressbook has changed."));
      
      dialog = rubrica_dialog_query(mes, GTK_BUTTONS_YES_NO);
      gtk_widget_show(dialog);
      g_free(mes);

      scelta = gtk_dialog_run(GTK_DIALOG(dialog));
      
      switch(scelta)
	{
	case GTK_RESPONSE_YES:
	  gtk_widget_destroy(dialog);
	  /* delete addressbook, go down an delete */
	  break;
	  
	case GTK_RESPONSE_NO:
	default:
	  gtk_widget_destroy(dialog);
	  return;
	  break;
	}  
    }
  
  /*      clean view window 
   */
  rubrica_data_view_clean_view();  
  rubrica_folder_remove_doc(doc);
  
  if (rubrica_folder_get_docs_number(NULL) == 0)
    rubrica_card_add_page_view(NULL);  
}
