/*
*  Rubrica
*  file: rubrica.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_H
#define _RUBRICA_H

#include <gnome.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <gconf/gconf-client.h>

#include "rubrica_type.h"


typedef enum {
  ADDRESSBOOK_FOLDER = 0,
  ACCOUNT_FOLDER,
  NOTES_FOLDER,
  UNKNOWN_FOLDER
} RubricaFolderType;


/*  fields attribute: r ==> read  (available get function) 
                      w ==> write (available get/set functions)
*/  

typedef struct _RubricaFolder RubricaFolder;
typedef struct _RubricaDoc    RubricaDoc;
typedef struct _RubricaDocTab RubricaDocTab;


/*   RubricaFolder struct and functions
*/

struct _RubricaFolder
{
  RubricaFolderType type;       /* documents folder type  */  

  GList* docs;                  /* documents in folder (RubricaDoc list) */
  RubricaDoc* current;          /* current document              */
  gint documents;               /* number of documents in folder */

  GtkWidget* container;         /* folder container (hbox)       */
  GtkWidget* label;             /* folder title     (label)      */
  GtkWidget* notebook;          /* the folder                    */
};


RubricaFolder*    rubrica_folder_new                  (RubricaFolderType type);
void              rubrica_folder_free                 (RubricaFolder* folder);

void              rubrica_folder_interface_new        (GtkWidget *box, 
						       GtkWidget *label,
						       RubricaFolderType type);

GList*            rubrica_folder_get_all_folders      (void);
RubricaFolder*    rubrica_folder_get_first_folder     (void);
RubricaFolder*    rubrica_folder_get_last_folder      (void);
RubricaFolder*    rubrica_folder_get_next_folder      (void);
RubricaFolder*    rubrica_folder_get_previous_folder  (void);
RubricaFolder*    rubrica_folder_get_folder           (RubricaFolderType type);

void              rubrica_folder_init_active_folder   (RubricaFolderType type);
void              rubrica_folder_set_active_folder    (RubricaFolderType type);

RubricaFolderType rubrica_folder_get_type             (RubricaFolder* folder);

GList*            rubrica_folder_get_documents        (RubricaFolder* folder);
gint              rubrica_folder_get_docs_number      (RubricaFolder* folder);

gint              rubrica_folder_get_page             (RubricaFolder* folder);
RubricaDoc*       rubrica_folder_get_current_document (void);
RubricaDoc*       rubrica_folder_get_nth_document     (RubricaFolder* folder,
						       gint n);

void              rubrica_folder_set_doc_as_current   (RubricaFolder* folder, 
						       RubricaDoc* doc);

void              rubrica_folder_append_doc           (RubricaFolder* folder,
						       RubricaDoc* doc, 
						       GtkWidget *child, 
						       GtkWidget *tabbox);
void              rubrica_folder_remove_doc           (RubricaDoc* doc);




/*  RubricaDocTab struct and functions
*/

struct _RubricaDocTab
{
  /* public */
  GtkWidget* label;     /* label (file name) on notebook's tab */

  /* private */
  GtkWidget* box;       /* widgets container           */
  GtkTooltips* ttips;   /* tooltips for close button   */
  GtkWidget* green;     /* green ligth, unchanged file */
  GtkWidget* red;       /* red ligth, file has changed */
  GtkWidget* close;     /* close button                */
};

GtkWidget*        rubrica_doc_tab_new          (RubricaDoc* doc, gchar* name);
void              rubrica_doc_tab_set_label    (RubricaDoc* doc, gchar *label);
void              rubrica_doc_tab_enable_green (RubricaDoc* doc);
void              rubrica_doc_tab_enable_red   (RubricaDoc* doc);



/*  RubrucaDoc struct and functions
*/

struct _RubricaDoc
{
  gulong id;               /* r first available id */
  RubricaDocType type;     /* r document type      */

  GList* docs;             /* r/w cards/accounts/... list */
  xmlDocPtr xmldoc;        /* r/w xml doc */
  
  gboolean new;            /* r/w true if this is a new addressbook  */
  gboolean modified;       /* r/w true if rubrica is modified */
  gboolean empty;          /* r/w true if rub list is empty   */

  gchar *name;             /* r/w addressbook's name */
  gchar *path;             /* r/w addressbook's path */
  gchar *tmpfile;          /* r/w temp file's name, used in autosave */

  gint   items;            /* r/w number of item in current document */
  gint   index;            /* r/w number of the item currently show  */
  gchar* list_path;        /* r/w currently shown item's path (GtkTreePath) */

  RubricaFolder* father;   /* r    folder that own this document */
  GtkTreeView*   view;     /* r    the tree_view      */
  GtkWidget*     child;    /* r    notebook page      */
  gint           page;     /* r    number of the page */
  RubricaDocTab* tab;      /* r/w  notebook tab       */
};


/*    RubricaDoc functions
 */

/*    Get
 */
RubricaDoc*    rubrica_doc_new             (RubricaDocType doctype);
gboolean       rubrica_is_valid_doc        (RubricaDoc* doc);

gulong         rubrica_doc_get_id          (RubricaDoc* doc);
RubricaDocType rubrica_doc_get_type        (RubricaDoc* doc);

GList*         rubrica_doc_get_doc_list    (RubricaDoc* doc);
xmlDocPtr      rubrica_doc_get_xml_tree    (RubricaDoc* doc);

gboolean       rubrica_doc_is_new          (RubricaDoc* doc);
gboolean       rubrica_doc_is_empty        (RubricaDoc* doc);
gboolean       rubrica_doc_is_modified     (RubricaDoc* doc);

gchar*         rubrica_doc_get_path        (RubricaDoc* doc);
gchar*         rubrica_doc_get_name        (RubricaDoc* doc);
gchar*         rubrica_doc_get_tmpfile     (RubricaDoc* doc);

gint           rubrica_doc_get_items       (RubricaDoc* doc);
gint           rubrica_doc_get_index       (RubricaDoc* doc);
gchar*         rubrica_doc_get_list_path   (RubricaDoc* doc);

RubricaFolder* rubrica_doc_get_father      (RubricaDoc* doc);
GtkWidget*     rubrica_doc_get_view        (RubricaDoc* doc);     

void           rubrica_doc_reject_id       (RubricaDoc* doc);

/*  Set
*/
void           rubrica_doc_set_new         (RubricaDoc* doc, gboolean state);
void           rubrica_doc_set_empty       (RubricaDoc* doc, gboolean state);
void           rubrica_doc_set_modified    (RubricaDoc* doc, gboolean state);

void           rubrica_doc_set_path        (RubricaDoc* doc, gchar *str);
void           rubrica_doc_set_name        (RubricaDoc* doc, gchar *str);
void           rubrica_doc_set_tmpfile     (RubricaDoc* doc, gchar *str);

void           rubrica_doc_set_items       (RubricaDoc* doc, gint n);
void           rubrica_doc_inc_items       (RubricaDoc* doc);
void           rubrica_doc_dec_items       (RubricaDoc* doc);

void           rubrica_doc_set_index       (RubricaDoc* doc, gint index);
void           rubrica_doc_set_list_path   (RubricaDoc* doc, gchar *path);

void           rubrica_doc_set_father      (RubricaDoc* doc, 
					    RubricaFolder* father);
void           rubrica_doc_set_child       (RubricaDoc* doc, GtkWidget* child);
void           rubrica_doc_set_view        (RubricaDoc* doc, 
					    GtkTreeView* treeview);
void           rubrica_doc_set_doc_tab     (RubricaDoc* doc, 
					    RubricaDocTab* tab);

void           rubrica_doc_build_xml_tree  (RubricaDoc* doc);
void           rubrica_doc_build_card_node (xmlNodePtr xmlnode, gpointer item);



/*  Items manage
*/
gpointer       rubrica_doc_item_new             (RubricaDocType type);

void           rubrica_doc_append_item          (RubricaDoc* doc,
						 gpointer item);

void           rubrica_doc_delete_item          (RubricaDoc* doc);
void           rubrica_doc_replace_item         (RubricaDoc* doc, 
						 gpointer old,
						 gpointer new);
gboolean       rubrica_doc_modify_item          (RubricaDoc* doc, 
						 gpointer item);

GList*         rubrica_doc_get_doc_list         (RubricaDoc* doc);
gpointer       rubrica_doc_get_item             (RubricaDoc* doc, gulong id);
gpointer       rubrica_doc_get_nth_item         (RubricaDoc* doc, gulong n);
gpointer       rubrica_doc_get_last_item        (RubricaDoc* doc);

gboolean       rubrica_doc_delete_selected_item (RubricaDoc* doc);
gboolean       rubrica_doc_delete_last_item     (RubricaDoc* doc);
gboolean       rubrica_doc_delete_nth_item      (RubricaDoc* doc, gint n);

void           rubrica_doc_copy_item            (RubricaDoc* doc);
void           rubrica_doc_cut_item             (RubricaDoc* doc);
void           rubrica_doc_paste_item           (RubricaDoc* doc);


/*  Free memory
*/
void           rubrica_doc_free_memory (RubricaDoc* doc);
void           rubrica_doc_free_string (gchar *str);


/* Closing functions
 */
gboolean       rubrica_doc_others_docs_are_modified (void);
void           rubrica_doc_close                    (RubricaDoc* doc);


#endif
