/*
*  Rubrica
*  file: view.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <config.h>
#include <gnome.h>

#include "rubrica.h"
#include "view.h"
#include "ui.h"
#include "card.h"
#include "prefer.h"
#include "prefer_configure.h"



/*
  rubrica_set_column_attributes

  spostare in un file appropriato per le funzioni
  generiche di supporto alla gestione delle liste
*/
void rubrica_view_set_column_attributes(GConfClient *client, 
					GtkTreeView *view, 
					gchar *key, gint col)
{
  GtkTreeViewColumn *column;
  
  column = gtk_tree_view_get_column(view, col);
//  gtk_tree_view_column_set_resizable(column, TRUE);
//  gtk_tree_view_column_set_clickable(column, TRUE);
  gtk_tree_view_column_set_reorderable(column, TRUE);

  rubrica_preferences_column_config(client, column, key);
}


GtkWidget *rubrica_view_new_view(RubricaDoc *doc, GtkTreeModel *model)
{
  GtkWidget *scrollwin;

  GtkWidget *view;
  GtkTreeSelection *select;
  
  scrollwin = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), 
				      GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);

  /*     create a new view using the model and connect the select
	 widget to changed signal
  */
  view   = gtk_tree_view_new_with_model(model);
  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
  
  rubrica_doc_set_view(doc, GTK_TREE_VIEW(view));


  g_object_unref(G_OBJECT(model));

  /*    set view's properties
  */
  gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (view), TRUE);
  gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW (view), TRUE);

  switch(doc->type)
    {
    case ADDRESSBOOK_DOC_TYPE:
      rubrica_card_add_columns(GTK_TREE_VIEW(view));

      g_signal_connect(G_OBJECT(select), "changed", 
		       G_CALLBACK(rubrica_card_row_selected), view);
      g_signal_connect(G_OBJECT(view), "row_activated", 
		       G_CALLBACK(rubrica_card_row_activated), view);
      break;

/*      
    case ACCOUNT_DOC_TYPE:
      rubrica_account_add_columns(GTK_TREE_VIEW(view));

      g_signal_connect(G_OBJECT(select), "changed", 
		       G_CALLBACK(rubrica_account_row_selected), view);
      g_signal_connect(G_OBJECT(view), "row_activated", 
		       G_CALLBACK(rubrica_account_row_activated), view); 
      break;
*/

    default:
      break;
    }
  
  gtk_container_add(GTK_CONTAINER(scrollwin), view);
  gtk_widget_show(view);
  gtk_widget_show(scrollwin);

  return scrollwin;
}


